package rogatkin.xbox.applet;
// system packages
import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

// our packages
import rogatkin.xbox.ui.*;
import rogatkin.xbox.shared.*;
import rogatkin.xbox.core.*;

public class AwtToolBox extends AwtFreePanel implements ItemSelectedListener, IToolBox {
	
    private final static int rowHeight = 20;
    private final static int numCols = 3;
    
    private static boolean debug = false;
    private static String jarsDirectory = null;

    private static Cursor crosshairCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
    private static Cursor defaultCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);

	private CompManager ctrlManager;
	private int topPad = 0;
        private int sidePad = 0;
        private Object pendingBean;
        private String pendingBeanLabel;
	private volatile int beanIndex;
	private boolean beanSelected = false;
	private WorkSpace beancontainer;

	AwtBtnPalette toolBoxMenu = new AwtBtnPalette(numCols);
    
	public AwtToolBox(WorkSpace bc) {
		beancontainer = bc;
		ctrlManager = beancontainer.getCompManager();

		setLayout(new BorderLayout());
		setBackground(SystemColor.control);
		setFont(new Font("Dialog", Font.PLAIN, 10));

		try {
			jbInit();
		}
		catch (Exception e) {
			e.printStackTrace();
		}

		// Insets doesn't provide correct infromation while it is hidden
	}

	void jbInit() throws Exception{
    
		toolBoxMenu.addItemSelectedListener(this);

		add(toolBoxMenu, "Center");

		if(ctrlManager != null) {
			toolBoxMenu.setMenuItems(ctrlManager.getControlNames()/*beanLabels*/);
			toolBoxMenu.setMenuImages(ctrlManager.getControlImages()/*beanIcons*/);
			toolBoxMenu.init();
		}
		else
			System.out.println("ToolBox.jbInit: ctrlManager is not initialized.");
    }
	
	public String getProfileString(String keyname){
		// Don't hard code jars directory.
		// jarsDirectory is set by an argument of jview.
		return new String(jarsDirectory);
	}

    //----------------------------------------------------------------------

	public boolean isBeanSelected()
	{
		return beanSelected;
	}
    
	public void setBeanSelected(boolean on)
	{
		beanSelected = on;
		if (isBeanSelected()) {
			toolBoxMenu.setSelection(getCurrentBeanIndex());
			setCursor(crosshairCursor);
			beancontainer.setCursor(crosshairCursor);
		}
		else {
			toolBoxMenu.setSelection(-1);
			setCursor(defaultCursor);
			beancontainer.setCursor(defaultCursor);
		}
		//ToolBoxMenu.repaint();
	}
	
	public int getCurrentBeanIndex()
	{
		return beanIndex;
	}

	public void setCurrentBeanIndex(int r)
	{
		beanIndex = r;
	}
	
	// Item Selected listener methods for this panel.

    public void selected (ItemSelectedEvent e) {
		setCurrentBeanIndex(e.getItemSelected());
		setBeanSelected(true);
		
		synchronized (this) {
			notifyAll();
		}
    }

	//----------------------------------------------------------------------

    public void error(String message, Exception ex) {
		String mess = message + ":\n" + ex;
		System.err.println(message);
		//ex.printStackTrace();
		// Popup an ErrorDialog with the given error message.
		new ErrorDialog(null, mess);
    }

    public void error(String message) {
		String mess = message;
		System.err.println(message);
		// Popup an ErrorDialog with the given error message.
		new ErrorDialog(null, mess);
	}

    static  void debug(String msg) {
		if (debug) {
			System.err.println("FB: "+msg);
		}
    }
}

