/*
 * File:		BaseChoice.java
 *
 * Description:	BaseChoice : implements basic functionality for radio button group
 * and combobox components
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */
package rogatkin.xbox.components;

import java.util.Vector;

public class BaseChoice {
	private String[] elements;
	private String[] tooltips;
	private int selection;
	public BaseChoice(){
		elements = new String[0];
		selection = -1;
	}
    
    public boolean validateSelection() {
        return (selection > -1 && selection < elements.length);
    }

	public void setChoices(int size) {
        if (elements.length != size)
        {
            String []result = new String[size];
            if (elements.length > 0)
                System.arraycopy(elements, 0, result, 0, (elements.length>size)?size:elements.length);
            elements = result;
        }
	}
	
	public int getChoices() {
		return elements.length;
	}
	
	public void setSelection(int selection) {
	    this.selection = selection;
	}
	
	public int getSelection() {
		return selection;
	}
	
	public void setText(String text) {
        if (validateSelection())
		    elements[getSelection()] = text;
	}
	
	public String getText(){
        if (validateSelection())
            return getTexts(selection);
        return new String("");
	}
	
	public void setTexts(String[] texts){
        elements = texts;
	}
	
	public String[] getTexts() {
		return elements;
	}
    
	public String getTexts(int index){
        return ((elements[index]==null)?"":elements[index]);
    }
    
	public void setTexts(int index, String text){
        elements[index] = text;
    }
}