/*
 * File:	DFDataBaseConnection.java
 * Author:	Dmitry Rogatkin
 * Date:	03/2/98
 *
 * Description:	DFDataBaseConnection 
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */
package rogatkin.xbox.components;

import java.net.URL;
import java.sql.*;
import java.util.Properties;
import java.util.Vector;
import java.util.Enumeration;

public class DFDataBaseConnection
{
    public String ms_dataSource;
    public String ms_userName;
    public String ms_password;
	private String ms_protocol = "odbc:";
    public Connection mh_connection;
	private Properties mp_properties = new Properties();
	private Driver mc_driver;
    
    public DFDataBaseConnection(String className, String protocol)
    {
		if (protocol != null && protocol != "")
			ms_protocol = protocol;
		// init bridge jdbc-odbc
		try {
			// Load the jdbc-odbc bridge driver
			if (className == null || className == "")
				className = "com.ms.jdbc.odbc.JdbcOdbcDriver";
			mc_driver = (Driver)Class.forName (className).newInstance();
		}
		catch (java.lang.Exception ex) {
			// Got some other type of exception.  Dump it.
			System.err.println("Driver "+className+" hasn't been loaded ("+ex+")");
		}
	}
	
	public DFDataBaseConnection(){
		this(null, null);
	}
	
    boolean getConnected()
    {
		try {
			if (mp_properties != null && !mp_properties.isEmpty())
				mh_connection = mc_driver.connect ("jdbc:"+
				ms_protocol, mp_properties);
			else
				mh_connection = DriverManager.getConnection ("jdbc:"+
				ms_protocol+ms_dataSource,
				ms_userName, ms_password);
			checkForWarning (mh_connection.getWarnings ());
			
			// Get the DatabaseMetaData object and display
			// some information about the connection
			
			DatabaseMetaData dma = mh_connection.getMetaData ();
			
			System.err.println("\nConnected to " + dma.getURL());
			System.err.println("Driver       " + 
				dma.getDriverName());
			System.err.println("Version      " +
				dma.getDriverVersion());
			System.err.println("===DRIVER CAPABILITIES===");
			System.err.println("-"+(dma.supportsPositionedUpdate()?" ":" doesn't ")+"support positioned update");
			System.err.println("-"+(dma.supportsPositionedDelete()?" ":" doesn't ")+"support positioned delete");
			System.err.println("-"+(dma.supportsANSI92EntryLevelSQL()?" ":" doesn't ")+"support ANSI92 entry level");
			System.err.println("-"+(dma.supportsANSI92FullSQL()?" ":" doesn't ")+"support full ANSI92");
		}
		catch (SQLException ex) {
			DisplaySQLError(ex);
			return false;
		}
		
		return true;
	}
	
	boolean getConnected(String dataSource, String userName, String password, Properties properties)
	{
		ms_dataSource = dataSource;
		ms_userName = userName;
		ms_password = password;
		mp_properties = properties;
		return getConnected();
	}
	
	public static void DisplaySQLError(SQLException ex){
		
		// A SQLException was generated.  Catch it and
		// display the error information.  Note that there
		// could be multiple error objects chained
		// together
		
		System.out.println ("\n*** SQLException caught ***\n");
		
		while (ex != null) {
			System.out.println ("SQLState: " +
				ex.getSQLState ());
			System.out.println ("Message:  " +
				ex.getMessage ());
			System.out.println ("Vendor:   " +
				ex.getErrorCode ());
			ex = ex.getNextException ();
			System.out.println ("");
		}
    }
	private static boolean checkForWarning (SQLWarning warn)
		throws SQLException
	{
		boolean rc = false;
		
		// If a SQLWarning object was given, display the
		// warning messages.  Note that there could be
		// multiple warnings chained together
		
		if (warn != null) {
			System.out.println ("\n *** Warning ***\n");
			rc = true;
			while (warn != null) {
				System.out.println ("SQLState: " +
					warn.getSQLState ());
				System.out.println ("Message:  " +
					warn.getMessage ());
				System.out.println ("Vendor:   " +
					warn.getErrorCode ());
				System.out.println ("");
				warn = warn.getNextWarning ();
			}
		}
		return rc;
	}

    public void close()
    {
        try {
            if (mh_connection != null)
                mh_connection.close();
			//deregisterDriver();
        }
		catch (SQLException ex) {
			DisplaySQLError(ex);
		}
    }
	
    public Statement createStatement() 
    {
        try {
            return mh_connection.createStatement();
        }
		catch (SQLException ex) {
			DisplaySQLError(ex);
			return null;
		}
    }

	public String[] getTableNames(){
		
		java.util.Vector TableNames=new java.util.Vector(3);
		if(mh_connection!=null)
			try{
			DatabaseMetaData dbmd=mh_connection.getMetaData();
			ResultSet rset=dbmd.getTables(null,null,null,new String[] {"TABLE", "VIEW"});
			while(rset.next())
				TableNames.addElement(rset.getString(3));
			String[] Tables=new String[TableNames.size()];
			TableNames.copyInto(Tables);
			return Tables;
		}
		catch(SQLException e){
			System.out.println("Error "+e+" in getting TableNames.");
		}
		
		return null;
	}

	// properties
	public Properties getProperties(){
		return mp_properties;
	}
	
	public String [] getPropertiesNames(){
		DriverPropertyInfo [] propsinfo = null;
		try {
			mc_driver = DriverManager.getDriver("jdbc:"+ms_protocol);
			propsinfo = 
				mc_driver.getPropertyInfo("jdbc:"+ms_protocol, getProperties());
			
		}catch (SQLException ex) {
			DisplaySQLError(ex);
			return null;
		}
		
		String [] result = null;
		if (propsinfo != null && propsinfo.length > 0){
			result = new String[propsinfo.length];
			for (int i = 0; i < propsinfo.length; i++)
				result[i] = propsinfo[i].name;
		}

		return result;
	}

}

