/*
 * File:	DFDataBaseTable.java
 * Author:	Dmitry Rogatkin
 * Date:	03/2/98
 *
 * Description:	DFDataBaseTable 
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */
package rogatkin.xbox.components;

import java.net.URL;
import java.sql.*;

public class DFDataBaseTable
{
    public String ms_tableName;
    public String ms_query;
    public String ms_tableDescription;
    public ResultSet mc_resultSet;
	
    private Statement mc_statement;
	private PreparedStatement mc_preparedstmt;
    private DFDataBaseConnection mc_dataBase;
    
    public DFDataBaseTable(DFDataBaseConnection dataBase, String tableName)throws SQLException
    {
        mc_dataBase = dataBase;
        mc_statement = mc_dataBase.createStatement ();
		mc_preparedstmt = null;
        ms_tableName = tableName;
    }
	
    public void setTableDescription(String tableDescription)
    {
        ms_tableDescription = tableDescription;
    }
	
	public ResultSetMetaData getSetMetaData(){
		try {
		 return mc_resultSet.getMetaData ();
		} catch (SQLException se){
			System.out.println("SQL error in getting SetMetaData "+se);
		}
		return null;
	}

    boolean updateQuery(String queryStr)
    {
        try {
			mc_statement.executeUpdate (queryStr);
        }
		catch (SQLException ex) {
            System.out.println(queryStr);
			DFDataBaseConnection.DisplaySQLError(ex);
			return false;
		}
		return true;
    }
    
    boolean createTable()
    {
        String ls_SQLStmt = "CREATE TABLE "+
            ms_tableName +" "+
            ms_tableDescription;
        try {
            try {
                mc_statement.executeUpdate("DROP TABLE "+ms_tableName);
            } catch (Exception e) {
                // do nothing
            }
            mc_statement.executeUpdate(ls_SQLStmt);
        }
		catch (SQLException ex) {
            System.out.println(ls_SQLStmt);
			DFDataBaseConnection.DisplaySQLError(ex);
			return false;
		}
        return true;
    }
    
    boolean insertRec(String recSpec)
    {
        String ls_SQLStmt = "INSERT INTO "+
            ms_tableName +" "+
            recSpec;
        return updateQuery(ls_SQLStmt);
    }
    
    boolean updateRec(String val_and_condSpec)
    {
        String ls_SQLStmt = "UPDATE "+
            ms_tableName +" SET "+
            val_and_condSpec;
        return updateQuery(ls_SQLStmt);
    }
    
	boolean deleteRec(String condSpec)
    {
        String ls_SQLStmt = "DELETE FROM "+
            ms_tableName +" WHERE "+condSpec;
        return updateQuery(ls_SQLStmt);
    }
    
    int generateNewKey(String keyFieldName)
    {
        int vnID;
        String lsQuery = "SELECT DISTINCT "+
            keyFieldName+" FROM "+ ms_tableName+
            " WHERE "+keyFieldName+" < 2147483647 ORDER BY 1 DESC";
        try {
            int oldMaxRows = mc_statement.getMaxRows();
            mc_statement.setMaxRows(1);
            mc_resultSet = mc_statement.executeQuery (lsQuery);
            if (mc_resultSet.next())
                vnID = mc_resultSet.getInt(keyFieldName) + 1;
            else
                vnID = 0;
            mc_statement.setMaxRows(oldMaxRows);
            return vnID;
        }
		catch (SQLException ex) {
            System.out.println(lsQuery);
			DFDataBaseConnection.DisplaySQLError(ex);
		}
        return 0;
    }
    
    boolean Requery(String query)
    {
        ms_query = query;
        return Requery();
    }
	
    boolean Requery()
    {
        try {
			// Submit a query, creating a ResultSet object
			if (mc_resultSet != null)
				mc_resultSet.close();
			
			mc_resultSet = mc_statement.executeQuery (ms_query);
        }
		catch (SQLException ex) {
            System.out.println(ms_query);
			DFDataBaseConnection.DisplaySQLError(ex);
			return false;
		}
		return true;
    }
	
    void close()
    {
        try {
			if (mc_resultSet != null)
				mc_resultSet.close();
            if (mc_statement != null)
                mc_statement.close();
        }
        catch (SQLException ex) {
			DFDataBaseConnection.DisplaySQLError(ex);
		}
    }
	
    Statement selectStatement(Statement newStmt)
    {
        Statement retStmt = mc_statement;
        mc_statement = newStmt;
        return retStmt;
    }

	void setCursorName(String name)
	{
		try {
			mc_statement.setCursorName(name);
		} catch (SQLException es){
			System.err.println("dB Tbl: "+es);
		}
	}
	
	PreparedStatement getPreparedStatement(String query){
		try {
			mc_preparedstmt = mc_dataBase.mh_connection.prepareStatement(query);
		}catch (SQLException es){
			System.err.println("dB Prep: "+es);
			return null;
		}catch (Exception e){
			return null;
		}
		return mc_preparedstmt;
	}
	
	int updatePreparedStatement(){
		if (mc_preparedstmt == null)
			return 0;
		try {
			return mc_preparedstmt.executeUpdate();
		}catch (SQLException es){
			System.err.println("dB ExUpdt: "+es);
			return 0;
		}
	}
}

