/*
 * File:	DataBaseControl.java
 * Author:	Dmitry Rogatkin
 * Date:	03/2/98
 *
 * Description:	DataBaseControl
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.components;

import java.awt.Dimension;
import java.awt.Color;
import java.util.Properties;
import com.ms.ui.*;

import java.sql.*;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

import rogatkin.xbox.shared.IBindingList;
import rogatkin.xbox.shared.IFCEvListener;
import rogatkin.xbox.shared.FCEvent;
import rogatkin.xbox.shared.Constants;

public class DataBaseControl extends UIScrollViewer implements IBindingList, IFCEvListener {
	private final static String CTRLID = "DBControl:";
	private String UserName = "user";
	private String Password = "";
	private String DSNOrProperty = "Type DSN here";  // Just for the demo!
	private String TableName = "";
	private String[] TableNames = new String[1];
	private Properties ConnectionProperties = null;
	private String PropertyName = "";
	private String[] PropertyNames = new String[1];
	private boolean ShowColumnName;
	private boolean GetConnected;
	private boolean GetProperties;
	private boolean SetProperty;
	private String JDBCDriver = "com.ms.jdbc.odbc.JdbcOdbcDriver";
	private String Subprotocol = "odbc:";
	
	public DataBaseControl() {
                super(new UIEdit());
		setBackground(Color.white);
		GetConnected = false;
		ShowColumnName = false;
		setTableNames(0, "~undefined~");
		setPropertyNames(0, "");
		md = null;
	}
	
	public Dimension getMinimumSize() {
		return new Dimension(80, 80);
	}

	 public String getTooltipString(){
	  return "DataBaseControl";
	}	
	
	public ResultSetMetaData getColumnInfo(){
		try {
			if (tb == null)
				tb = new DFDataBaseTable(db, getTableName());
			tb.Requery(SELALLCOLUMN+getTableName());
			return tb.getSetMetaData();
		} catch (Exception e) {
			System.err.println(CTRLID+" get metadata "+e);
		}
		return null;
	}
	
	public void setShowColumnName(boolean on){
		ShowColumnName = on;
		if (ShowColumnName) {
			// java.sql.Types
			try {
				if (tb == null)
					tb = new DFDataBaseTable(db, getTableName());
				tb.Requery(SELALLCOLUMN+getTableName());
				md = tb.getSetMetaData();
                                StringBuffer sb = new StringBuffer();
				for (int i = 1; i<=md.getColumnCount(); i++)
					sb.append(md.getColumnName(i)).append(' ').append(md.getColumnTypeName(i))
					.append('(').append(md.getPrecision(i)).append(")\n");
                                ((UIEdit)getContent()).setValueText(sb.toString());
			} catch (Exception e) {
				System.err.println(CTRLID+" get column name "+e);
			}
		}
		else
			((UIEdit)getContent()).setValueText("");
	}
	
	public boolean isShowColumnName(){
		return ShowColumnName;
	}
	public String getUserName(){
		return UserName;
	}
	
	public void setUserName(String name){
		UserName = name;
	}
	
	public String getPassword(){
		return Password;
	}
	
	public void setPassword(String name){
		Password = name;
	}

	public String getTableName(){
		return TableName;
	}
	
	public void setTableName(String name){
		if (name == null || name.length() == 0)
			return;
		if (tb != null) {
			tb.close();
			tb = null;
		}
		TableName = name;
	}

	public String getPropertyName(){
		return PropertyName;
	}
	
	public void setPropertyName(String name){
		PropertyName = name;
	}
	
	public String getDSNOrProperty(){
		return DSNOrProperty;
	}
	
	public void setDSNOrProperty(String name){
		DSNOrProperty = name;
	}
	public String getJDBCDriver(){
		return JDBCDriver;
	}
	
	public void setJDBCDriver(String name){
		JDBCDriver = name;
		if (isGetConnected())
		{
			setGetConnected(false);
			setGetConnected(true);
		}
	}
	public String getSubprotocol(){
		return Subprotocol;
	}
	
	public void setSubprotocol(String name){
		Subprotocol = name;
	}

	public boolean isBuiltList(){
		return md != null;
	}

	public void getBuiltList(boolean on){
		if (on){
			String tblname = getTableName();
			if (tblname != null && tblname != "")
				try {
				if (tb == null)
					tb = new DFDataBaseTable(db, tblname);
				tb.Requery(SELALLCOLUMN+tblname);
				tblname = null;
				md = tb.getSetMetaData();
			} catch (Exception e) {
				System.err.println(CTRLID+" getBuiltList "+e);
			}
			else
				md = null;
		}
	}

	public int getElementsCount(){
		try{
			return (md != null)?md.getColumnCount():0;
		}catch (SQLException e){
		}
		return 0;
	}

	public void setElementsCount(int c){
	}

	public String getElementNames(int index){
		try{
			return md.getColumnName(index+1);
		} catch (Exception e) {
			System.err.println(CTRLID+" getElementNames(int) "+e);
		}
		return null;
	}
	
	public String[] getElementNames(){
		String [] result = new String[getElementsCount()];
		try{
			for (int i = 0; i<result.length; i++)
				result[i] = getElementNames(i);
		} catch (Exception e) {
			System.err.println(CTRLID+" []getElementNames() "+e);
		}
		return result;
	}
	
	public void setElementNames(String[] names){
	}
	
	public void setElementNames(int index, String name){
	}
	
	public String getElementInfos(int index){
		try{
			return md.getColumnTypeName(index+1);
		} catch (Exception e) {
			System.err.println(CTRLID+" getElIs "+e);
		}
		return null;
	}

	public String[] getElementInfos(){
		String [] result = new String[getElementsCount()];
		try{
			for (int i = 0; i<result.length; i++)
				result[i] = getElementInfos(i);
		} catch (Exception e) {
			System.err.println(CTRLID+" getElIs[] "+e);
		}
		return result;
	}
	
	public void setElementInfos(String [] names){
	}
	
	public void setElementInfos(int index, String name){
	}
	
	public void setTableNames(int index, String name) throws ArrayIndexOutOfBoundsException {
		TableNames[index] = name;
	}
	
	public String getTableNames(int index) throws ArrayIndexOutOfBoundsException {
		return TableNames[index];
	}
	
	public void setTableNames(String names[]){
		TableNames = names;
	}
	
	public String[] getTableNames() {
		return TableNames;
	}
	public void setPropertyNames(int index, String name) throws ArrayIndexOutOfBoundsException {
		PropertyNames[index] = name;
	}
	
	public String getPropertyNames(int index) throws ArrayIndexOutOfBoundsException {
		return PropertyNames[index];
	}
	
	public void setPropertyNames(String names[]){
		PropertyNames = names;
	}
	
	public String[] getPropertyNames() {
		return PropertyNames;
	}
	
	public void setGetConnected(boolean on){
		GetConnected = on;
		if (GetConnected) {
			if (db != null)
				return;
			db = new DFDataBaseConnection(getJDBCDriver(), getSubprotocol());
			/*GetConnected = */db.getConnected(getDSNOrProperty(), getUserName(), getPassword(), ConnectionProperties);
			setTableNames(db.getTableNames());
		}
		else{
			if (db != null)
				db.close();
			db = null;
			ConnectionProperties = null;
		}
	}
	public boolean isGetConnected(){
		return GetConnected;
	}
	
	public void setSetProperty(boolean on){
		SetProperty = on;
		if (SetProperty){
			if (ConnectionProperties == null)
				ConnectionProperties = new Properties();
			ConnectionProperties.put(getPropertyName(), getDSNOrProperty());
		}
	}
	
	public boolean isSetProperty(){
		return SetProperty;
	}
	
	public void setGetProperties(boolean on){
		GetProperties = on;
		if (GetProperties) {
			if (db != null){
				System.err.println(CTRLID+" You are already connected, please disconnect first");
				return;
			}
			db = new DFDataBaseConnection(getJDBCDriver(), getSubprotocol());
			PropertyNames = db.getPropertiesNames();
			if (PropertyNames == null)
			{
				String [] empnam = {"~empty~"};
				setPropertyNames(empnam);
			}
			db.close();
			db = null;
		}
	}
	public boolean isGetProperties(){
		return GetProperties;
	}

	public boolean updateQuery(String query){
		try {
			if (tb == null)
				tb = new DFDataBaseTable(db, getTableName());
			//tb.setCursorName("test");
			if (query == null || query == "")
				query = SELALLCOLUMN+getTableName();
			else{
				int metabpos = query.indexOf("%t");
				if (metabpos > 0) 
					query = query.substring(0, metabpos)+getTableName()+query.substring(metabpos+2);
			}
			if ("INSERT".regionMatches(true, 0, query, 0, 6)
                            || "UPDATE".regionMatches(true, 0, query, 0, 6)
                            || "DELETE".regionMatches(true, 0, query, 0, 6))
                           tb.updateQuery(query);
                        else 
			   tb.Requery(query);
		} catch (Exception e) {
			System.err.println(CTRLID+" query "+e);
			return false;
		}
		return true;
	}

	public boolean insertRec(String recSpec){
		try {
			if (tb == null)
				tb = new DFDataBaseTable(db, getTableName());
			tb.insertRec(recSpec);
		} catch (Exception e) {
			System.err.println(CTRLID+" insert "+e);
			return false;
		}
		return true;
	}
	
	public boolean updateRec(String val_and_condSpec, String cursorname){
		try {
			if (tbextra == null)
				tbextra = new DFDataBaseTable(db, getTableName());
			if (cursorname != null && cursorname != "")
				tbextra.setCursorName(cursorname);
			tbextra.updateRec(val_and_condSpec);
			
		} catch (Exception e) {
			System.err.println(CTRLID+" update "+e);
			return false;
		} finally{
			tbextra.close();
		}
		return true;
	}
	
	public boolean deleteRec(String condSpec, String cursorname){
		try {
			if (tbextra == null)
				tbextra = new DFDataBaseTable(db, getTableName());
			if (cursorname != null && cursorname != "")
				tbextra.setCursorName(cursorname);
			tbextra.deleteRec(condSpec);
		} catch (Exception e) {
			System.err.println(CTRLID+" delete "+e);
			return false;
		} finally{
			tbextra.close();
		}
		return true;
	}
	
	public boolean createTable(String tableDescription){
		try {
			if (tb == null)
				tb = new DFDataBaseTable(db, getTableName());
			tb.ms_tableDescription = tableDescription;
			return tb.createTable();
		} catch (Exception e) {
			System.err.println(CTRLID+" create "+e);
                        e.printStackTrace();
			return false;
		}
		
	}

	public ResultSet getResultSet() {
		
		return (tb == null)?null:tb.mc_resultSet;
	}

	public PreparedStatement getPreparedStatement(String templ){
		return (tb == null)?null:tb.getPreparedStatement(templ);
	}

	public int updatePreparedStatement(){
		return (tb == null)?0:tb.updatePreparedStatement();
	}

	public void Command (FCEvent e) {
		String action = e.getCommand();
		if (action.equals("Query")) {
			doQuery(e.getParam1());
		} else if (action.equals("Submit")) { 
			doSubmit();
		} else if (action.equals("Next")) {
			doNext();
		} else if (action.equals("Update")) {
			doUpdate();
		} else if (action.equals("Delete")) {
			doDelete();
		} else if (action.equals("FillTable")) {
			doFillTable();
		} else if (action.equals("TableName")) {
			setTableName(e.getParam1());
		} else if (action.equals("Create")) {
			doCreateTable(e.getParam1());
		} else if (action.equals("Reconnect")) {
			setGetConnected(false);
			setGetConnected(true);
		}
	}
			// commands
			/*
			1. BuildBindingList, 1001, null, dbcs
			2. CMDGetBindedElement, 1002, null, tables
			3. SetPropertyFor, 1003, propname, propval
			4. GetPropertyFor, 1004, propname, propval
			*/
	// we also need create a class including command constant definiton

	void doQuery(String q){
		// ask container prepareExchangeData();
		tables = new Hashtable(12);
		fireCommand(null, Constants.CMDGetBindedElements, null, tables);
		updateQuery(q);
	}
	
	void doCreateTable(String tbldescr) {
		createTable(tbldescr);
	}

	void doSubmit(){
		if (tables == null)
			return;
        Enumeration ten = tables.keys();
		while (ten.hasMoreElements()){
			String tbl = (String)ten.nextElement();
			Vector fields = (Vector)tables.get(tbl);
			String sqlFieldPart = "(";
			String sqlValuePart = ") VALUES (";
			String quote = "";
			if (fields != null && fields.size() > 0){
				if (usePrepared){
					String[] compns = (String[])fields.elementAt(0);
                    sqlFieldPart = "INSERT INTO "+tbl+" (";
                    sqlFieldPart += compns[2];
                    sqlValuePart += "?";
					for (int i = 1; i < fields.size(); i++){
						compns = (String[])fields.elementAt(i);
						// potential problem, when binded bean gives failure and this field has to be skipped
                        sqlFieldPart += ", "+compns[2];
                        sqlValuePart += ", ?";
					}
					//System.err.println(CTRLID+" Formed stmt: "+sqlFieldPart+sqlValuePart+")");
					java.sql.PreparedStatement pstmt = getPreparedStatement(sqlFieldPart+sqlValuePart+")");
					if (pstmt == null)
						continue;
					
					int r = 1;
					for (int i = 0; i < fields.size(); i++){
						compns = (String[])fields.elementAt(i);
						Object []propvals = new Object[1];
						propvals[0] = compns[1]; // prop name
						fireCommand(null, Constants.CMDGetPropertyFor, compns[0], propvals); // bean name
						Object propval = propvals[0];
						if (propval instanceof String || propval instanceof Integer)
						try{
							// System.err.println(CTRLID+" Value :"+propval);
							pstmt.setObject(r, propval);
						}catch(java.sql.SQLException es){
							System.err.println(CTRLID+" setObj() "+r+": "+es);
						}
						r++;
					}
					updatePreparedStatement();
				}else{
					for (int i = 0; i < fields.size(); i++){
						String[] compns = (String[])fields.elementAt(i);
						Object []propvals = new Object[1];
						propvals[0] = compns[1]; // prop name
						fireCommand(null, Constants.CMDGetPropertyFor, compns[0], propvals); // bean name
						Object propval = propvals[0];
						if (propval instanceof String)
							quote = "'"+propval+"'";
						else if (propval instanceof Integer)
							quote = String.valueOf(((Integer)propval).intValue());
						else quote = "";
						if (quote != ""){
							if (i != 0){
								sqlFieldPart += ", ";
								sqlValuePart += ", ";
							}
							sqlFieldPart += compns[2];
							sqlValuePart += quote;
						}
					}
					insertRec(sqlFieldPart+sqlValuePart+")");
				}
			}
		}
	}
	
	void doUpdate(){
		if (tables == null)
			return;
		Enumeration ten = tables.keys();			
		while (ten.hasMoreElements()){
			String tbl = (String)ten.nextElement();
			Vector fields = (Vector)tables.get(tbl);
			java.sql.ResultSet resset = getResultSet();
			String cursor = null;
			try{
				cursor = resset.getCursorName();
			}catch (java.sql.SQLException es){
				System.err.println(CTRLID+" Cursor "+es);
				continue;
			}
			// fields has to be sorted by 3rd string
			String sqlUpdtEqs = "";
			if (fields != null){
				for (int i = 0; i < fields.size(); i++){
					String[] compns = (String[])fields.elementAt(i);
					Object []propvals = new Object[1];
					propvals[0] = compns[1]; // prop name
					fireCommand(null, Constants.CMDGetPropertyFor, compns[0], propvals); // bean name
					Object propval = propvals[0];					
					if (propval != null && propval != ""){
						if (i != 0){
							sqlUpdtEqs += ", ";
						}
						sqlUpdtEqs += compns[2] + "=" + "'"+propval+"'";
					}
				}
				updateRec(sqlUpdtEqs+" WHERE CURRENT OF "+cursor, null);
			}
		}
		
	}

	void doNext(){
		if (tables == null)
			return;
		Enumeration ten = tables.keys();
		while (ten.hasMoreElements()) {
			String tbl = (String)ten.nextElement();
			Vector fields = (Vector)tables.get(tbl);
			java.sql.ResultSet resset = getResultSet();
			try{
				if (!resset.next()){
					System.err.println(CTRLID+" end of table's been reached");
					continue;
				}
			}catch (java.sql.SQLException es){
				System.err.println(CTRLID+" next "+es);
				continue; // for other tables
			}
			// fields has to be sorted by 3rd string
			if (fields != null){
				Object val = null;
				String[] compns = null;
				String[] prevcompns;
				for (int i = 0; i < fields.size(); i++){
					prevcompns = compns;
					compns = (String[])fields.elementAt(i);
					try {
						//<<<<<
						System.err.println(CTRLID+" reading '"+compns[2]+"'");
						if ((prevcompns != null && !compns[2].equals(prevcompns[2])) ||
							prevcompns == null){
							
							val = resset.getObject(compns[2]);
						}
					} catch (java.sql.SQLException es){
						System.err.println(CTRLID+" getObject for "+compns[2]+" causes "+es);
						val = "";
					} finally {
						if (val == null)
							val = "";
						if (val instanceof String)
							val = ((String)val).trim();
						Object []propvals = new Object[2];
						propvals[0] = compns[1];
						propvals[1] = val;
						fireCommand(null, Constants.CMDSetConvPropertyFor, compns[0], propvals); // bean name
					}
				}
			}
		}
	}
	
	void doFillTable() {
		if (tables == null)
			return;
		Enumeration ten = tables.keys(); // available tables
		while (ten.hasMoreElements()) {
			String tbl = (String)ten.nextElement();
			Vector fields = (Vector)tables.get(tbl);
			if (fields == null)
				continue;
			java.sql.ResultSet resset = getResultSet();
			Object val = null;
			String[] compns = null;
			String[] prevcompns;
			Hashtable beans = new Hashtable(fields.size());
			try{
				// we should do one navigation for each table bean, so
				if (resset.next()){
					for (int i = 0; i < fields.size(); i++){
						prevcompns = compns;
						compns = (String[])fields.elementAt(i);
						if (beans.get(compns[0]) == null) {
							beans.put(compns[0], compns[0]);
							fireCommand(null, Constants.CMDSetRow, compns[0], "0");
						}
						try{
							//System.err.println(CTRLID+" reading '"+compns[2]+"'");
							if ((prevcompns != null && !compns[2].equals(prevcompns[2])) ||
								prevcompns == null){
								val = resset.getObject(compns[2]);
							}
						}catch (java.sql.SQLException es){
							System.err.println(CTRLID+" getObject for "+compns[2]+" causes "+es);
							val = "";
						} finally {
							if (val == null)
								val = "";
							Object []propvals = new Object[2];
							propvals[0] = compns[1];
							propvals[1] = val;
							fireCommand(null, Constants.CMDSetConvPropertyFor, compns[0], propvals); // bean name
						}
					}
				}
				Enumeration be = beans.keys();
				while (be.hasMoreElements())
					fireCommand(null, Constants.CMDNextRow, (String)be.nextElement(), null);
				while (resset.next()){
					for (int i = 0; i < fields.size(); i++){
						prevcompns = compns;
						compns = (String[])fields.elementAt(i);
						try{
							if ((prevcompns != null && !compns[2].equals(prevcompns[2])) ||
								prevcompns == null){
								val = resset.getObject(compns[2]);
							}
						}catch (java.sql.SQLException es){
							System.err.println(CTRLID+" getObject for "+compns[2]+" causes "+es);
							val = "";
						} finally {
							if (val == null)
								val = "";
							Object []propvals = new Object[2];
							propvals[0] = compns[1];
							propvals[1] = val;
							//System.err.println(CTRLID+" set '"+compns[1]+"' to "+propvals[1]);
							fireCommand(null, Constants.CMDSetConvPropertyFor, compns[0], propvals); // bean name
						}
					}
					be = beans.keys();
					while (be.hasMoreElements())
						fireCommand(null, Constants.CMDNextRow, (String)be.nextElement(), null);
				}
				be = beans.keys();
				while (be.hasMoreElements())
					fireCommand(null, Constants.CMDTruncByCurrent, (String)be.nextElement(), null);
			}catch (java.sql.SQLException es){
				System.err.println(CTRLID+" next "+es);
			}
		}
	}

	void doDelete(){
		if (tables == null)
			return;
		Enumeration ten = tables.keys();
		while (ten.hasMoreElements()){
			String tbl = (String)ten.nextElement();
			Vector fields = (Vector)tables.get(tbl);
			java.sql.ResultSet resset = getResultSet();
			String cursor = null;
			try{
				cursor = resset.getCursorName();
			}catch (java.sql.SQLException es){
				System.err.println(CTRLID+" cursor "+es);
				continue;
			}
			deleteRec("CURRENT OF "+cursor, null);
		}
		
	}

	//---------------------------------------------------------
	public synchronized void addCommandListener(IFCEvListener l) throws java.util.TooManyListenersException {
		if (container != null)
			throw new java.util.TooManyListenersException();
		container = l;
	}
	
	public synchronized void removeCommandListener(IFCEvListener l){
		container = null;
	}
	
	public void fireCommand(String com, int code, String name, Object data){
		// we do not need synchronize, because we use just one listener
		if (container == null)
			return;
		synchronized (this){
			container.Command(new FCEvent(this, com, code, name, data));
		}
	}

	private IFCEvListener container = null;
	private static String SELALLCOLUMN = "SELECT * FROM ";
	private DFDataBaseConnection db;
	private DFDataBaseTable tb;
	private DFDataBaseTable tbextra; // extra statement for update/delete
	private ResultSetMetaData md;
	private Hashtable tables;
	private boolean usePrepared = true;
}
