/*
 * File:		DigitalPicture.java
 *
 * Description:	DigitalPicture : represents a digal picture in the album editor
 * 
 *
 * Copyright (C) 2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.components;

import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Point;
import java.io.File;
import java.net.URL;
import java.net.MalformedURLException;
import javax.swing.*;
import javax.swing.border.*;

import photoorganizer.formats.*;
import photoorganizer.*;

public class DigitalPicture extends JLabel {
    private String border;
    private String title;
    private String imageSource;
    
    public DigitalPicture() {
        title = "Picture";
    }

    public String getBorderName() {
	return border;
    }

    public void setBorderName(String text) {
	if (!border.equals(text)) {
	    border = text;
	    if (border.equals("Empty")) {
		setBorder(BorderFactory.createEmptyBorder());
	    } else if (border.equals("Etched")) {
		setBorder(BorderFactory.createEtchedBorder());
	    } else if (border.equals("Titled")) {
		setBorder(BorderFactory.createTitledBorder(border));
		((TitledBorder)getBorder()).setTitle(title);
	    } else if (border.equals("Line")) {
		setBorder(BorderFactory.createLineBorder(Color.black));
	    } else if (border.equals("Bevel Up")) {
		setBorder(BorderFactory.createRaisedBevelBorder());
	    } else if (border.equals("Bevel Down")) {
		setBorder(BorderFactory.createLoweredBevelBorder());
	    } else if (border.equals("Soft Bevel")) {
		setBorder(BorderFactory.createBevelBorder(2));
	    } else if (border.equals("Compound")) {
		setBorder(BorderFactory.createCompoundBorder());
	    } else if (border.equals("Matte")) {
		setBorder(BorderFactory.createMatteBorder(2,2,2,2,Color.white));
	    }
	}
    }
    
    public String getImageLocation() {
        return (imageSource == null)?"":imageSource;
    }
    
    public void setImageLocation(String l) {
        if (l != null)
            imageSource = l;
        if (getIcon() == null) {
            BasicJpeg format = new BasicJpeg(new File(l));
            try {
		setIcon(format.getImageInfo().getThumbnailIcon(format));
            } catch(Throwable t) {
                // drop off
            }
        }
    }
    
    public String getSVGString() {
        //Rectangle r = getBounds();
        Dimension d = getSize();
        Point p;
        if (java.beans.Beans.isDesignTime())
            p = getParent().getLocation();
        else
            p = getLocation();
        StringBuffer result = new StringBuffer(40);
        result.append("<image x=\"");
        result.append(p.x).append("\" y=\"").append(p.y);
        result.append("\" width=\"").append(d.width);
        result.append(/*"px*/"\" height=\"").append(d.height);
        result.append(/*"px*/"\" \n xlink:href=\"");
        try {
            result.append(new URL("file", "localhost/", getImageLocation()).toString());
        } catch(MalformedURLException mfpe) {
            result.append(getImageLocation());
        }
        result.append("\">\n ");
        result.append("<title> will be added soon</title>");
        result.append("</image>");
        return result.toString();
    }
    
    public void setSVGString(String s) {
    }
}
