/*
 * File:	DigitalPictureCustomizer.java
 *
 * Description:	DigitalPictureCustomizer : Customizer for DigitalPicture
 * 
 *
 * Copyright (C) 2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.components;

import java.util.Vector;
import java.beans.*;
import javax.swing.*;
import javax.swing.table.TableModel;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;

import rogatkin.*;
import photoorganizer.renderer.*;
import photoorganizer.formats.*;
import photoorganizer.*;
import rogatkin.xbox.photoorganizer.AlbumEditor; // todo: move this class to PhotoOrganizer tree

public class DigitalPictureCustomizer extends JPanel
    implements Customizer {
    
    DigitalPicture pic;
    BaseController controller;
    PhotoCollectionPanel cp;
    
    public DigitalPictureCustomizer() {
        JTabbedPane tabbedpane = new JTabbedPane(SwingConstants.TOP);
        tabbedpane.insertTab("Collection", (Icon)null,
                             new JScrollPane(list = new JList(getCollection())),
                             "Select a picture from the current collection",
                             0);
        add(tabbedpane);
	list.addListSelectionListener(new ListSelectionListener() {
	    public void valueChanged(ListSelectionEvent e) {
		int index = list.getSelectedIndex();
		if (index >= 0) {
		    BasicJpeg format = (BasicJpeg)((Thumbnail)list.getModel().getElementAt(index)).getFormat();
		    pic.setIcon(format.getImageInfo().getThumbnailIcon(format));
                    pic.setImageLocation(format.getLocationName());
		}
	    }
	});
    }
    
    public void setObject(Object bean) {
        pic = (DigitalPicture) bean;
        list.setModel(getCollection());
	list.updateUI();
    }
    
    ListModel getCollection() {
        // try to find out controller to reach PhotoOrganizer resources
        // for fast result with further replacment by Class.forName
        controller = AlbumEditor.getController();
        // should be not null
        cp = (PhotoCollectionPanel)controller.component(new Integer(5)/*COMP_COLLECTION*/);
        return (ListModel)cp.getModel();
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
    }
    
    public void removePropertyChangeListener(PropertyChangeListener listener) {
    }
    
    JList list;
}
