/*
 * File:		rfChoice.java
 *
 * Description:	rfChoice : implements combobox
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */
package rogatkin.xbox.components;

import javax.swing.*;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;

public class rfChoice extends JComboBox {
    private String lookandfeel;
	private ChoiceModel model;

	public rfChoice() {
		setModel(model = new ChoiceModel());
        lookandfeel = "Java";
	}
	// work around sun bug
	public void setSelectedIndex(int anIndex) {
		if (anIndex == 0 && model.getSize() == 0)
			setSelectedItem( null );
		else
			super.setSelectedIndex(anIndex);
	}

    public void setChoices(int size) {
		model.setSize(size);
        invalidate();
    }
    
    public int getChoices() {
        return getItemCount();
    }
	
    public void setSelection(int selection){
		try {
			setSelectedIndex(selection);
		} catch (IllegalArgumentException iae) {
		}
	}
	
    public int getSelection() {
		return getSelectedIndex();
	}
	
    public void setText(String text){
		model.setSelectedItem(text);
        repaint();
	}
	
    public String getText(){
        return (String)model.getSelectedItem();
	}
	
	public void setTexts(String[] texts){
        model.setTexts(texts);
        invalidate();
	}

	public String[] getTexts() {
		return model.getTexts();
	}
    
    public void setLookAndFeel(String lookandfeel) {
        this.lookandfeel = lookandfeel;
        try {
            if (lookandfeel.equals("Motif"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
            else if (lookandfeel.equals("Windows"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            else
                UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
            SwingUtilities.updateComponentTreeUI(this);
        } 
        catch (Exception e) {
        }
    }

    public String getLookAndFeel() {
        return lookandfeel;
    }

    public void setToolTipText(String text) {
        super.setToolTipText(text);
    }

    public String getToolTipText() {
        return (super.getToolTipText() == null)?"":super.getToolTipText();
    }
}

class ChoiceModel implements ComboBoxModel {
	
	public ChoiceModel()
	{
        _delegate = new BaseChoice();
	}

	public void setSize(int size) {
        _delegate.setChoices(size);
	}

    public void setText(String text){
        if (_delegate.validateSelection()) {
	        _delegate.setText(text);
		}
	}
	
    public String getText(){
        return _delegate.getText();
	}
	
	public void setTexts(String[] texts){
		_delegate.setTexts(texts);
	}

	public String[] getTexts() {
		return _delegate.getTexts();
	}

    public void setSelection(int selection){
		_delegate.setSelection(selection);
	}
	
    public int getSelection() {
		return _delegate.getSelection();
	}
	
	// list and combobox model interface methods
	public int getSize() {
		return _delegate.getChoices();
	}
	public java.lang.Object getElementAt(int index) {
		_delegate.setSelection(index);
		return _delegate.getText();
	}
	public void addListDataListener(ListDataListener l) {
		// is not supported for now
	}
	public void removeListDataListener(ListDataListener l) {
		// is not supported for now
	}
	public void setSelectedItem(java.lang.Object anItem) {
		_delegate.setText((String)anItem);
	}
	public java.lang.Object getSelectedItem() {
		return _delegate.getText();
	}
	private BaseChoice _delegate;
}