/*
 * File:		rfDataBaseAction.java
 *
 * Description:	rfDataBaseAction implements different type database manipulations
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */
package rogatkin.xbox.components;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import rogatkin.xbox.shared.*;

public class rfDataBaseAction implements ActionListener, IBindingList {
	private final String CLASSID = "rfDataBaseAction:";
	private String Query = "SELECT * FROM %t"; // meta symbol %t should be replace on table name
	private String dBActionName = "Submit";
	private String tablename = "";
	private boolean action = false;

	public String getQuery(){
		return Query;
	}
	
	public void setQuery(String q){
		Query = q;
	}

	public String getTableName(){
		return tablename;
	}
	
	public void setTableName(String name){
		tablename = name;
	}
	
	public String getDBActionName(){
		return dBActionName;
	}
	
	public void setDBActionName(String act){
		dBActionName = act;
	}
	
	// ActionListener interface
	public void actionPerformed(ActionEvent e){
		fireBtnEvent();
	}
	
	public void setDoAction(boolean on) {
		if (action)
			fireBtnEvent();
		action = on;
	}
	
	public boolean isDoAction() {
		return action;
	}

	void fireBtnEvent() {
		Vector l;
		FCEvent s;
		if (getDBActionName().equals("Create"))
		{
			s = new FCEvent (this, "TableName", 0, getTableName());
			synchronized (this)
			{
				l = (Vector) dBActionListeners.clone();
			}
			for (int c=0;c<l.size();c++)
			{
				IFCEvListener dal = (IFCEvListener)l.elementAt(c);
				dal.Command(s);
			}
		}
		s = new FCEvent (this, getDBActionName(), 0, substituteBinding(getQuery()));
		synchronized (this)
		{
			l = (Vector) dBActionListeners.clone();
		}
		for (int c=0;c<l.size();c++)
		{
			IFCEvListener dal = (IFCEvListener)l.elementAt(c);
			dal.Command(s);
		}
	}

	public void fireCommand(String com, int code, String name, Object data){
		Vector l;
		FCEvent s = new FCEvent (this, com, code, name, data);
		synchronized (this)
		{
			l = (Vector) dBActionListeners.clone();
		}
		for (int c=0;c<l.size();c++)
		{
			IFCEvListener dal = (IFCEvListener)l.elementAt(c);
			dal.Command(s);
		}
	}
	
	private Vector dBActionListeners = new Vector();
	
	public synchronized void addCommandListener(IFCEvListener l)
	{
		dBActionListeners.addElement(l);
	}
	
	public synchronized void removeCommandListener(IFCEvListener l)
	{
		dBActionListeners.removeElement(l);
	}
	
	String substituteBinding(String src) {
		// intern implementation (current implementation dependent,
		// it doesn't use calling interface methods)
		String result;
		int idx;
		Hashtable substitutions = new Hashtable(12);
		String []compns;
		fireCommand(null, Constants.CMDGetBindedElements, null, substitutions);
        Enumeration sen = substitutions.keys();
		while (sen.hasMoreElements()){
			String chunk = (String)sen.nextElement();
			Vector fields = (Vector)substitutions.get(chunk);
			if (fields == null)
				continue;
			Object []propvals = new Object[2];
			for (int i = 0; i < fields.size(); i++){
				compns = (String[])fields.elementAt(i);
				idx = src.indexOf(compns[2]);
				if (idx < 0)
					continue;
				propvals[0] = compns[1]; // prop name
				fireCommand(null, Constants.CMDGetPropertyFor, compns[0], propvals); // bean name
				
				if (propvals[0] != null && propvals[0] instanceof String) {
					if (compns[2].equals((String)propvals[0]))
						continue;
					while(idx > 0) { // perform all substitutions
						src = src.substring(0, idx) + propvals[0] +
							src.substring(idx+compns[2].length());
						idx = src.indexOf(compns[2]);
					}
				}
			}
		}
		System.err.println(CLASSID+"String after substitutions "+src);
		return src;
	}
	
	public void setElementName(String name) {
		setElementNames(getCurrentElement(), name);
	}
	public String getElementName() {
		return getElementNames(getCurrentElement());
	}
	public int getCurrentElement() {
		return currentelement;
	}
	public void setCurrentElement(int c) {
		if (c < getElementsCount() && c >= 0)
			currentelement = c;
	}
	public  boolean	isBuiltList() {
		return buildlist;
	}
	public  void getBuiltList(boolean on) {
		buildlist = on;
	}
	public  int getElementsCount() {
		return count;
	}
	public  void setElementsCount(int c) {
		if (c < 0)
			return;
		String[] result = new String[c];
		if (c > 0) 
			System.arraycopy(elementnames, 0, result, 0, (elementnames.length>c)?c:elementnames.length);
		count = c;
		elementnames = result;
	}
	public  String getElementNames(int index) {
		try {
		return elementnames[index];
		} catch(Exception e) {
			return "";
		}
	}
	public  String[] getElementNames() {
		return elementnames;
	}
	public  void setElementNames(String[] names) {
		elementnames = names;
	}
	public  void setElementNames(int index, String name) {
		try {
		elementnames[index] = name;
		} catch(Exception e) {
		}
	}
	public  String getElementInfos(int index) {
		return getElementNames(index);
	}
	public  String[] getElementInfos() {
		return getElementNames();
	}
	public  void setElementInfos(String [] names) {
		setElementNames(names);
	}
	public  void setElementInfos(int index, String name) {
		setElementNames(index, name);
	}

private boolean buildlist = false;
private int count = 0;
private String[] elementnames = new String[count];
private int currentelement = 0;
}
