/*
 * File:		rfDecoration.java
 *
 * Description:	rfDecoration : implements elements of decoration and can be
 * background bean
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.components;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Point;
import javax.swing.*;
import javax.swing.border.TitledBorder;

public class rfDecoration extends JPanel {
    private boolean tiled;
    private String title;
    private String border;
    private String lookandfeel;
    private String imagename;
    private ImageIcon icon;
    
    public rfDecoration() {
        lookandfeel = "Java";
        title = "";
        border = "Empty";
        imagename = "";
        tiled = false;
        icon = null;
    }
    public boolean isTiled() {
	return tiled;
    }
    public void setTiled(boolean on) {
	tiled = on;
    }
    public String getTitle() {
	return title;
    }
    public void setTitle(String title) {
	this.title = title;
	if (border.equals("Titled")) {
	    if (getBorder() instanceof TitledBorder)
		((TitledBorder)getBorder()).setTitle(title);
	}
    }
    public String getBorderName() {
	return border;
    }
    public void setBorderName(String text) {
	if (!border.equals(text)) {
	    border = text;
	    if (border.equals("Empty")) {
		setBorder(BorderFactory.createEmptyBorder());
	    } else if (border.equals("Etched")) {
		setBorder(BorderFactory.createEtchedBorder());
	    } else if (border.equals("Titled")) {
		setBorder(BorderFactory.createTitledBorder(border));
		((TitledBorder)getBorder()).setTitle(title);
	    } else if (border.equals("Line")) {
		setBorder(BorderFactory.createLineBorder(Color.black));
	    } else if (border.equals("Bevel Up")) {
		setBorder(BorderFactory.createRaisedBevelBorder());
	    } else if (border.equals("Bevel Down")) {
		setBorder(BorderFactory.createLoweredBevelBorder());
	    } else if (border.equals("Soft Bevel")) {
		setBorder(BorderFactory.createBevelBorder(2));
	    } else if (border.equals("Compound")) {
		setBorder(BorderFactory.createCompoundBorder());
	    } else if (border.equals("Matte")) {
		setBorder(BorderFactory.createMatteBorder(2,2,2,2,Color.white));
	    }
	}
    }
    public String getBackgroundImage() {
	return imagename;
    }
    public void setBackgroundImage(String text) {
	imagename = text;
        icon = new ImageIcon(text);
    }
    
    public void setLookAndFeel(String lookandfeel) {
        this.lookandfeel = lookandfeel;
        try {
            if (lookandfeel.equals("Motif"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
            else if (lookandfeel.equals("Windows"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            else
                UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
            SwingUtilities.updateComponentTreeUI(this);
        } 
        catch (Exception e) {
        }
	
    }
    
    public String getLookAndFeel() {
        return lookandfeel;
    }

    public String getSVGString() {
        //Rectangle r = getBounds();
        Dimension d = getSize();
        Point p;
        if (java.beans.Beans.isDesignTime())
            p = getParent().getLocation();
        else
            p = getLocation();
        StringBuffer result = new StringBuffer(40);
        result.append("<rect x=\"");
        result.append(p.x).append("\" y=\"").append(p.y);
        result.append("\" width=\"").append(d.width);
        result.append("\" height=\"").append(d.height);
        String hs = Integer.toHexString(getBackground().getRGB() & 0xffffff);
        while (hs.length() < 6)
            hs = "0"+hs;
        result.append("\" \n style=\"fill:#"+hs);
        result.append("\">\n ");
        result.append("<title> for rect will be added soon</title>");
        result.append("</rect>");
        return result.toString();
    }
    
    public void setSVGString(String s) {
    }
    
    public void paint(Graphics  g) {
        if (icon == null) {
            super.paint(g);
            return;
        }
        Dimension d = getSize();
        if (tiled) {
            for (int x=0; x<d.width ; x+=icon.getIconWidth())
		for (int y=0; y<d.height; y+=icon.getIconHeight())
                icon.paintIcon(this, g, x, y);
            icon.paintIcon(this, g, 0, 0);
        } else {
            icon.paintIcon(this, g, (d.width - icon.getIconWidth())/2, 
                (d.height - icon.getIconHeight())/2);
        }
    }
}