/*
 * File:		rfDispatcher.java
 *
 * Description:	rfDispatcher organizers a sequence of action events to
 *              controls work of other components
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */
package rogatkin.xbox.components;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

import rogatkin.xbox.ut.OrderedQueue;
import rogatkin.xbox.shared.IBindingList;
import rogatkin.xbox.shared.IFCEvListener;
import rogatkin.xbox.shared.FCEvent;
import rogatkin.xbox.shared.Constants;

public class rfDispatcher implements ActionListener, IBindingList {
	final static int MAXDISPATCHEDBEANS = 5;
	final static String CLASSID = "rfDispatcher: ";
	String [] actionnames;
	String tablename; 

	public rfDispatcher() {
		actionnames = new String[MAXDISPATCHEDBEANS];
		for (int i=0; i<MAXDISPATCHEDBEANS; i++)
			actionnames[i] = "Action"+i;
		tablename = "";
	}
	// IBindingList interface functions
	public  boolean	isBuiltList(){
		return true;
	}

	public  void getBuiltList(boolean on){
	}

	public  int getElementsCount(){
		return MAXDISPATCHEDBEANS;
	}

	public  void setElementsCount(int c){
	}

	public  String getElementNames(int index){
		return actionnames[index];
	}

	public  String[] getElementNames(){
		return actionnames;
	}
	
	public  void setElementNames(String[] names){
	}

	public  void setElementNames(int index, String name){
	}
	
	public  String getElementInfos(int index){
		return actionnames[index];
	}
	
	public  String[] getElementInfos(){
		return actionnames;
	}
	
	public  void setElementInfos(String [] names){
	}
	
	public  void setElementInfos(int index, String name){
	}

    public String getTableName(){
        return tablename;
    }
	
    public void setTableName(String name){
		tablename = name;
    }

    // ActionListener interface function
	public void actionPerformed(ActionEvent ae) {
		Hashtable ctrlslist = new Hashtable(MAXDISPATCHEDBEANS);
		OrderedQueue oq = new OrderedQueue();
		oq.setOrder(OrderedQueue.ANCESSOR);
		String []compns;
		fireCommand(null, Constants.CMDGetBindedElements, null, ctrlslist);
		Enumeration ten = ctrlslist.keys();
		while (ten.hasMoreElements()) {
			String tbl = (String)ten.nextElement();
			Vector fields = (Vector)ctrlslist.get(tbl);
			// fields has to be sorted by 3rd string
			if (fields == null)
				continue;
			
			for (int i = 0; i < fields.size(); i++) {
				compns = (String[])fields.elementAt(i);
				System.err.println(CLASSID+"Fields "+compns[0]+" "+compns[1]+" "+compns[2]);
				oq.push(compns[2].hashCode(), compns);
			}
		}
		Object []propvals = new Object[2];
		while ((compns = (String[])oq.pop()) != null)
		{
			//System.err.println(CLASSID+"Set fields "+compns[0]+" "+compns[1]+" "+compns[2]);
			propvals[0] = compns[1];
			propvals[1] = Boolean.TRUE;
			fireCommand(null, Constants.CMDSetPropertyFor, compns[0], propvals); // bean name
			fireCommand(null, Constants.CMDGetPropertyFor, compns[0], propvals);
			if (propvals[0] instanceof Boolean)
				if (!((Boolean)propvals[0]).booleanValue())
					break;
		}
	}

	public synchronized void addCommandListener(IFCEvListener l) throws java.util.TooManyListenersException {
		if (container != null)
			throw new java.util.TooManyListenersException();
		container = l;
	}
	
	public synchronized void removeCommandListener(IFCEvListener l){
		container = null;
	}
	
	public void fireCommand(String com, int code, String name, Object data){
		// we do not need synchronize, because we use just one listener
		if (container == null)
			return;
		synchronized (this){
			container.Command(new FCEvent(this, com, code, name, data));
		}
	}

	private IFCEvListener container = null;

}