/*
 * File:		rfList.java
 *
 * Description:	rfList : implements editbox
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */

package rogatkin.xbox.components;

import javax.swing.*;

public class rfList extends JScrollPane {
    private JList list;
    private String lookandfeel;
    private ChoicesModel model;
    public rfList() {
	_delegate = new BaseChoice();
        list = new JList(model = new ChoicesModel());
        getViewport().setView(list);
    }
    
    public void setLookAndFeel(String lookandfeel) {
        this.lookandfeel = lookandfeel;
        try {
            if (lookandfeel.equals("Motif"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
            else if (lookandfeel.equals("Windows"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            else
                UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
            SwingUtilities.updateComponentTreeUI(this);
        } 
        catch (Exception e) {
        }
	
    }
    
    public String getLookAndFeel() {
        return lookandfeel;
    }
    
    public void setToolTipText(String text) {
        list.setToolTipText(text);
    }
    
    public String getToolTipText() {
        return (list.getToolTipText() == null)?"":list.getToolTipText();
    }
    
    public void setNumElements(int size) {
        model.setSize(size);
    }
    
    public int getNumElements() {
	return _delegate.getChoices();
    }
    
    public void setText(String text) {
        model.setText(text);
    }
    
    public String getText() {
        return (String)model.getSelectedItem();
    }
    
    public void setTexts(String[] texts) {
        model.setTexts(texts);
    }
    
    public String[] getTexts() {
	return _delegate.getTexts();
    }
    
    public void setSelection(int selection) {
        list.setSelectedIndex(selection);
	_delegate.setSelection(selection);
    }
    
    public int getSelection() {
        return list.getSelectedIndex();
	//return _delegate.getSelection();
    }
    
    private BaseChoice _delegate;
    
    class ChoicesModel extends AbstractListModel {
	
        public void setSize(int size) {
            int old_size = _delegate.getChoices();
            _delegate.setChoices(size);
            if (old_size < size)
                fireIntervalAdded(this, old_size, size);
            else
                fireIntervalRemoved(this, size, old_size);
        }
	
        public void setText(String text) {
            if (_delegate.validateSelection()) {
                _delegate.setText(text);
            }
            fireContentsChanged(this, _delegate.getSelection(), _delegate.getSelection());
        }
	
        public void setTexts(String[] texts) {
            _delegate.setTexts(texts);
            fireContentsChanged(this, 0, _delegate.getChoices());
        }
	
        // list and combobox model interface methods
	public int getSize() {
	    return _delegate.getChoices();
	}
	
	public Object getElementAt(int index) {
	    return _delegate.getTexts(index);
	}
	
	public void setSelectedItem(Object anItem) {
	    _delegate.setText((String)anItem);
	}
	
	public Object getSelectedItem() {
	    return _delegate.getText();
	}
    }
}

