/*
 * File:		rfRadioBtnGrp.java
 *
 * Description:	rfRadioBtnGrp : implements radio button group
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */
package rogatkin.xbox.components;

import java.util.Vector;
import java.awt.Font;
import java.awt.GridLayout;

import javax.swing.*;
import javax.swing.border.TitledBorder;

public class rfRadioBtnGrp extends JPanel {
    private BaseChoice _delegate;
    private ButtonGroup btngroup;
    private TitledBorder border;
    private String lookandfeel;

    public rfRadioBtnGrp() {
        _delegate = new BaseChoice();
	setLayout(new GridLayout(0, 1));
        border = BorderFactory.createTitledBorder("Radio Buttons");
        setBorder(border);
        lookandfeel = "Java";
        rebuild();
    }
    
    public void setOpaque(boolean on) {
	super.setOpaque(on);
        for (int i=0; i< getChoices(); i++) {
	    try {
		if (getComponent(i) instanceof JRadioButton)
		    ((JComponent)getComponent(i)).setOpaque(on);
	    }catch(Throwable e) {
	    }
	}
    }
    
    public boolean isOpaque() {
	return super.isOpaque();
    }
    
    public void setChoices(int size) {
        _delegate.setChoices(size);
        rebuild();
    }
    
    public int getChoices() {
	try {
	    return _delegate.getChoices();
	} catch (NullPointerException e) {
	}
	return 0;
    }
	
    public void setSelection(int selection) {
	_delegate.setSelection(selection);
        if (_delegate.validateSelection() &&
	    getComponent(_delegate.getSelection()) instanceof JRadioButton)
	    ((JRadioButton)getComponent(_delegate.getSelection())).setSelected(true);
    }
    
    public int getSelection() {
        for (int i=0; i< getChoices(); i++) {
	    if (getComponent(i) instanceof JRadioButton &&
		((JRadioButton)getComponent(i)).isSelected()) {
		_delegate.setSelection(i);
		return (i);
	    }
	}
        return -1;
    }
    
    public void setText(String text) {
        if (_delegate.validateSelection()) {
	    ((JRadioButton)getComponent(getSelection())).setText(text);
	    _delegate.setText(text);
	}
    }
    
    public String getText(){
        getSelection();
        return _delegate.getText();
    }
    
    public void setTexts(String[] texts){
	_delegate.setTexts(texts);
        rebuild();
    }
    
    public String[] getTexts() {
	return _delegate.getTexts();
    }
    
    public void setTitle(String title){
        border.setTitle(title);
    }
    
    public String getTitle(){
        return border.getTitle();
    }
    
    public Font getFont(){
        return super.getFont();
    }
    
    public void setFont(Font font) {
        super.setFont(font);
        for (int i = 0; i < getComponentCount(); i++)
            getComponent(i).setFont(font);
    }
    
    public void rebuild(){
        removeAll();
        btngroup = new ButtonGroup();
        //for (;;;)
        //    btngroup.remove(button);
        JRadioButton rb ;
        for (int i=0; i< getChoices(); i++) {
            add(rb = new JRadioButton(_delegate.getTexts(i)));
            btngroup.add(rb);
            if (i == _delegate.getSelection())
                rb.setSelected(true);
            //.setKeyAccelerator();
        }     
    }
    
    public void setLookAndFeel(String lookandfeel) {
        this.lookandfeel = lookandfeel;
        try {
            if (lookandfeel.equals("Motif"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
            else if (lookandfeel.equals("Windows"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            else
                UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
            SwingUtilities.updateComponentTreeUI(this);
        } 
        catch (Exception e) {
        }
	
    }
    
    public String getLookAndFeel() {
        return lookandfeel;
    }
    
    /*public void actionPerformed(ActionEvent e) {
    if (e.getActionCommand() == first) {
    }
    }*/
    
    /*public void itemStateChanged(ItemEvent e) {
    }*/
}
