/*
 * File:		rfTable.java
 *
 * Description:	rfTable : implements Table
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */
package rogatkin.xbox.components;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Color;
import java.util.Hashtable;

import javax.swing.*;
import javax.swing.border.TitledBorder;
import javax.swing.table.TableModel;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.table.TableColumn;

import rogatkin.xbox.shared.IRowProperties;

public class rfTable extends JScrollPane implements TableModel, IRowProperties {
	
	public rfTable() {
        super(new JTable());
        table = (JTable)getViewport().getView();
		colnames = new Hashtable();
        numcols = 1;
		numrows = 1;
		row = 0;
		col = 0;
        contents = new String[numcols];
		contents[0] = EMPTYSTRING;
        headers = new String[numcols];
		headers[0] = "Col0";
        types = new String[numcols];
		types[0] = "string";
        widths = new int[numcols];
        lookandfeel = "Java";
        table.setModel(this);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
	}
    
    public boolean isShowGrid(){
        return table.getShowVerticalLines() && table.getShowHorizontalLines();
    }

    public void setShowGrid(boolean on) {
        table.setShowGrid(on);
    }

    public void setNumRows(int numrows) {
		if (numrows == this.numrows)
			return;
		if (this.numrows*numcols == contents.length)
		{
			int newdim = numrows*numcols;
			String []result = new String[newdim];
			int ncopy = (contents.length > newdim)?newdim:contents.length;
			System.arraycopy(contents, 0, result, 0, contents.length);
			contents = result;
		}
		this.numrows = numrows;
        table.tableChanged(new TableModelEvent(this));
        table.revalidate();
	}

	public int getNumRows() {
        return numrows;
	}
	
	public void setNumCols(int newnumcols) {
		if (numcols == newnumcols)
			return;
		if (numcols*numrows == contents.length)
		{
			int newdim = numrows*newnumcols;
			int ncopy = (numcols>newnumcols)?newnumcols:numcols;
			String[] result = new String[newdim];
			for (int i = 0; i < numrows; i++)
				System.arraycopy(contents, i*numcols, result, i*newnumcols, ncopy);
			contents = result;
			result = new String[newnumcols];
			for (int i = 0; i < newnumcols; i++)
				result[i] = "Col"+i;
			System.arraycopy(headers, 0, result, 0, ncopy);
			
			headers = result;
			updatesColHeadersTabl();
			result = new String[newnumcols];
			System.arraycopy(types, 0, result, 0, ncopy);
			types = result;
			int[] iresult = new int[newnumcols];
			System.arraycopy(widths, 0, iresult, 0, ncopy);
			widths = iresult;
		}
		numcols = newnumcols;
		table.tableChanged(new TableModelEvent(this, TableModelEvent.HEADER_ROW));
		table.revalidate();
	}

	public int getNumCols() {
		return numcols;
	}
	
	public void setRow(int i) {
        row = i;
	}
	
	public int getRow() {
		return row;
	}
	
	public void setCol(int i) {
        col = i;
	}
	
	public int getCol() {
		return col;
	}
	
	public void setCellString(String text) {
        try {
			if (numcols*numrows == contents.length)
				contents[row*numcols+col] = text;
			// else put it to some temporary cell and set when dim will valid
        } catch (ArrayIndexOutOfBoundsException iobe) {
        } catch (NullPointerException npe) {
        }
	}
	
	public String getCellString() {
        try {
            return contents[getRow()*getNumCols()+getCol()];
        } catch (ArrayIndexOutOfBoundsException iobe) {
            return EMPTYSTRING;
        } catch (NullPointerException npe) {
            return EMPTYSTRING;
        }
	}
	
	public String[] getContents() {
        return contents;
	}

	public void setContents(String[] texts) {
        contents = texts;
		table.tableChanged(new TableModelEvent(this, TableModelEvent.HEADER_ROW));
		table.revalidate();
        //updateWidths();
	}
	
    public String[] getHeaders() {
        return headers;
	}

	public void setHeaders(String[] texts) {
        headers = texts;
		updatesColHeadersTabl();
        table.tableChanged(new TableModelEvent(this, TableModelEvent.HEADER_ROW));
        table.revalidate();
        updateWidths();
	}
	
    public String getHeader() {
        if (col > -1 && col < numcols && headers[col] != null)
            return headers[col];
        else
            return EMPTYSTRING;
	}

	public void setHeader(String text) {
        if (col > -1 && col < numcols) {
			colnames.remove(headers[col]);
            headers[col] = text;
			colnames.put(headers[col], new Integer(col));
            table.tableChanged(new TableModelEvent(this, TableModelEvent.HEADER_ROW));
            table.revalidate();
            updateWidths();
        }
	}
	
    public String[] getTypes() {
        return types;
	}

    public void setTypes(String[] texts) {
        types = texts;
	}
    
    public String getType() {
        if (col > -1 && col < numcols)
            if (types[col] != null)
                return types[col];
            return EMPTYSTRING;
	}

    public void setType(String text) {
        if (col > -1 && col < numcols)
            types[col] = text;
	}

    public void setLookAndFeel(String lookandfeel) {
        this.lookandfeel = lookandfeel;
        try {
            if (lookandfeel.equals("Motif"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
            else if (lookandfeel.equals("Windows"))
                UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
            else
                UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
            SwingUtilities.updateComponentTreeUI(this);
        } 
        catch (Exception e) {
        }

    }

    public String getLookAndFeel() {
        return lookandfeel;
    }
    
    public Dimension getMinimumSize() {
        return minimumsize;
    }

    public void setTableFont(Font font) {
        table.setFont(font);
    }

    public Font getTableFont() {
        return table.getFont();
    }

    public void setTableBackground(Color color) {
        table.setBackground(color);
    }

    public Color getTableBackground() {
        return table.getBackground();
    }

    public void setTableForeground(Color color) {
        table.setForeground(color);
    }

    public Color getTableForeground() {
        return table.getForeground();
    }

    public void setSelectionBackground(Color color) {
        table.setSelectionBackground(color);
    }

    public Color getSelectionBackground() {
        return table.getSelectionBackground();
    }

    public int[] getColWidths() {
        return widths;
	}

    public void setColWidths(int[] values) {
        widths = values;
		if (widths.length == numcols)
			updateWidths();
	}
    
    public int getColWidth() {
        if (col > -1 && col < numcols)
        return widths[col];
        return 0;
	}

    public void setColWidth(int value) {
        if (col > -1 && col < numcols) {
            widths[col] = value;
            TableColumn tc = table.getColumn(table.getColumnName(col));
            if (tc != null && widths[col] > 0) {
                tc.setWidth(widths[col]);
            }
        }
	}
    
    void updateWidths() {
		TableColumn tc;
		for (int i = widths.length-1; i > 0; i--) // first col is a rest
		{
			tc = table.getColumn(table.getColumnName(i));
			if (tc != null && widths[i] > 0)
				tc.setWidth(widths[i]);
		}
    }
	
	void updatesColHeadersTabl() {
		colnames.clear();
		for (int i = 0; i < headers.length; i++)
			colnames.put(headers[i], new Integer(i));
	}

	// table model interface implementation
    public int getRowCount() {
        return getNumRows();
    }
    public int getColumnCount() {
        return getNumCols();
    }
    public String getColumnName(int columnIndex) {
        try {
            return headers[columnIndex];
        } catch (ArrayIndexOutOfBoundsException iobe) {
            return EMPTYSTRING;
        } catch (NullPointerException npe) {
            return EMPTYSTRING;
        }
    }
    public Class getColumnClass(int columnIndex) {
        return String.class;
    }
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return false;
    }
    public Object getValueAt(int rowIndex, int columnIndex) {
        try {
            return contents[rowIndex*getNumCols()+columnIndex];
        } catch (ArrayIndexOutOfBoundsException iobe) {
            return EMPTYSTRING;
        } catch (NullPointerException npe) {
            return EMPTYSTRING;
        }
    }
    public void setValueAt(Object aValue,
                                 int rowIndex,
                                 int columnIndex) {
        try {
            if (aValue instanceof String)
                contents[rowIndex*getNumCols()+columnIndex] = (String)aValue;
            else if (aValue instanceof Integer)
                contents[rowIndex*getNumCols()+columnIndex] = String.valueOf((Integer)aValue);
            else
                contents[rowIndex*getNumCols()+columnIndex] = ""+aValue;

        } catch (ArrayIndexOutOfBoundsException iobe) {
            
        }
    }
    public void addTableModelListener(TableModelListener l) {
        // do not support
    }
    public void removeTableModelListener(TableModelListener l) {
        // do not support
    }

	// 	IRowProperties interface implementation
	
	public String[] getPropertyNames()
	{
		return getHeaders();
	}

	public void setPropertyNames(String[] names) {
		// oneway, only for bean compatability
	}

	public void setPropertyName(String text) {
		try {
		setCol(((Integer)colnames.get(text)).intValue());
		} catch (Throwable e) {
		}
	}
	
	public String getPropertyName() {
		// oneway, only for bean compatability
		return getHeader();
	}
	
	public void setPropertyValue(Object value) {
		setValueAt(value, getRow(), getCol());
	}

	public Object getPropertyValue() {
		return getValueAt(getRow(), getCol());
	}

	public void setNextRow(boolean on) {
		if (on) {
			if (getRow() >= (getNumRows()-1))
				setNumRows(getRow()+2);
			setRow(getRow()+1);
		}
		else
			setNumRows(getRow());
	}
	
	public boolean isNextRow()
	{
		return getRow() < (getNumRows() - 1);
	}

	private JTable table;
    private int numcols;
    private int numrows;
    private int col;
    private int row;
    private String lookandfeel;
    private String[] contents;
    private String[] headers;
    private String[] types;
    private int[]    widths;
    private final static String EMPTYSTRING = "";
    private Dimension minimumsize = new Dimension(10, 10);
	// for IRowProperties
	private Hashtable colnames;
}