/*
 * $RCSfile: ArchiveXML.java,v $
 * @author		Dmitry Rogatkin 
 * $Date: 2001/02/25 08:37:58 $:		02/19/98
 * $Version$:  1.0
 * $Revision: 1.3 $
 * $Log: ArchiveXML.java,v $
 * Revision 1.3  2001/02/25 08:37:58  dmitriy
 * moving to new XML standards
 *
 * Revision 1.2  2001/02/24 10:28:16  dmitriy
 * new XML parser fix
 *
 *        1. General idea 6/04/97   
 *        2. Added correct work with whitespace nodes.
 */
/**
 * $Description$:  Persistent/serialization in XML format 
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.core;

import java.io.*;
import java.lang.reflect.*;
import java.util.Enumeration;
import java.beans.*;
import java.awt.Rectangle;
import java.awt.Color;
import java.awt.Font;
import java.util.StringTokenizer;
import java.util.Hashtable;
import java.util.Stack;
import java.util.EmptyStackException;

import org.w3c.dom.Document;
//import org.w3c.dom.Node;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;

import org.xml.sax.Locator;
import org.xml.sax.AttributeList;
import org.xml.sax.SAXParseException;
import org.xml.sax.SAXException;
import org.xml.sax.HandlerBase;
import org.xml.sax.helpers.AttributeListImpl;
import javax.xml.parsers.*;

import rogatkin.xbox.shared.*;
import rogatkin.xbox.ut.INFO;
import rogatkin.xbox.ut.XMLToolFactory;

public class ArchiveXML extends HandlerBase {
    public static String DOM = System.getProperty(XMLToolFactory.DOM_DOCUMENT_PROP);
    
    public ArchiveXML() {
    }
    
    Element formHeader() {
	try {
	    document = XMLToolFactory.createDocument(DOM);
	    Element el;
	    el = document.createElement(KWForm);
	    el.setAttribute("Name", beanbox.getFormName());
	    el.setAttribute(KWVersion, "1.001");
	    document.appendChild(el);
	    return el;
	} catch(DOMException e) {
            e.printStackTrace();
	}
	return null;
    }
    
    void addControls(Element el, BasicSpace canvas){
	for (int i = 0; i < canvas.getComponentCount(); i++)
	    addControl(el, (Wrapper)canvas.getComponent(i), i);
    }
    
    void addControl(Element el, Wrapper element, int pos) {
	Element control;
	Element property;
	EventSetDescriptor esd[];
	PropertyDescriptor properties[];
	String ws;
	try {
	    BeanInfo bi = Introspector.getBeanInfo(element.getBean().getClass());
	    properties = bi.getPropertyDescriptors();
	    esd = bi.getEventSetDescriptors();
	    
	} catch (IntrospectionException ex) {
	    INFO.error(CLASSID+": Couldn't introspect "+ex);
	    return;
	}
	// control header
	control = document.createElement( KWControl);
	control.setAttribute(KWName, element.getDesignName());
	control.setAttribute(KWBean, element.getBeanName());
	control.setAttribute(KWZ, ""+pos);
        if (beanbox.getBackgroundBean() == element)
            control.setAttribute(KWBackground, ""+true);
	if (element.isKeepInPlace())
	    control.setAttribute(KWKeepInPlace, ""+true);
	control = (Element)el.appendChild(control).appendChild(document.createElement(KWBounds));
	// control bounds
	Rectangle r = element.getRealBounds();
	// X
	property =	document.createElement(KWX);
	control.appendChild(property);
	property.appendChild(document.createTextNode(String.valueOf(r.x)));
	// Y
	property =	document.createElement(KWY);
	control.appendChild(property);
	property.appendChild(document.createTextNode(String.valueOf(r.y)));
	// Width
	property =	document.createElement(KWWidth);
	control.appendChild(property);
	property.appendChild(document.createTextNode(String.valueOf(r.width)));
	// Height
	control.appendChild(document.createElement(KWHeight)).appendChild(document.createTextNode(String.valueOf(r.height)));
	
	// control properties
	control = (Element)control.getParentNode().appendChild(document.createElement(KWProperties));
	
	for (int i=0; i<properties.length; i++) {
	/**
	<PROPERTIES>
	<T_Y_P_E NAME="Label" VALUE="Control 1">
	    **/
	    Method getter = properties[i].getReadMethod();
	    
	    Object args[] = { };
	    try {
		String propType;
		String propValue;
		Object result = getter.invoke(element.getBean(), args);
		
		if (properties[i].getPropertyType() == String.class) {
		    propType = DTString;
		    propValue = (String)result;
		} else if (properties[i].getPropertyType() == int.class) {
		    propType = DTInt;
		    propValue = result.toString();
		} else if (properties[i].getPropertyType() == Color.class) {
		    propType = DTColor;
		    propValue = String.valueOf(((Color)result).getRGB());
		} else if (properties[i].getPropertyType() == Font.class) {
		    propType = DTFont;
		    propValue = result.toString();
		} else if (properties[i].getPropertyType() == boolean.class) {
		    propType = DTBool;
		    propValue = result.toString();
		} else if (properties[i].getPropertyType() == String[].class) {
		    propType = DTStringArr;
		    propValue = ((String[])result)[0];
		    for(int j=1;j<((String[])result).length;j++)
			propValue += DELIM+((String[])result)[j];
		} else if (properties[i].getPropertyType() == int[].class) {
		    propType = DTIntArr;
		    propValue = String.valueOf(((int[])result)[0]);
		    for(int j=1;j<((int[])result).length;j++)
			propValue += DELIM+((int[])result)[j];
		} else { 
		    propType = DTUnsupported;
		    propValue = result.toString();
		}
		property = document.createElement(propType);
		property.setAttribute(KWProperty, properties[i].getDisplayName());
		if (propType.equals(DTFont)) {
		    property.setAttribute(FAName, ((Font)result).getName());
		    property.setAttribute(FASize, String.valueOf(((Font)result).getSize()));
		    property.setAttribute(FAStyle, String.valueOf(((Font)result).getStyle()));
		}
		//				property.setAttribute(KWValue, propValue);
		if (propValue.charAt(0) != 0) {
		    property.appendChild(document.createTextNode(propValue));
		}
		control.appendChild(property);
	    } catch (Exception ex) {
		if (debug)
		    INFO.println("Skipping property " + properties[i].getDisplayName() + " : " + ex);
	    }
	}
	// control events
	createListenersList((Element)control.getParentNode().appendChild(document.createElement(KWListeners)), element);
	// dB binding
	createBindNode((Element)control.getParentNode().appendChild(document.createElement(KWBinds)), element, properties);
    }

    void createListenersList(Element el, Wrapper element) {
    /**
    <LISTENERS>
    <EVENT Name="eeee">
    <LISTENER Name="nnnn"/>
    <LISTENER Name="nnnn"/>
    </>
    </>
	**/
	Hashtable em = beanbox.getEventMap();
	Hashtable eventNames = (Hashtable)em.get(element.getDesignName());
	if (eventNames == null)
	    return; // no listeners at all
	Enumeration een = eventNames.keys();
	while (een.hasMoreElements()) {
	    String eventname = (String)een.nextElement();
	    Element eventelement = document.createElement(KWEvent);
	    eventelement.setAttribute(KWName, eventname);
	    el.appendChild(eventelement);
	    Hashtable listeners = (Hashtable)eventNames.get(eventname);
	    if (listeners == null)
		continue;
	    Enumeration len = listeners.keys();
	    while (len.hasMoreElements()) {
		String listenername = (String)len.nextElement();
		Element listenerelement = document.createElement(KWListener);
		listenerelement.setAttribute(KWName, listenername);
		eventelement.appendChild(listenerelement);
	    }
	}
    }

    void createBindNode(Element el, Wrapper element, PropertyDescriptor[] props) {
	// binding information could be provided together with property
	// description, but separated allowing remove it, to save
	// space if it is not needed
	/**
	<BINDS>
	<PROPERTY Name="xxxx">
	<FIELD>yyyy</>
	<TABLE>zzzz</>
	<DATABASE>@@@@</>
	</>
	</>
	**/
	if (props == null)
	    return;
	// bind standard prop item
	for (int i=0; i<props.length; i++) {
	    //param = addBindedElement(param, props[i].getDisplayName());
	    String[] triple = element.getBindItem(props[i].getDisplayName());
	    if (triple != null && triple.length == 3){
		Element bind = document.createElement(KWProperty);
		el.appendChild(bind);
		bind.setAttribute(KWName, props[i].getDisplayName());
		bind.appendChild(document.createElement(KWField)).appendChild(document.createTextNode(triple[0]));
		bind.appendChild(document.createElement(KWTable)).appendChild(document.createTextNode(triple[1]));
		bind.appendChild(document.createElement(KWDatabase)).appendChild(document.createTextNode(triple[2]));
	    }
	}
	if (Beans.isInstanceOf(element.getBean(), IRowProperties.class)) {
	    String[] addproperties = ((IRowProperties)element.getBean()).getPropertyNames();
	    for (int i = 0; i < addproperties.length; i++)
		addBindedElement(addproperties[i], element, el);
	}
    }
    
    void addBindedElement(String bindelement, Wrapper element, Element el) {
	String[] triple = element.getBindItem(bindelement);
	if (triple != null && triple.length == 3) {
	    Element bind = document.createElement(KWProperty);
	    el.appendChild(bind);
	    bind.setAttribute(KWName, bindelement);
	    bind.appendChild(document.createElement(KWField)).appendChild(document.createTextNode(triple[0]));
	    bind.appendChild(document.createElement(KWTable)).appendChild(document.createTextNode(triple[1]));
	    bind.appendChild(document.createElement(KWDatabase)).appendChild(document.createTextNode(triple[2]));
	}
    }
    
    public void save(BasicSpace canvas, FileOutputStream stream) throws java.io.IOException {
	beanbox = canvas;
	addControls(formHeader(), canvas);
	XMLToolFactory.saveDocument(document, stream);
	document = null; 
    }
    
    public void load(BasicSpace canvas, InputStream in) throws java.io.IOException {
        beanbox = canvas;
	try {
			SAXParserFactory parserFactory = SAXParserFactory.newInstance();
			parserFactory.setValidating(true);
			parserFactory.newSAXParser().parse(in, this);
	} catch (Exception e) {
	    INFO.error("Load "+e);
            if(e instanceof SAXException && ((SAXException)e).getException() != null)
	        ((SAXException)e).getException().printStackTrace();
	    else
		e.printStackTrace();
	    throw new java.io.IOException(""+e);
	}
    }
    
    // interface DocumentHandler
    public void setDocumentLocator(Locator locator) {
	this.locator = locator;
    }
    
    public void startDocument() throws SAXException {
	parsingstack = new Stack();
	parsingstack.push(new ParsingStatusHelper());
	
    }
    
    public void endDocument() throws SAXException {
	parsingstack.pop();
	if (!parsingstack.empty())
	    throw new SAXException("Unexpected end of the XML document");
	activateListeners();
    }
    
    /*
    * WARNING: this method includes many extra checking. DTD allows avoid that.
    */
    public void startElement(String name, AttributeList atts)
	throws SAXException {
	ParsingStatusHelper ph = (ParsingStatusHelper)parsingstack.peek();
	if (ph.getState() == ParsingStatusHelper.PROPERTIES) {
	    parsingstack.push(new ParsingStatusHelper(name, ParsingStatusHelper.PROPERTY, atts));
	    return;
	} else if (ph.getState() == ParsingStatusHelper.BOUNDS || ph.getState() == ParsingStatusHelper.BINDPROP) {
	    parsingstack.push(new ParsingStatusHelper(name));
	}
	if (name.equals(KWControl)) {
	    if (ph.getState() != ParsingStatusHelper.FORM)
		throw new SAXException("An attempt add component to wrong place at "+locator);
	    parsingstack.push(new ParsingStatusHelper(name, restoreComponent(atts)));
	} else if (name.equals(KWProperties)) {
	    if (ph.getState() != ParsingStatusHelper.COMP)
		throw new SAXException("An attempt to set properties not for component at "+locator);
	    PropertyDescriptor[] pds;
	    try {
		BeanInfo bi = Introspector.getBeanInfo(ph.getWrapper().getBean().getClass());
		pds = bi.getPropertyDescriptors();
	    } catch (IntrospectionException e) {
		INFO.error("Introspection: "+e);
		pds = null;
	    } catch (NullPointerException e) {
		pds = null;
	    }
	    parsingstack.push(new ParsingStatusHelper(name, pds, ph.getWrapper()));
	    
	} else if (name.equals(KWBounds)) {
	    parsingstack.push(new ParsingStatusHelper(name, ParsingStatusHelper.BOUNDS, ph.getWrapper()));
	} else if (name.equals(KWListeners)) {
	    parsingstack.push(new ParsingStatusHelper(name, ParsingStatusHelper.LISTENERS, ph.getWrapper()));
	} else if (name.equals(KWEvent)) {
	    if (ph.getState() == ParsingStatusHelper.LISTENERS) 
		parsingstack.push(new ParsingStatusHelper(name, ParsingStatusHelper.EVENT, atts, ph.getWrapper()));
	    else
		throw new SAXException("Event element isn't allowed at "+locator);
	} else if (name.equals(KWListener)) {
	    if (ph.getState() == ParsingStatusHelper.EVENT)
		parsingstack.push(new ParsingStatusHelper(name, ParsingStatusHelper.LISTENER, atts));
	    else
		throw new SAXException("Listener element isn't allowed at "+locator);
	} else if (name.equals(KWProperty)) {
	    if (ph.getState() == ParsingStatusHelper.BINDS)
		parsingstack.push(new ParsingStatusHelper(name, ParsingStatusHelper.BINDPROP, atts, ph.getWrapper()));
	    else
		throw new SAXException("Property element isn't allowed at "+locator);
	} else if (name.equals(KWBinds)) {
	    parsingstack.push(new ParsingStatusHelper(name, ParsingStatusHelper.BINDS, ph.getWrapper()));
	} else if (name.equals(KWForm)) {
	    parsingstack.push(
		new ParsingStatusHelper(name, ParsingStatusHelper.FORM, atts));
	}
    }
    
    public void endElement(String name) throws SAXException {
	try {
	    ParsingStatusHelper sh = (ParsingStatusHelper)parsingstack.pop();
	    if (!name.equals(sh.getName()))
		throw new SAXException("Start name "+sh.getName()+" doesn't correspond end name "+name+" at "+locator);
	    switch (sh.getState()) {
	    case ParsingStatusHelper.PROPERTY:
		if (!DTUnsupported.equals(name))
		    setProperty(sh.getData(), sh.getAttributes(), 
		    ((ParsingStatusHelper)parsingstack.peek()).getPropertyDescriptors(),
		    ((ParsingStatusHelper)parsingstack.peek()).getWrapper());
		break;
	    case ParsingStatusHelper.DATA:
		if (((ParsingStatusHelper)parsingstack.peek()).getState() == ParsingStatusHelper.BINDPROP) {
		    if (name.equalsIgnoreCase(KWField))
			((ParsingStatusHelper)parsingstack.peek()).setTriple(sh.getData(), 0);
		    else if (name.equalsIgnoreCase(KWTable))
			((ParsingStatusHelper)parsingstack.peek()).setTriple(sh.getData(), 1);
		    else if (name.equalsIgnoreCase(KWDatabase))
			((ParsingStatusHelper)parsingstack.peek()).setTriple(sh.getData(), 2);
		    
		} else {
		    if (name.equalsIgnoreCase(KWX))
			((ParsingStatusHelper)parsingstack.peek()).setX(
			Integer.parseInt(sh.getData()));
		    else if (name.equalsIgnoreCase(KWY))
			((ParsingStatusHelper)parsingstack.peek()).setY(
			Integer.parseInt(sh.getData()));
		    else if (name.equalsIgnoreCase(KWWidth))
			((ParsingStatusHelper)parsingstack.peek()).setWidth(
			Integer.parseInt(sh.getData()));
		    else if (name.equalsIgnoreCase(KWHeight))
			((ParsingStatusHelper)parsingstack.peek()).setHeight(
			Integer.parseInt(sh.getData()));
		}
		break;
	    case ParsingStatusHelper.BOUNDS:
		try {
                    ((ParsingStatusHelper)parsingstack.peek()).getWrapper().setRealBounds(sh.getRectangle());
		} catch(NullPointerException e) {
		}
		break;
	    case ParsingStatusHelper.COMP:
		sh.getWrapper().doLayout();
		break;
	    case ParsingStatusHelper.LISTENER:
		((ParsingStatusHelper)parsingstack.peek()).getWrapper().addListenerFor(
		    ((ParsingStatusHelper)parsingstack.peek()).getAttributes().getValue(KWName),
		    sh.getAttributes().getValue(KWName));
		break;
	    case ParsingStatusHelper.BINDPROP:
		sh.getWrapper().addBindItem(sh.getAttributes().getValue(KWName), sh.getTriple(0), sh.getTriple(1), sh.getTriple(2));
		break;
	    case ParsingStatusHelper.FORM:
		beanbox.setFormName(sh.getAttributes().getValue(KWName));
		//beanbox.updateCaption();
		break;
	    }
	} catch(EmptyStackException e) {
	    throw new SAXException("The document inconsistent, extra end node "+name+" at "+locator);
	}
	
	
    }
    
    public void characters(char[] ch, int start, int length)
	throws SAXException {
	((ParsingStatusHelper)parsingstack.peek()).appendData(new String(ch, start, length));
    }
    
    public void ignorableWhitespace(char[] ch, int start, int length)
	throws SAXException {
    }
    
    public void processingInstruction(String target, String data)
	throws SAXException {
	INFO.println("Processing instr for "+target+" with "+data);
    }
    
    // Interface ErrorHandler
    public void warning(SAXParseException exception) throws SAXException
    {
        INFO.println("l: "+exception.getLineNumber()+", c: "+exception.getColumnNumber()+" "+exception);
    }
    
    public void error(SAXParseException exception) throws SAXException
    {
        INFO.error("l: "+exception.getLineNumber()+", c: "+exception.getColumnNumber()+" "+exception);
    }
    
    public void fatalError(SAXParseException exception) throws SAXException
    {
        INFO.error("l: "+exception.getLineNumber()+", c: "+exception.getColumnNumber()+" "+exception.getException());
        throw exception;
    }
    
    Wrapper restoreComponent(AttributeList attributes) {
	Wrapper wrapper;
	boolean keepinplace = attributes.getValue(KWKeepInPlace) != null && attributes.getValue(KWKeepInPlace).equalsIgnoreCase(""+true);
        int pos = 0; 
        try {
            pos = Integer.parseInt(attributes.getValue(KWZ));
        } catch(Exception e) {
            // use default
        }
	if (keepinplace) 
	{
	    wrapper = beanbox.findWrapperWithName(attributes.getValue(KWName));
	    if (wrapper != null)
		return wrapper; // already in place
	}
	wrapper = (Wrapper) beanbox.
                                    doInsertWithNameIfPresent(attributes.getValue(KWBean), attributes.getValue(KWName), pos);
	if (wrapper == null)
	    return null;
	wrapper.setKeepInPlace(keepinplace);
	if (attributes.getValue(KWBackground) != null && attributes.getValue(KWBackground).toString().equalsIgnoreCase(""+true))
	    beanbox.setBackgroundBean(wrapper);
	return wrapper;
    }
    
    void setProperty(String value, AttributeList atts, PropertyDescriptor[] pds, Wrapper wrapper) {
	String propName = atts.getValue(KWProperty);
	if (propName == null) 
	    return;
	PropertyDescriptor pd = BasicSpace.findPropWName(pds, propName);
	if (pd == null)
	    return;
	Method setter = pd.getWriteMethod();
	Object args[] = { };
	if (pd.getPropertyType() == String.class)
	{
	    args = new String[1];
	    args[0] = value;
	} else if (pd.getPropertyType() == int.class) {
	    args = new Integer[1];
	    args[0] = new Integer(value);
	} else if (pd.getPropertyType() == boolean.class) {
	    args = new Boolean[1];
	    args[0] = new Boolean (value);
	} else if (pd.getPropertyType() == Color.class) {
	    args = new Color[1];
	    args[0] = new Color(Integer.parseInt(value));
	} else if (pd.getPropertyType() == Font.class) {
	    args = new Font[1];
	    args[0] = new Font(atts.getValue(FAName),
		Integer.parseInt(atts.getValue(FAStyle)),
		Integer.parseInt(atts.getValue(FASize)));
	} else if (pd.getPropertyType() == String[].class) {
	    StringTokenizer arrstr = new StringTokenizer(value, DELIM, false);
	    args = new Object[1];
	    args[0]	= new String[arrstr.countTokens()];
	    for (int k=0;k<((String[])args[0]).length;k++)
		((String[])args[0])[k] = arrstr.nextToken();
	} else if (pd.getPropertyType() == int[].class) {
	    StringTokenizer arrstr = new StringTokenizer(value, DELIM, false);
	    args = new Object[1];
	    args[0]	= new int[arrstr.countTokens()];
	    for (int k=0;k<((int[])args[0]).length;k++)
	    {
		try {
		    ((int[])args[0])[k] = Integer.parseInt(arrstr.nextToken());
		} catch(NumberFormatException e) {
		    ((int[])args[0])[k] = 0;
		}
	    }
	} else
	    INFO.error("An error in an attempt to set property type "+pd.getPropertyType().getName());
	try {
	    Object result = setter.invoke(wrapper.getBean(), args);
	} catch (Exception e) {
	    INFO.error("Setter err for "+pd.getDisplayName()+" "+e+" "+args[0]);
	}
    }
    
    public void saveComponent(Wrapper element, FileOutputStream stream) throws java.io.IOException {
	addControl(formHeader(), element, 0);
	XMLToolFactory.saveDocument(document, stream);
	document = null;
    }
    
    void close() {
	beanbox = null;
    }
    
    void activateListeners() {
	Hashtable em = beanbox.getEventMap();
	if (em == null)
	    return; // nothing to do
	Enumeration men = em.keys();
	
	while (men.hasMoreElements()) {
	    EventSetDescriptor []esd;
	    String eventsrc = (String)men.nextElement();
	    Hashtable eventNames = (Hashtable)em.get(eventsrc);
	    if (eventNames == null)
		continue; // no listeners at all
	    Wrapper w = beanbox.findWrapperWithName(eventsrc);
	    if (w == null)
		continue;
	    Object bean = w.getBean();
	    try {
		BeanInfo bi = Introspector.getBeanInfo(bean.getClass());
		esd = bi.getEventSetDescriptors();
		
	    } catch (IntrospectionException ex) {
		INFO.error("XML restore: Couldn't introspect "+ex);
		continue;
	    }
	    
	    Enumeration een = eventNames.keys();
	    Element prev = null;
	    while (een.hasMoreElements()) {
		String eventname = (String)een.nextElement();
		Method m = null;
		for (int i = 0; i < esd.length; i++) {
		    if (eventname.equals(esd[i].getDisplayName())){
			m = esd[i].getAddListenerMethod();
			break;
		    }
		}
		if (m == null)
		    continue;
		Hashtable listeners = (Hashtable)eventNames.get(eventname);
		if (listeners == null)
		    continue;
		Enumeration len = listeners.keys();
		while (len.hasMoreElements()) {
		    String listenername = (String)len.nextElement();
		    Object arg[] = {beanbox.findWrapperWithName(listenername).getBean()};
		    // good approach if we cache somewhere pointers to beans
		    try {
			m.invoke(bean, arg);
		    } catch(IllegalAccessException e){
			INFO.error("XML add lstnrs: "+e);
		    } catch(IllegalArgumentException e){
			INFO.error("XML add lstnrs: "+e);
		    } catch(InvocationTargetException e){
			INFO.error("XML add lstnrs: "+e);
		    }
		}
	    }
	}
    }
    
    private Document document;
    private BasicSpace beanbox = null;
    private Locator locator;
    private Stack  parsingstack;
    
    private final static String CLASSID = "ArchiveXML";
    private final static String KWXML = "xml";
    private final static String KWVersion = "version";
    private final static String KWControl = "Control";
    private final static String KWName = "Name";
    private final static String KWBean = "Bean";
    private final static String KWBounds = "Bounds";
    private final static String KWX = "X";
    private final static String KWY = "Y";
    private final static String KWZ = "Z";
    private final static String KWWidth = "Width";
    private final static String KWHeight = "Height";
    private final static String KWProperties = "Properties";
    private final static String KWListeners = "Listeners";
    private final static String KWListener = "Listener";
    private final static String KWEvent = "Event";
    private final static String KWBinds = "Binds";
    private final static String KWForm = "Form";
    private final static String KWType = "Type";
    private final static String KWProperty = "Property";
    private final static String KWValue = "Value";
    private final static String KWField = "FIELD";
    private final static String KWTable = "TABLE";
    private final static String KWDatabase = "DATABASE";
    private final static String KWBackground = "BACKGROUND";
    private final static String KWKeepInPlace = "KEEPINPLACE";
    private final static String KWOrder = "ORDER";
    private final static String DELIM = "#";
    // supported data types 
    private final static String DTString = "STRING";
    private final static String DTStringArr = "ARRAYOFSTRINGS";
    private final static String DTInt = "INTEGER";
    private final static String DTIntArr = "ARRAYOFINTEGERS";
    private final static String DTBool = "BOOLEAN";
    private final static String DTFont = "FONT";
    private final static String DTColor = "COLOR";
    private final static String DTDimension = "DIMENSION";
    private final static String DTUnsupported = "UNSUPPORTED";
    // Font attributes
    private final static String FAName = "Name";
    private final static String FASize = "Size";
    private final static String FAStyle = "Style";
    private static boolean debug = false;
}

class ParsingStatusHelper {
    final static int DOC = 0;
    final static int FORM = 1;
    final static int COMP = 2;
    final static int PROPERTIES = 3;
    final static int PROPERTY = 4;
    final static int BOUNDS = 5;
    final static int DATA = 6;
    final static int LISTENERS = 7;
    final static int BINDS = 8;
    final static int LISTENER = 9;
    final static int EVENT = 10;
    final static int BINDPROP = 11;
    
    public ParsingStatusHelper() {
        this.state = DOC;
    }
    
    public ParsingStatusHelper(String name, Wrapper wrapper) {
        this(name);
        this.state = COMP;
        this.wrapper = wrapper;
    }
    
    public ParsingStatusHelper(String name, int state, Wrapper wrapper) {
        this(name);
        this.state = state;
        this.wrapper = wrapper;
    }
    
    public ParsingStatusHelper(String name, PropertyDescriptor[] pds, Wrapper wrapper) {
        this(name, wrapper);
        this.state = PROPERTIES;
        this.pds = pds;
    }
    
    public ParsingStatusHelper(String name, AttributeList attributes) {
        this(name);
        this.state = DATA;
        this.attributes = new AttributeListImpl(attributes);
    }
    
    public ParsingStatusHelper(String name, int state, AttributeList attributes) {
        this(name, attributes);
        this.state = state;
    }
    
    public ParsingStatusHelper(String name, int state, AttributeList attributes, Wrapper wrapper) {
        this(name, attributes);
        this.wrapper = wrapper;
        this.state = state;
    }
    
    public ParsingStatusHelper(String name) {
        this.name = name;
        state = DATA;
        data = new StringBuffer();
        rect = new Rectangle();
        triple = new String[3];
    }
    
    public String getName() {
        return name;
    }
    
    public String getData() {
        return data.toString();
    }
    
    public int getState() {
        return state;
    }
    
    public void appendData(String value) {
        data.append(value);
    }
    
    public PropertyDescriptor[] getPropertyDescriptors() {
        return pds;
    }
    
    public AttributeList getAttributes() {
        return attributes;
    }
    
    public Wrapper getWrapper() {
        return wrapper;
    }
    
    public Rectangle getRectangle() {
        return rect;
    }
    
    public void setX(int x) {
        rect.x = x;
    }
    
    public void setY(int y) {
        rect.y = y;
    }
    
    public void setWidth(int w) {
        rect.width = w;
    }
    
    public void setHeight(int h) {
        rect.height = h;
    }
    
    public void setTriple(String value, int index) {
        triple[index] = value;
    }
    
    public String getTriple(int index) {
        return triple[index];
    }
    
    private String name;
    private StringBuffer data;
    private int state;
    private AttributeList attributes;
    private PropertyDescriptor[] pds;
    private Wrapper wrapper;
    private Rectangle rect;
    private String [] triple;
}
