/*
 * File:		BasicSpace.java
 * Author:		Dmitry Rogatkin
 * Date:		10/06/98
 *
 * Description:	BasicSpace
 *
 * Copyright (C) 1998 Dmitry Rogatkin .  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */

package rogatkin.xbox.core;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.beans.*;
import java.net.*;
import java.applet.*;

import rogatkin.xbox.shared.*;
import rogatkin.xbox.ui.*;
import rogatkin.xbox.io.*;
import rogatkin.xbox.ut.*;

public class BasicSpace implements Serializable, IFCEvListener {

    final static long serialVersionUID = -1930952830824256717L;
    
    // Constants
    
    public final static String untitledName = "rfuntitled";
    public final static String defaultXMLExt = ".xml";

    public final static String BUILD = ".110";
    public final static String BASEVERSION = "1.0.1";
    public final static String VERSIONID = BASEVERSION+BUILD;
    
    private transient RangeredMap formbeans = new RangeredMap();
    // Here we defined two hash tables including information about binding and
    // events. If to follow rules of good design, it has to be one hash table.
    // 
    /* 
    event source controls (hashtable)
		  event type (hash table)
		  event listener controls (hash table)
    */
    private transient Hashtable eventMap; // event mappimg
    private String formname = new String(untitledName); // untitled
    private String FormDirectory;
    Container container;
    private ArchiveXML archive;
    private Frame frame = null;
    
    transient Wrapper backgroundbean;
    private transient CompManager ctrlManager;
    
    public BasicSpace(Container container) {
	archive = new ArchiveXML();
	this.container = container;
	eventMap = null;
	ctrlManager = null;
	container.setLayout(null);
	try {
	    FormDirectory = System.getProperty("user.dir");
	} catch (SecurityException se) {
	    FormDirectory = "";
	} catch (Throwable t) {
	    FormDirectory = "";
	}
    }

    public void init(CompManager compmanager) {
	ctrlManager = compmanager;
    }
    
    public CompManager getCompManager() {
	return ctrlManager;
    }
	
    //-----------------------------------------------
    // Component/Container style APIs
    //-----------------------------------------------
    public Container getContainer() {
        return container;
    }
    
    public void validate() {
	detatchAllBeans();
	Enumeration beans = formbeans.order();
	for (int i = 0; beans.hasMoreElements(); i++)
	    container.add((Component)beans.nextElement(), i);
    }
    // Add a bean to the end of the bean list but before the
    // background components.
    public Component add(Component comp) {
	return add(comp, container.getComponentCount() - 1);
    }
	
    // Insert the bean at the specified location.
    // If pos is larger than the size of the bean list,
    // the new bean is added at the end of the bean list.
    // Still it is in front of the background components.
    public Component add(Component comp, int pos) {
	formbeans.put(((Wrapper)comp).getDesignName(), comp);
	return container.add(comp, pos);
    }
    
    // Remove a bean.
    public void remove(Component comp) {
	if (formbeans.remove(((Wrapper)comp).getDesignName()) == comp)
	    container.remove(comp);
    }
    
    // Remove all the bean components.  But this leaves
    // the background components.
    public void removeAll() {
	detatchAllBeans();
	formbeans.clear();
    }
    
    public void detatchAllBeans() {
	Enumeration beans = formbeans.elements();
	while (beans.hasMoreElements())
	    container.remove((Component)beans.nextElement());
    }
    
    public int getComponentCount() {
	return container.getComponentCount();
    }
    
    public Component getComponent(int index) {
	return (Component) container.getComponent(index);
    }
	
    public String generateUniqueNameBasedOn(String name) {
        if (isNamePresent(name))
        {
            for (int i = 0; i < 1000; i++)
                if (!isNamePresent(name+i))
		return new String(name+i);
        }
        return name;
    }
    
    public boolean isNamePresent(String name) {
        return formbeans.containsKey(name);
    }

    public Object findBeanWithNameAndClass(String name, Class _class) {
        Wrapper w = (Wrapper)formbeans.get(name);
        if (w != null)
            if (Beans.isInstanceOf(w.getBean(), _class))
	    return w.getBean();
	return null;
    }
    
    public Wrapper findWrapperWithName(String name){
        return (Wrapper)formbeans.get(name);
    }

    public Wrapper doInsertWithNameIfPresent(String beanName, String designName, int pos) {
	Object bean = ctrlManager.getBeanByName(beanName);
	if (bean != null)
	    return doInsertNoPlace(bean, beanName, designName, pos);
	else
	    return null;
    }
    
    private Wrapper doInsertNoPlace(Object bean, String beanName, String designName) {
            return doInsertNoPlace(bean, beanName, designName, 0);
    }
    
    private Wrapper doInsertNoPlace(Object bean, String beanName, String designName, int pos) {
	Wrapper child = new Wrapper(this, bean, beanName);
        child.setDesignName(generateUniqueNameBasedOn(designName));
	// Insert and reshape the child component to the top.
	add(child, pos);
	if (Beans.isInstanceOf(bean, Applet.class)) {
	    Applet apl = (Applet) Beans.getInstanceOf(bean, Applet.class);
	    apl.start();
	}
	return child;
    }
    
    static public PropertyDescriptor findPropWName(PropertyDescriptor[] descriptors, String name){
	for (int i = 0; i<descriptors.length; i++){
	    //if (descriptors[i].getDisplayName().equalsIgnoreCase(name))
	    //	System.err.println("!!!: name '"+name+"' prop display name '"+descriptors[i].getDisplayName()+
	    //	"' prop name '"+descriptors[i].getName()+"'");
	    // work around applet prop descriptor bug
	    String propName = descriptors[i].getDisplayName();
	    if (propName.regionMatches(true, 0, name, 0, 1) &&
		//propName.substring(0,1).equalsIgnoreCase(name.substring(0,1)) &&
		propName.substring(1).equals(name.substring(1)))
		//if (descriptors[i].getDisplayName().equals/*IgnoreCase*/(name))
		return descriptors[i];
	}
	return null;
    }

    public Hashtable getEventMap() {
	if (eventMap == null)
	    eventMap = new Hashtable();
	return eventMap;
    }
    
    // iterators: for some reason, when we rename or delete existing control
    // we should detach it from all structures, or change information about it
    public void enumerateListeners(IControlEnvCallBack callback) {
	if (eventMap == null)
	    return;
	Enumeration ce = eventMap.keys(); // for all controls
	while (ce.hasMoreElements()) {
	    String es = (String)ce.nextElement();
	    Hashtable et = (Hashtable)eventMap.get(es); // for all events of the control
	    Enumeration ee = et.keys();
	    while (ee.hasMoreElements()) {
		String en = (String)ee.nextElement();
		Hashtable lt = (Hashtable)et.get(en); // for all lesteners of the event
		Enumeration le = lt.keys();
		while (le.hasMoreElements()) {
		    String el = (String)le.nextElement();
		    callback.changeListener(es, en, el);
		}
	    }
	}
    }
    
    public String getFormName(){
	return formname;
    }
    
    public void setFormName(String newName){
	formname = new String(newName);
    }
    
    public boolean bringBeanToForward(Wrapper wb) {
        int ci = findComponentIndex(wb);
        if (ci == 0)
            return false;
        container.remove(wb);
        container.add(wb, ci-1);
        wb.repaint();
        return true;
    }
    
    public boolean bringBeanToBackward(Wrapper wb) {
        int ci = findComponentIndex(wb);
        if (ci == container.getComponentCount()-1)
            return false;
        container.remove(wb);
        container.add(wb, ci+1);
        wb.repaint();
        return true;
    }
    
    private int findComponentIndex(Component c) {
        for (int i=0; i<container.getComponentCount(); i++)
            if (c == container.getComponent(i))
                return i;
        return -1;
    }
    
    // background bean manipulation
    public void setBackgroundBean(Wrapper bkgbeanwrapper)
    {
	backgroundbean = bkgbeanwrapper;
	if (backgroundbean != null)
	{
	    container.remove(backgroundbean);
	    container.add(backgroundbean, container.getComponentCount());
	    Dimension d = container.getSize();
	    fitBackgroundBean(d.width, d.height);
	}
    }
    
    public Wrapper getBackgroundBean() {
	return backgroundbean;
    }
    
    // Attantion: base class implementation dependent code
    void fitBackgroundBean(int w, int h) {
        if (backgroundbean != null) {
            Insets is = container.getInsets();
            backgroundbean.setBounds(is.top, is.left, w - is.left - is.right,
                h - is.top - is.bottom);
        }
    }

    // Frame support
    public synchronized Frame getFrame() {
	Component c = container;
	while (c != null) {
	    if (c instanceof Frame) {
		return (Frame) c;
	    }
	    c = c.getParent();
	}
	// This should never happen
	if (frame != null)
	    return frame;
	throw new Error("Couldn't find frame ?!?");
    }
    
    public void setFrame(Frame frame) {
	this.frame = frame;
    }
    
    /** IFCEvListener interface
    */
    public void Command (FCEvent e) {
	switch(e.getIDCommand())
	{
	}
    }
    
    //====================================================
    // implements saveXML() of WorkSpace
    //====================================================
    public void saveXML(File itsFile)
    {
	try {
	    extractFormNameFromFile(itsFile);
	    FileOutputStream f = new FileOutputStream(itsFile);
	    archive.save(this, f);
            f.close();
	    //            updateCaption();
	}
	catch (Exception ex) {
	    System.err.println(Constants.ERRID+" caught "+ex+" while trying to save XML file "+itsFile.getName());
	    ex.printStackTrace();
	    return;
	}
    }
    
    void extractFormNameFromFile(File file) {
	String path = file.getAbsolutePath();
	int pd = path.lastIndexOf(File.separatorChar);
	if (pd > -1)
	    setFormDirectory(path.substring(0, pd));
	else
	    setFormDirectory("");
	String fname = file.getName();
	if (fname.lastIndexOf(".") > -1)
	    setFormName(fname.substring(0, fname.lastIndexOf(".")));
	else {
	    setFormName(fname);
	}
    }
    
    public void setFormDirectory(String fd){
	FormDirectory = fd;
    }
    
    public String getFormDirectory(){
	return FormDirectory;
    }
    
    //====================================================
    // implements loadXML() of WorkSpace
    //====================================================
    public void loadXML(URL url) {
	if (url == null)
	    return;
        try {
	    setFormName(url.getFile());
            archive.load(this, url.openStream());
        }
	catch (Exception ex) {
	    System.err.println(Constants.ERRID+" caught "+ex+" while trying to load XML URL "+url);
	    ex.printStackTrace();
	    return;
	} 
    }
    
    public void loadXML(File file)
    {
	if (file == null)
	    return;
        try {
	    extractFormNameFromFile(file);
            FileInputStream f = new FileInputStream(file);
            archive.load(this, f);
        }
	catch (Exception ex) {
	    System.err.println(Constants.ERRID+" caught "+ex+" while trying to load XML file "+file.getName());
	    ex.printStackTrace();
	    return;
	}
    }
    
    // Support for serialization
    
    public static String getVersionID() {
	return VERSIONID;
    }
    
    Object writeReplace() throws ObjectStreamException {
	return "xml string here";
    }
    
    //Object readResolve() throws ObjectStreamException {
    //}
    
    private void writeObject(java.io.ObjectOutputStream oos)
	throws java.io.IOException {
	writeContents(oos);
    }
    
    private void readObject(java.io.ObjectInputStream ois)
	throws java.lang.ClassNotFoundException,
	java.io.IOException {
	readContents(ois);
    }
    
    void writeContents(java.io.ObjectOutputStream s)
	throws java.io.IOException {
	int count = getComponentCount();
	
	// We unhook all the event wiring from the beans while
	// they are being written out.
        for (int i=0; i<count; i++) {
	    Wrapper w = (Wrapper) getComponent(i);
	    w.removeListeners();
	}
	
	// Now actually write out all the wrappers.
	s.writeInt(count);
	for (int i = 0; i < count; i++) {
	    Wrapper w = (Wrapper) getComponent(i);
	    s.writeObject(w);
	}
	
	// Now reattach all the event wiring.
	for (int i=0; i<count; i++) {
	    Wrapper w = (Wrapper) getComponent(i);
	    w.attachListeners();	
	}
    }
    
    void readContents(java.io.ObjectInputStream ois)
	throws java.lang.ClassNotFoundException,
	java.io.IOException {
	
	removeAll();
	
	ClassLoader cl = SimpleClassLoader.ourLoader;
	
	int count = ois.readInt();
	
	for (int i=0; i<count; i++) {
	    Component c = (Component) ois.readObject();
	    Wrapper w = (Wrapper) c;
	    
	    Object bean = w.getBean();
	    if (bean instanceof Applet) {
		// Assign an AppletStub
		assignAppletStub((Applet) bean, cl, bean.getClass());
	    }
	    
	    // Add the new component to our AWT container at the top.
	    add(c, 0);
	    c.invalidate();
	    
	    if (bean instanceof Applet) {
		// start the Applet.
		((Applet)bean).start();
	    }
	}
	
	// We need to wait until the entire graph has been read back in
	// before we can redo the wiring.
	for (int i=0; i<count; i++) {
	    Wrapper w = (Wrapper) getComponent(i);
	    w.attachListeners();	
	}
    }

	// Guess a codebase for a class in a classloader
    private void assignAppletStub(Applet applet, ClassLoader cl, Class k) {
	String className = k.getName();
	String resourceName = className.replace('.','/').concat(".class");
	
	// This code is identical to portions of Beans.intantiate()
	URL objectUrl = null;
	URL codeBase = null;
	URL docBase = null;
	
	// Now get the URL correponding to the resource name.
	if (cl == null) {
	    objectUrl = ClassLoader.getSystemResource(resourceName);
	} else {
	    objectUrl = cl.getResource(resourceName);
	}
	
	// If we found a URL, we try to locate the docbase by taking
	// of the final path name component, and the code base by taking
	// of the complete resourceName.
	// So if we had a resourceName of "a/b/c.class" and we got an
	// objectURL of "file://bert/classes/a/b/c.class" then we would
	// want to set the codebase to "file://bert/classes/" and the
	// docbase to "file://bert/classes/a/b/"
	
	if (objectUrl != null) {
	    String s = objectUrl.toExternalForm();
	    if (s.endsWith(resourceName)) {
		try{ 
		    int ix = s.length() - resourceName.length();
		    codeBase = new URL(s.substring(0,ix));
		    docBase = codeBase;
		    ix = s.lastIndexOf('/');
		    if (ix >= 0) {
			docBase = new URL(s.substring(0,ix+1));
		    }
		} catch (Exception ex) {
		    docBase = codeBase = null;
		}
	    }
	}
	
	// Setup a default context and stub.
	BeanBoxAppletContext context = new BeanBoxAppletContext(applet);
	BeanBoxAppletStub stub = new BeanBoxAppletStub(applet, context, codeBase, docBase);
	applet.setStub(stub);
	
	// object is always deserialized
	stub.active = true;
    }

}