/*
 * File:	CompManager.java
 * Author:	Dmitry Rogatkin 
 * Date:	03/5/98
 *
 * Description:	high level access to beans repository 
 *				
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.core;

import java.util.*;
import java.beans.*;
import java.awt.*;
import java.io.*;
import java.net.*;

import rogatkin.xbox.shared.*;
import rogatkin.xbox.io.*;
import rogatkin.xbox.ui.Starter;

public class CompManager {

    private Helper helper = new Helper();

    Vector beanLabels = new Vector();
    Vector beanNames = new Vector();
    Vector beanIcons = new Vector();
    Vector beanJars = new Vector();

    // keep it with package scope for the inner class' sake
    void addWithUniqueName(Vector v, String s) {
        if (v.contains(s)) {
            int count = 2;
            while (v.contains(s+"<"+count+">")) {
                count += 1;
            }
            s += "<"+count+">";
        }
        //Log(ERRID, s);
        v.addElement(s);
        return;
    }

    private class Helper implements DoOnBean {
        public void action(JarInfo ji, BeanInfo bi, String beanName) {
            String label;
            Image img = null;

            BeanDescriptor bd = bi.getBeanDescriptor();

            Class beanClass = bd.getBeanClass();
            if (beanName.equals(beanClass.getName())) {
                label = bi.getBeanDescriptor().getDisplayName();
                img = bi.getIcon(BeanInfo.ICON_COLOR_16x16);
            } else {
                label = beanName;
                int ix = beanName.lastIndexOf('.');
                if (ix >= 0) {
                    label = beanName.substring(ix+1);
                }
                img = null;
            }
            addWithUniqueName(beanLabels, label);
            beanNames.addElement(beanName);
            beanIcons.addElement(img);
            beanJars.addElement(ji);
        }
    }    

    public void buildControlList(Starter starter, String jarsdirArg, URL baseUrl) {
        // jarsdirArg can be as directories list, as jar URLs
	StringTokenizer jdt = new StringTokenizer(jarsdirArg, ";");
        if (jdt == null)
            return;
        String [] directories = new String[jdt.countTokens()];
        for(int i = 0; jdt.hasMoreTokens(); i++)
            directories[i] = jdt.nextToken();
        Vector jarNames = getJarNames(directories, baseUrl);
	if (starter != null)
	    starter.setRange(jarNames.size());
        for (int i = 0; i < jarNames.size(); i++) {
	    Object jarLoc = jarNames.elementAt(i);
            if (jarLoc instanceof URL)
		addBeansInJar((URL)jarLoc);
	    else if (jarLoc instanceof String)
		addBeansInJar((String)jarLoc);
	    if (starter != null)
		starter.setPos(i);
        }

    }
    public Vector getControlNames(){
        return beanLabels;
    }

    public Vector getControlImages(){
        return beanIcons;
    }

    public Vector getBeanNames(){
        return beanNames;
    }

    public Object getSelectedBean(int index) {
        String beanName = (String) beanNames.elementAt(index);
        String beanLabel = (String) beanLabels.elementAt(index);
        JarInfo ji = (JarInfo) beanJars.elementAt(index);
        try {
            return ji.getInstance(beanName);
        } catch (Exception ex) {
            System.err.println(Constants.ERRID+"instantion of "+beanName+" bean failed "+ ex);
            return null;
        }
    }
    
    public Object getBeanByName(String Name){
        for (int i = 0; i< beanNames.size(); i++){ //should use HashTable
            if (Name.equals((String) beanNames.elementAt(i)))
                return getSelectedBean(i);
        }
        return null;
    }

    /**
     * Provide information on all the loaded JAR files, regardless of origin.
     */
    public Vector getLoadedJarInfo() {
	return beanJars;
    }
    
    private static Vector getJarNames(String []locations, URL baseUrl) {
        Vector result = new Vector();
        File jars;
	URL jarsUrl;
        if (locations == null)
            locations = new String[1];
        for (int j = 0; j < locations.length; j++) {
	    jars = null;
	    jarsUrl = null;
            if (locations[j] == null) {
                File cwd = new File(System.getProperty("user.dir"));
                File pwd = new File(cwd.getParent());
                jars = new File(pwd, "jars");
            } else {
		try {
		    jarsUrl = new URL(locations[j]);
		} catch(MalformedURLException mue) {
		    if (baseUrl != null) {
			try {
			    jarsUrl = new URL(baseUrl, locations[j]);
			} catch(MalformedURLException mue2) {
			}
		    }
		    if (baseUrl == null) 
			jars = new File(locations[j]);
		}
            }
	    if (jars != null) {
		if (!jars.isDirectory()) {
		    System.err.println(Constants.ERRID+jars+" is not a directory.");
		    continue;
		}
		
		String names[];
		names = jars.list(new FileExtension(".jar"));
		for (int i=0; i<names.length; i++) {
		    result.addElement(jars.getPath() + File.separatorChar + names[i]);
		}
		names = jars.list(new FileExtension(".JAR"));
		for (int i=0; i<names.length; i++) {
		    result.addElement(jars.getPath() + File.separatorChar + names[i]);
		}
	    } else {
		result.addElement(jarsUrl);
	    }
        }
        return result;
    }
    /**
     * Special action.
     *
     * Currently only used to register the BeanBox
     */
    public synchronized void addBeanClass(String beanClassName) {
		Class beanClass;
		try {
			beanClass = Class.forName(beanClassName);
		} catch (Exception ex) {
			System.err.println("Toolbox: couldn't instantiate " +
									 beanClassName);
			return;
		}
		BeanInfo bi;
		try {
			bi = Introspector.getBeanInfo(beanClass);
		} catch (Exception ex) {
			System.err.println("Toolbox: couldn't find BeanInfo for " +
									beanClassName);
			ex.printStackTrace();
			return;
		}
		String label = bi.getBeanDescriptor().getDisplayName();
		Image img = bi.getIcon(BeanInfo.ICON_COLOR_16x16);
		addWithUniqueName(beanLabels, label);
		beanNames.addElement(beanClassName);
		beanIcons.addElement(img);
		beanJars.addElement(null);
    }
    /**
     * Add all the beans in a Jar file into the ToolBox
     */
    public synchronized void addBeansInJar(String jarFile) {
		JarLoader.loadJarDoOnBean(jarFile, helper);
    }
    
    /**
     * Add all the beans in a Jar URL into the ToolBox
     */
    public synchronized void addBeansInJar(URL jarURL) {
		JarLoader.loadJarDoOnBean(jarURL, helper);
    }

    static public String getJarsdirParm(){
        return jarsdirparm;
    }

    static public void setJarsdirParm(String p){
        jarsdirparm = p;
    }

    private static String jarsdirparm; // dirs or URLs list
}

class FileExtension implements FilenameFilter {
    private String extension;

    public FileExtension(String ext) {
	this.extension = ext;
    }

    public boolean accept(File dir, String name) {
	return name.endsWith(extension);
    }
}
