/*
 * File:		FormConstructor.java
 * Author:		Dmitry Rogatkin
 * Date:		3/15/98
 *
 * Description:	main module of FormConstructor project
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.core;

import java.awt.*;
import java.beans.*;
import java.applet.Applet;
import java.io.File;
import java.net.URL;
import java.net.MalformedURLException;

import rogatkin.xbox.io.*;
import rogatkin.xbox.ui.*;
import rogatkin.xbox.ut.WindowCloser;
import rogatkin.xbox.ut.XMLToolFactory;
import rogatkin.xbox.shared.Constants;

public class FormConstructor extends /*AwtUI*/Applet {
    
    static String formUrl;
    boolean isApplication = false;
    
    public FormConstructor() {
	// super(new eFormApplet());
    }
    
    public static void main(String args[])  {
	// Parse the arguments.
	if (args.length > 6) {
	    throw new Error("Incorrect arguments number "+args.length);
	}
        for (int i = 0; i < args.length; i++)
        {
	    if (args[i].equals("-version")) {
		System.out.println(Constants.PRODUCTID+": "+ WorkSpace.getVersionID());
		System.exit(0);
	    } else if (args[i].equals("-jarsdir")){
		i++;
		if (i < args.length)
		    CompManager.setJarsdirParm(args[i]);
	    } else if (args[i].equals("-form")){
		i++;
		if (i < args.length)
		    setFormUrl(args[i]);
	    } else {
		WorkSpace.setTmpDir(args[i]);
	    }
	    
        }
	if (args.length > 0) {
	    if (args[0].equals("-version")) {
		System.out.println("FormConstructor version: "+ WorkSpace.getVersionID());
		System.exit(0);
	    } 
	}
	MainFrame f = new MainFrame ();
	new WindowCloser(f, true);
	FormConstructor app = new FormConstructor();
	app.isApplication = true;
	f.setLayout(new BorderLayout());
	f.add (app, "Center");
	app.init();
	f.pack();
	f.setVisible (true);
	f.setIconImage(ImageHelper.loadImage("rfico.gif")); // need create an image ###
	f.setBounds(170, 20, 400, 550);
    }
    
    static void setFormUrl(String url) {
	formUrl = url;
    }
    
    static String getFormUrl() {
	return formUrl;
    }
    
    public void init() {
	//AppletContext ac = getAppletContext();
	String jdir = null;
	try {
	    jdir = getParameter("jarsdir");
	    if (jdir != null)
		CompManager.setJarsdirParm(jdir);
	} catch(Exception e) {
	}
	try {
	    String fn = getParameter("form");
	    if (fn != null && fn != "")
		setFormUrl(fn);
	} catch(Exception e) {
	}
	
	Starter starter = null;

	if (!isApplication)
	    ImageHelper.init(this);
	else {
	    starter = new StartFrame();
	    starter.setComponentName("xBox");
	    starter.setDescription("Loading componets...");
	}
	// Create a ControlsManager and load the jar files.
	CompManager formmanager = new CompManager();
	setLayout(new BorderLayout());
	
	try {
	    System.err.println("prepare for basic panel");
	    formmanager.buildControlList(starter, CompManager.getJarsdirParm(), getDocumentBase());
	    BasicPanel basicpanel;
	    if(isApplication)
		basicpanel = new JBasicPanel();
	    else
		basicpanel = new AwtBasicPanel();
	    add((Component)basicpanel, "Center");
	    System.err.println("basic panel added");
	    // Set to the design mode.
	    Beans.setDesignTime(false);
	    basicpanel.init(formmanager);
	    // load form 
	    try {
		basicpanel.getWorkSpace().loadXML(new URL(getFormUrl()));
	    } catch (MalformedURLException mfue) {
	    }
	} finally {
	    if(starter != null)
		starter.dispose();
	}
    }
    
    public String getInfo ()  {
	String s = "Info about FormConstructor.class\r\n";
	s += "Version: 1.0.1\r\n";
	return s;
    }
}
