/*
 * File:		WorkSpace.java
 * Author:		Dmitry Rogatkin 
 * Date:		2/18/98
 *
 * Version:  1.0
 * Revision:  10.0
 * History:
 *           
 * Description:
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.core;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import java.util.*;
import java.beans.*;
import java.net.*;

import rogatkin.xbox.shared.*;
import rogatkin.xbox.ui.*;
import rogatkin.xbox.io.*;

public class WorkSpace extends BasicSpace implements MouseListener, MouseMotionListener {

    //----------------------------------------------
    // Private static constants
    //----------------------------------------------
    private final static int padX = 10;
    private final static int padY = 10;
    //*!
    
    // Shorthands for the cursors.
    private final static Cursor nwResizeCursor = Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR);
    private final static Cursor neResizeCursor = Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR);
    private final static Cursor swResizeCursor = Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR);
    private final static Cursor seResizeCursor = Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR);
    private final static Cursor eResizeCursor = Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR);
    private final static Cursor nResizeCursor = Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR);
    private final static Cursor sResizeCursor = Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR);
    private final static Cursor wResizeCursor = Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR);
    private final static Cursor crosshairCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
    private final static Cursor defaultCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
    private final static Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
    
    //----------------------------------------------
    // Private static variables
    //----------------------------------------------
    private static Hashtable serNames = new Hashtable();
    private static boolean doShowTimes = false;
    private static String clipDir = "tmp";
    private static String clipFile = "xBoxClip.ser";
    private static String clipResource = "xBoxClip";
    private static String clipLabel;
    private static String clipName;
    private static String tmpDir = "tmp";
    private final static Color RubberBandColor = Color.white;
    //  private static boolean defineOnDemand = true;	// Moved to JarLoader.
    private final static String defaultSerializeFile = "xbox.ser";
    private final static String defaultStoreFile = "xbox.tmp";
    
    //-------------------------------------------------
    // Private variables
    //-------------------------------------------------
    private boolean Modified = false; //*!
    // builder components
    private IBuilderMenu mainmenu;
    private IToolBox toolbox = null;
    private IPropertySheet propertySheet = null;
    private IStatusInfo statusline = null; // have to be replaced by a listener mouse coord
    private Cursor prevCursor = defaultCursor;
    
    //-------------------------------------------------
    // Private transient variables
    //-------------------------------------------------
    private transient Hashtable formbeans = new Hashtable();
    
    private transient Wrapper sourceWrapper = null;
    private transient int oldX;
    private transient int oldY;
    private transient Rectangle oldRubberBox;
    private transient Class customizerClass;
    
    // If moveChild is non-null, we're moving that child
    private transient Wrapper moveChild;
    private transient int startX;
    private transient int startY;
    
    // If resizeChild is non-null, we're resizing that child
    private transient Wrapper resizeChild;
    private transient Cursor resizeCursor;
    
    // MouseClickEvent describes the moset recent mouse click event.
    private transient MouseEvent mouseClickEvent;
    private transient long mousePressedWhen = 0;
    private transient Wrapper currentFocus;
    private transient Wrapper lastFocused;
    private transient Point mousePressedPt = null;
    private transient int minW;	// minimum width of a wrapper.
    private transient int minH;	// minimum height of a wrapper.
    
    // Binding two beans.
    //	private transient boolean bindingMode = true;
    
    /**
    * Initialize a new BeanBox.
    */	
    public WorkSpace(Container container) {
	super(container);
	container.addMouseListener(this);
	container.addMouseMotionListener(this);
	currentFocus = null;
	lastFocused = null;
    }
    
    public void init(CompManager cm) {
	super.init(cm);
	setCurrentComponent(null);
	updateCaption();
    }
    
    //*<
    public Object getCurrentBean() {
	if(currentFocus != null)
	    return currentFocus.getBean();
	else
	    return null;
    }
    
    public void setCurrentComponent(Component focus) {
	doSetCurrentFocus(focus);
    }
    
    public void repaintCurrentComponent(){
	if (currentFocus != null)
	    currentFocus.repaint();
    }
    
    public Wrapper getCurrentWrapper() {
	return currentFocus;
    }
    
    public Wrapper getLastFocused() {
	return lastFocused;
    }
    
    private void doSetCurrentFocus(Component focus) {
	Wrapper target = findWrapper(focus);
	
	if (target == currentFocus) {
	    return;
	}
	
	// Deactivate the old Wrapper.
	if (currentFocus != null) {
	    lastFocused = currentFocus;
	    currentFocus.deactivate();
	}
	
	currentFocus = target;
	
	updateMenuBar();

        // Activate the new Wrapper.
	if(currentFocus != null)
	    currentFocus.activate();
	
        // Point the property sheet at the new focus.
	if (propertySheet != null)
	    propertySheet.setTarget(currentFocus);
	
    }
    
    public void setDesignTime(boolean designMode) {
	Beans.setDesignTime(designMode);
	Enumeration beans = formbeans.elements();
        while (beans.hasMoreElements()){
	    ((Wrapper)beans.nextElement()).updateBorders();
	}
    }
    
    /**
    * Update the MenuBar for the current beanBox and focus bean.
    */
    
    public synchronized void updateMenuBar() {
	
	if (mainmenu == null)
	    return;
	mainmenu.updateViewMenu(Beans.isDesignTime(), getHideInvisibleBeans());
	mainmenu.removeCustom();
	mainmenu.enableBackgroundBean(currentFocus != null);
	mainmenu.setBackgroundBean(backgroundbean != null && backgroundbean == currentFocus);
	
	// Take a look at the current focus bean, and create appropriate
	// menu items for its events, customizer, etc.
	
	// customizers are not supported temporary
    }
    
    /**
    * Provide an identifier for serializing this object
    */
    private synchronized String serFileName(Object o) {
	if (o == null) {
	    // distinguished root-component
	    return getTmpDir()+File.separator+"___comp_ROOT.ser";
	} else {
	    String s = (String) serNames.get(o);
	    if (s == null) {
		String name = "___comp_"+"generate some ID here"+".ser";
		s = getTmpDir()+File.separator+name;
		serNames.put(o, s);
	    }
	    return s;
	}
    }
    
    private boolean generateManifestTemplate(File file) {
	File dir = new File(file.getParent());
	if (dir != null) {
	    dir.mkdirs();
	}
	OutputStream os;
	try {
	    os = new FileOutputStream(file);
	    PrintWriter pos = new PrintWriter(os);
	    
	    int count = getComponentCount();
	    for (int i = 0; i < count; i++) {
		Wrapper w = findWrapper(getComponent(i));
		pos.println("Name: "+serFileName(w).replace(File.separatorChar, '/'));
		pos.println("Java-Bean: True");
		pos.println();
	    }
	    pos.flush();
	    pos.close();
	} catch (Exception ex) {
	    error("Problems creating manifest template file", ex);
	    return false;
	}
	return true;
    }
    
    /**
    * This implements the "save" menu item.  This stores away the
    * current state of the BeanBox to a named file.
    *
    * DISCLAIMER:  We do not claim that this is the "proper way" to
    * save a collection of inter-dependent beans.  Future BDKs will
    * expand in this area.
    */
    public void save() {
	// Write a JAR file that contains all the hookups and a
	// single serialized stream.
	// Then, on load, read the components and mock-up the AppletStub.
	
	boolean saveAdaptors = true;
	
	FileDialog fd = new FileDialog(getFrame(), "Save File", FileDialog.SAVE);
	// the setDirectory() is not needed, except for a bug under Solaris...
	fd.setDirectory(System.getProperty("user.dir"));
	fd.setFile(defaultStoreFile);
	fd.show();
	String fname = fd.getFile();
	if (fname == null) {
	    return;
	}
	String dname = fd.getDirectory();
	File file = new File(dname, fname);	    
	
	try {
	    // create the single ObjectOutputStream
	    File serFile = new File(serFileName(null));
	    File dir = new File(serFile.getParent());
	    if (dir != null) {
		dir.mkdirs();
	    }
	    FileOutputStream f = new FileOutputStream(serFile);
	    ObjectOutputStream oos = new ObjectOutputStream(f);
	    writeContents(oos);
	    oos.close();
	    
	    // Generate the manifest file?
	    
	    Vector hfiles = new Vector(0);	// hookups
	    if (saveAdaptors) {
		//hfiles = HookupManager.getHookupFiles();
	    }
	    String args[] = new String[3+hfiles.size()];
	    
	    args[0] = "cf";
	    args[1] = file.getPath();	// name for the JAR file
	    // no manifest file - so far
	    args[2] = serFileName(null);
	    for (int i=0; i<hfiles.size(); i++) {
		args[3+i] = (String) hfiles.elementAt(i);
	    }
	    
	    sun.tools.jar.Main jartool =
		new sun.tools.jar.Main(System.out, System.err, "jar");
	    
	    boolean ok = jartool.run(args);
	    if (!ok) {
		error("Jar tool invocation failed");
	    }
	} catch (Exception ex) {
	    error("Save failed", ex);
	}
    }
    
    
    /**
    * This implements the "serializeComponent" menu item.
    * The selected component is serialized to the chosen file
    */
    private void serializeComponent() {
	FileDialog fd = new FileDialog(getFrame(), "Serialize Component into File", FileDialog.SAVE);
	// needed for a bug under Solaris...
	fd.setDirectory(System.getProperty("user.dir"));
	fd.setFile(defaultSerializeFile);
	fd.show();
	String fname = fd.getFile();
	if (fname == null) {
	    return;
	}
	String dname = fd.getDirectory();
	File file = new File(dname, fname);
	
	Wrapper w = getCurrentWrapper();
	
	try {
	    FileOutputStream f = new FileOutputStream(file);
	    ObjectOutputStream oos = new ObjectOutputStream(f);
	    // Ask the Wrapper to serialize the "naked" bean.
	    // as in copy()
	    w.writeNakedBean(oos);
	    oos.close();
	} catch (Exception ex) {
	    error("Serialization of Component failed", ex);
	}
    }
    
    
    /**
    * This implements the "load" menu item.  This loads the BeanBox state
    * from a named file.
    */
    private void load() {
	
	removeAll();
	FileDialog fd = new FileDialog(getFrame(), "Load File", FileDialog.LOAD);
	// needed for a bug under Solaris...
	fd.setDirectory(System.getProperty("user.dir"));
	fd.setFile(defaultStoreFile);
	fd.show();
	String fname = fd.getFile();
	if (fname == null) {
	    return;
	}
	String dname = fd.getDirectory();
	File file = new File(dname, fname);
	
	JarLoader jl = null;
	try {
	    jl = new JarLoader(file.getPath());
	    if (jl == null) {
		System.err.println(Constants.ERRID+" file name passed is not a valid JarFile");
		return;
	    }
	} catch (Exception ex) {
	    System.err.println(Constants.ERRID+" caught "+ex+" while trying to load "+file.getPath());
	    ex.printStackTrace();
	    return;
	}
	
	JarInfo ji = jl.loadJar();
	// OK, loaded all the classes -- now, instantiate them...
	
	try {
	    // get the one object as a bean...
	    ClassLoader cl = jl.getLoader();
	    InputStream is = cl.getResourceAsStream(serFileName(null).replace(File.separatorChar, '/'));
	    ObjectInputStream ois = new rogatkin.xbox.io.ObjectInputStreamWithLoader(is, cl);
	    
	    // Read all the contained beans into this BeanBox.
	    readContents(ois);
	    
	} catch (Exception ex) {
	    ex.printStackTrace();
	    return;
	}
	
    }
    
    // form contents manipulation functions
    
    public void setCursor(Cursor cursor) {
	container.setCursor(cursor);
	if (backgroundbean != null)
	{
	    if (backgroundbean.getBean() instanceof Component)
		((Component)backgroundbean.getBean()).setCursor(cursor);
	}
    }
    
    public void waitCursor() {
        Cursor cc = container.getCursor();
        if (cc == waitCursor)
            setCursor(prevCursor);
        else {
            prevCursor = cc;
            setCursor(waitCursor);
        }
    }
    
    public Object[] getBeans() {
        int beanSize = formbeans.size();
        Object[] list = new Object[beanSize];
        Enumeration beans = formbeans.elements();
        for (int i = 0; beans.hasMoreElements(); i++)
            list[i] = ((Wrapper)beans.nextElement()).getBean();
        return list;
    }
    
    public String extractShortClassName(String longname){
        int ix = longname.lastIndexOf('.');
        if (ix >= 0)
            return longname.substring(ix+1);
        else
            return longname;
    }
    
    public boolean renameWrapper(String oldname, String newname, Wrapper w) {
        try {
	    w = (Wrapper)formbeans.remove(oldname);
	    formbeans.put(newname, w);
        } catch(Throwable e) {
            return false;
        }
        return true;
    }
    
    /* 
    * Support for printing
    * Print the current content of this Beanbox
    */
    private void print() {
	
	PrintJob pj = Toolkit.getDefaultToolkit().getPrintJob(getFrame(), 
	    "Printing Test", (Properties)null);
	
	if (pj != null) {
	    Graphics g;
	    Dimension pageDim = pj.getPageDimension();
	    int pageRes = pj.getPageResolution();
	    boolean lastFirst = pj.lastPageFirst();
	    
	    Graphics gr  = pj.getGraphics();
	    if (gr!=null) {
		// We print all components one after the other in the
		// same page. 		  
		int count = getComponentCount();
		for (int i=0; i<count; i++) {
		    Wrapper wr = (Wrapper) getComponent(i);
		    Object bean = wr.getBean();
		    if (bean instanceof Component) {
			Component c;
			c= (Component) Beans.getInstanceOf(bean, Component.class);
			Dimension d = c.getSize();
			Point o = wr.getLocation();
			Image offScreen = c.createImage(d.width, d.height);
			c.paint(offScreen.getGraphics());
			gr.drawImage(offScreen, o.x, o.y, Color.white, null);
		    }
		}
	    }
	    else 
		System.err.println("Could not get Graphics handle.");
	    
	    gr.dispose();
	    pj.end();
	}
	else {
	    System.err.println("PrintJob cancelled.");
	}
    }
    
    /**
    * This implements the "loadJar" menu item.
    * Load the contents of a JAR file
    */
    private void loadJar() {
	
	FileDialog fd = new FileDialog(getFrame(), "Load from JAR File", FileDialog.LOAD);
	// the setDirectory() is not needed, except for a bug under Solaris...
	fd.setDirectory(System.getProperty("user.dir"));
	fd.setFile(defaultStoreFile);
	fd.show();
	String fname = fd.getFile();
	if (fname == null) {
	    return;
	}
	String dname = fd.getDirectory();
	File file = new File(dname, fname);
	
	getCompManager().addBeansInJar(file.getPath());
	
    }
    
    
    /**
    * Serialize the current focus bean to our clipboard file, and
    * then remove it from the BeanBox.
    */
    
    private void cut() {
	Wrapper wrapper = getCurrentWrapper();
	Object bean = wrapper.getBean();
	if (bean != this) {
	    if (copy()) {
		// succeeded in serializing the component
		setCurrentComponent(null);
		remove(wrapper);
		wrapper.cleanup();
		container.repaint();
	    }
	}
    }
    
    private void del() {
        // save info for undo
	Wrapper wrapper = getCurrentWrapper();
	if (wrapper != null) {
	    if (wrapper == backgroundbean)
		backgroundbean = null;
	    Object bean = wrapper.getBean();
	    if (bean != this) {
		setCurrentComponent(null);
		remove(wrapper);
		wrapper.cleanup();
		container.repaint();
	    }
	}
    }
    
    /**
    * Serialize the current focus bean to our clipboard file.
    */
    private boolean copy() {
	Wrapper wrapper = getCurrentWrapper();
	
	setClipLabel(null);
	setClipName(null);
	try {
	    File f = new File(getClipFileName());
	    File dir = new File(f.getParent());
	    dir.mkdirs();
            FileOutputStream fos = new FileOutputStream(f);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
	    // Ask the Wrapper to serialize the "naked" bean.
	    // This causes the Wrapper to remove all listeners
	    // before serializing, and add them back after.
	    wrapper.writeNakedBean(oos);
	    oos.close();
	    fos.close();
            setClipLabel(wrapper.getDesignName());
            setClipName(wrapper.getBeanName());
	    
	    mainmenu.enablePaste(true);
	    return true;
	} catch (Exception ex) {
	    error("Copy failed", ex);
	    mainmenu.enablePaste(false);
	    return false;
	}
    }
    
    /**
    * Read in a bean from our clipboard file and insert it into
    * the BeanBox.
    */
    private void paste() {
	synchronized (this) {
	    mouseClickEvent = null;
        }
	try {
	    // Set the insert cursor before reading the clipboard.
	    setCursor(crosshairCursor);
	    
	    SimpleClassLoader loader = SimpleClassLoader.ourLoader;
	    MyProducer p = new MyProducer(getClipFileName());
	    String clipName = getClipName();
	    String beanLabel = getClipLabel();
	    
	    Object bean = loader.instantiate(clipName, p);
	    //doInsert(bean, beanLabel, clipName, true);
	} catch (Exception ex) {
	    error("Paste failed", ex);
	    mainmenu.enablePaste(false);
	    setCursor(defaultCursor);
	}
    }
    
    /**
    * Get the address of a target component relative to the 
    * current BeanBox.
    */
    
    private Rectangle getLocalCoordinates(Component c) {
	int width = c.getSize().width;
	int height = c.getSize().height;
	int x = 0;
	int y = 0;
	for (;;) {
	    if(c == container) {
		return new Rectangle(0,0,0,0);
	    }
	    if(c == container) {
		return new Rectangle(x, y, width, height);
	    }
	    x += c.getLocation().x;
	    y += c.getLocation().y;
	    c = c.getParent();
	}
    }
    
    //----------------------------------------------------------------------
    
    // Mouse listener methods.
    //
    // Note that in general these methods must NOT be "synchronized" or we
    // risk serious deadlock problems.
    //
    // At this level we mostly just do rubber-banding and simple
    // move/resize handling.
    
    public void mouseClicked(MouseEvent evt) {
	// To workaround #4039858 we do mouseClicked processing in mouseRelease
	// and do nothing here.
    }
    
    public void mousePressed(MouseEvent evt) {
	clearSelections();
	
	mousePressedWhen = evt.getWhen();
	mousePressedPt = evt.getPoint();
	Component comp = evt.getComponent();
	
	// Get the wrapper fot the selected component.
	sourceWrapper = findWrapper(comp);
	if (sourceWrapper == backgroundbean)
	    sourceWrapper = null;
	
	// Convert to the container's coordinate.
	while(comp != container && comp != null) {
	    Point pt = comp.getLocation();
	    mousePressedPt.x += pt.x;
	    mousePressedPt.y += pt.y;
	    comp = comp.getParent();
	}
	
	// Initialize rubber band.
	oldX = mousePressedPt.x;
	oldY = mousePressedPt.y;
    }
    
    public void mouseReleased(MouseEvent evt) {
	
	long deltaWhen = (evt.getWhen() - mousePressedWhen);
	mousePressedWhen = 0;
	
	int x = evt.getX();
	int y = evt.getY();
	Component comp = evt.getComponent();
	
	if (moveChild != null) {
	    // If we're in the middle of a move, complete the move.
	    // Make sure the event coordinates are relative to the child.
	    if (comp == container) {
		x -= moveChild.getLocation().x;
		y -= moveChild.getLocation().y;
	    }
	    finishMove(x, y);
	    Modified = true;
	    return;
	} else if (resizeChild != null) {
	    // If we're in the middle of a resize, complete the resize.
	    // Make sure the event coordinates are relative to the child.
	    if (comp == container) {
		x -= resizeChild.getLocation().x;
		y -= resizeChild.getLocation().y;
	    }
	    finishResize(x, y);
	    Modified = true;
	    return;
	}
	else {
	    // No child is selected.
	    if(sourceWrapper != null){
		deleteRubberBand();	// Delete the previous conneting line.
	    } else {
		deleteRubberBox();	// Delete the previous rubber box.
		// find selected wrappers and mark it
		Rectangle bbox = getRubberBox(x, y);
		int count = getComponentCount();
		for (int i=0; i<count; i++) {
		    if (getComponent(i) instanceof Wrapper) {
			Wrapper w = (Wrapper) getComponent(i);
			if (bbox.intersection(w.getBounds()).equals(w.getBounds()))
			    w.setSelected(true);
			w.repaint();
		    }
		}
		
	    }
	}
	
	// Because of bug #4039858 we get a bogus mouseClicked and mouseReleased
	// event when a menuItem is selected.  Therefore we manually check
	// if there has been a mousePressed within the preceeding 5 seconds before
	// the mouseRelease, and only then do we treat it as a genuine mouse
	// click.						KGH 4/3/97
	if (deltaWhen < 6000) {
	    // OK, treat this as a genuine mouse click.	
	    // Wakeup anyway who is waiting for a mouse down.
	    synchronized (this) {
		
		mouseClickEvent = new MouseEvent(evt.getComponent(), evt.getID(),
		    evt.getWhen(), evt.getModifiers(), evt.getX(), evt.getY(),
		    evt.getClickCount(), evt.isPopupTrigger());
		notifyAll();
		
		// set the new focus, unless we're in the middle of an insert.
		if (container.getCursor() != crosshairCursor && Beans.isDesignTime()) {
		    Component target = evt.getComponent();
		    setCurrentComponent(target);
                    if (statusline != null)
                        statusline.updateRectangleCoord(target.getBounds());
		    
		}
	    }
	}
	
	if(container.getCursor() == crosshairCursor) {	// Insert a component.
	    // Convert to the container's coordinate.
	    while(comp != container && comp != null) {
		Point pt = comp.getLocation();
		x += pt.x;
		y += pt.y;
		comp = comp.getParent();
	    }
	    
	    // Do insertion at the mouse release.
	    Rectangle bbox = getRubberBox(x, y);
	    if (toolbox != null){
		if(toolbox.isBeanSelected()) {
		    doInsert(bbox);
		    toolbox.setBeanSelected(false);
		}
	    }
	}
	else {
	    Wrapper targetWrapper = getCurrentWrapper();
	    if(sourceWrapper == null || targetWrapper == null) {
		//				System.out.println("source or target is null.");
	    }
	    else if(sourceWrapper == targetWrapper) {
		//				System.out.println("source and target are the same.");
	    }
	    else {
		//				System.out.println("source: " + sourceWrapper);
		//				System.out.println("target: " + targetWrapper);
	    }
	}
	sourceWrapper = null;
}

public void mouseEntered(MouseEvent evt) {
}

public void mouseExited(MouseEvent evt) {
}

public synchronized void mouseDragged(MouseEvent evt) {
    int x = evt.getX();
    int y = evt.getY();
    
    if (statusline != null)
	statusline.updateCursorCoord(x, y);
    
    if (moveChild != null) {
	// If we're in the middle of a move, repaint the child.
	// Make sure the event coordinates are relative to the child.
	if(evt.getComponent() == container) {
	    x -= moveChild.getLocation().x;
	    y -= moveChild.getLocation().y;
	}
	updateChild(getMoveBox(x, y));
    } else if (resizeChild != null) {
	// If we're in the middle of a resize, repaint the child.
	// Make sure the event coordinates are relative to the child.
	if(evt.getComponent() == container) {
	    x -= resizeChild.getLocation().x;
	    y -= resizeChild.getLocation().y;
	}
	updateChild(getResizeBox(x, y));
    } else {
	// Convert to the container's coordinate.
	Component c = evt.getComponent();
	while(c != container && c != null) {
	    Point pt = c.getLocation();
	    x += pt.x;
	    y += pt.y;
	    c = c.getParent();
	}			
	if(sourceWrapper != null)
	    drawRubberBand(x, y);	// draw the connection line.
	else
	    drawRubberBox(getRubberBox(x, y));	// draw the rectangle.
    }
}

public synchronized void mouseMoved(MouseEvent evt) {
}

public synchronized void keyDelete() {
    del();
}

public void clearForm(){
    removeAll();
    setCurrentComponent(null);
    Modified = false;
    getEventMap().clear();
    container.repaint();
}

/*public void setBackgroundBean(Wrapper bkgbeanwrapper) {
currentFocus = bkgbeanwrapper;
setBackgroundBean(true);
currentFocus = null;
}*/

//----------------------------------------------------------------------
public void Command (FCEvent e) {
    super.Command(e);
    switch(e.getIDCommand())
    {
    case Constants.CMDSave:	
	save();
	break;
    case Constants.CMDOpen:
	load();
	break;
    case Constants.CMDSerializeComponent:
	serializeComponent();
	break;
    case Constants.CMDBringFwd:
	bringBeanToForward(getCurrentWrapper());
	break;
    case Constants.CMDBringBck:
	bringBeanToBackward(getCurrentWrapper());
	break;
    case Constants.CMDLoadJar:
	loadJar();
	break;
    case Constants.CMDPrint:
	print();
	break;
    case Constants.CMDCut:
	cut();
	break;
    case Constants.CMDDelete:
	del();
	break;
    case Constants.CMDCopy:
	copy();
	break;
    case Constants.CMDPaste:
	paste();
	break;
    }
}

/**
* Insert a given Component instance into the current BeanBox.
*/
private void doInsert(Rectangle bbox) {
    int row = toolbox.getCurrentBeanIndex();
    Object bean = getCompManager().getSelectedBean(row);
    String beanName = (String) getCompManager().getBeanNames().elementAt(row);
    if (bean != null)
	doInsert(bean, beanName, bbox);
}

private void doInsert(Object bean, String beanName, Rectangle bbox) {
    // Revert to the default cursor.
    setCursor(defaultCursor);
    // Create a Wrapper for the child
    Wrapper wrapper = new Wrapper(this, bean, beanName);
    
    // get unique name
    wrapper.setDesignName(generateUniqueNameBasedOn(extractShortClassName(beanName)));
    // Insert and reshape the child component to the top.
    add(wrapper, 0);
    
    if(bbox != null) {
	int inset = wrapper.getBorderWidth();
	int w = bbox.width + 2*inset;
	int h = bbox.height + 2*inset;
	Dimension minSize = wrapper.getMinimumSize();
	if(minSize.width > w)
	    w = minSize.width;
	if(minSize.height > h)
	    h = minSize.height;
	wrapper.setBounds(bbox.x - inset, bbox.y - inset, w, h);
    }
    else {
	int wrapperWidth = wrapper.getPreferredSize().width;
	int wrapperHeight = wrapper.getPreferredSize().height;
	int x = mouseClickEvent.getX() - wrapperWidth/2;
	int y = mouseClickEvent.getY() - wrapperHeight/2;
	wrapper.setBounds(x, y, wrapperWidth, wrapperHeight);
    }
    
    if (Beans.isInstanceOf(bean, Applet.class)) {
	Applet apl = (Applet) Beans.getInstanceOf(bean, Applet.class);
	apl.start();
    }
    
    // Make the child the focus.
    setCurrentComponent(wrapper);
    Modified = true;
    
    // Notice that we do not attach mouse event listeners to the child
    // or to the wrapped bean.  This allows them to use the old
    // AWT event model if the bean is a transitional bean.
    // We rely on the wrapper calling our MouseListener and
    // MouseMotionListener methods when mouse events happen.
    
}

public void clearSelections(){
    for (int i=0; i<getComponentCount(); i++) {
	Component c = getComponent(i);
	if (c instanceof Wrapper && ((Wrapper)c).isSelected()){
	    ((Wrapper)c).setSelected(false);
	    ((Wrapper)c).repaint();
	}
    }
}

//----------------------------------------------------------------------

/**
* Someone has done a mouse button down on the "wrapper" around the bean.
* This starts a move that will be terminated when they release the
* mouse button.  We will draw a rubbed-banded box to indicate the
* current move position.
*/

public void startMove(Wrapper child, int x, int y) {
    moveChild = child;
    startX = x;
    startY = y;
}

/**
* Figure out the on-screen box to be drawn to represent the proposed
* new move location of a bean we're moving.
*/

Rectangle getMoveBox(int mx, int my) {
    Rectangle r = moveChild.getBounds();
    r.x += mx - startX;	
    r.y += my - startY;
    return r;
}

/**
* Complete a move started with "startMove".
*/

void finishMove(int mx, int my) {
    Rectangle box = getMoveBox(mx, my);
    moveChild.setBounds(box.x, box.y, box.width, box.height);
    moveChild = null;
}

//----------------------------------------------------------------------

/**
* Someone has done a mouse button down on a corner of the "wrapper"
* around the bean.  This starts a resize sequence that will be
* terminated when they release the mouse button.  We will draw a 
* rubbed-banded box to indicate the proposed resize shape..
*/

public void startResize(Wrapper wrapper, int x, int y, Cursor cursor) {
    resizeChild = wrapper;
    resizeCursor = cursor;
    
    // get the minimum size of this wrapper.
    Dimension minSize = wrapper.getMinimumSize();
    minW = minSize.width;
    minH = minSize.height;
}

/**
* Figure out the box to draw to imndicate the result of an
* in-progress resize.  This is slightly complicated because
* it depends on which corner they picked-up to begin the
* resize.
*/
Rectangle getResizeBox(int mx, int my) {
    int x, y, x1, y1, w, h;
    
    x = resizeChild.getLocation().x;
    y = resizeChild.getLocation().y;
    w = resizeChild.getSize().width;
    h = resizeChild.getSize().height;
    x1 = x + w;
    y1 = y + h;
    
    if (resizeCursor == nwResizeCursor) {
	w = w - mx;
	h = h - my;
	x = x + mx;
	y = y + my;
	if(w < minW) {
	    x = x1 - minW;
	    w = minW;
	}
	if(h < minH) {
	    y = y1 - minH;
	    h = minH;
	}
    }
    else if (resizeCursor == swResizeCursor) {
	w = w - mx;
	h = my;
	x = x + mx;
	if(w < minW) {
	    x = x1 - minW;
	    w = minW;
	}
	if(h < minH) {
	    h = minH;
	}
    } else if (resizeCursor == neResizeCursor) {
	w = mx;
	h = h - my;
	y = y + my;
	if(w < minW) {
	    w = minW;
	}
	if(h < minH) {
	    y = y1 - minH;
	    h = minH;
	}
    }
    else if (resizeCursor == seResizeCursor) {
	w = mx;
	h = my;
	if(w < minW) {
	    w = minW;
	}
	if(h < minH) {
	    h = minH;
	}
    }
    else if (resizeCursor == nResizeCursor) {
	h = h - my;
	y = y + my;
	if(h < minH) {
	    y = y1 - minH;
	    h = minH;
	}
    }
    else if (resizeCursor == sResizeCursor) {
	h = my;
	if(h < minH) {
	    h = minH;
	}
    }
    else if (resizeCursor == wResizeCursor) {
	w = w - mx;
	x = x + mx;
	if(w < minW) {
	    x = x1 - minW;
	    w = minW;
	}
    }
    else if (resizeCursor == eResizeCursor) {
	w = mx;
	if(w < minW) {
	    w = minW;
	}
    }
    return new Rectangle(x, y, w, h);
}

/** 
* Complete a resize begin with startResize.
*/

void finishResize(int mx, int my) {
    Rectangle box = getResizeBox(mx, my);
    resizeChild.setBounds(box.x, box.y, box.width, box.height);
    resizeChild.doLayout();
    resizeChild = null;
}

/**
* updateChild
*
* repaint the moved or resized child component.
*/
private void updateChild(Rectangle box) {
    Component child = null;
    
    if(moveChild != null)
	child = moveChild;
    else if(resizeChild != null)
	child = resizeChild;
    
    if(child != null) {
	child.setBounds(box.x, box.y, box.width, box.height);
	child.repaint();
	if (statusline != null)
	    statusline.updateRectangleCoord(box);
	
    }
}

//----------------------------------------------------------------------

Rectangle getRubberBox(int mx, int my) {
    if (mousePressedPt == null)
	mousePressedPt = new Point(0, 0);
    int x0 = mousePressedPt.x;
    int y0 = mousePressedPt.y;
    int w, h;
    
    if(mx > x0) {
	w = mx - x0;
    }
    else {
	w = x0 - mx;
	x0 = mx;
    }
    
    if(my > y0) {
	h = my - y0;
    }
    else {
	h = y0 - my;
	y0 = my;
    }
    
    return new Rectangle(x0, y0, w, h);
}

private void drawRubberBox(Rectangle box) {
    Graphics g = container.getGraphics();
    if (g == null) {
	return;
    }
    // First remove any existing rubber box.
    deleteRubberBox();
    // Now draw the new rubber box.
    g.setXORMode(RubberBandColor); //!!!
    //g.setColor(Color.blue);
    g.drawRect(box.x, box.y, box.width, box.height);	
    oldRubberBox = box;	
}

private void deleteRubberBox() {
    if (oldRubberBox != null) {
	Graphics g = container.getGraphics();
	if (g == null) {
	    return;
	}
	//        g.setColor(container.getBackground());
	g.setXORMode(RubberBandColor); //!!!
	g.drawRect(oldRubberBox.x, oldRubberBox.y, 
	    oldRubberBox.width, oldRubberBox.height);	
	oldRubberBox = null;
    }
}

private void drawRubberBand(int x, int y) {
    Graphics g = container.getGraphics();
    if (g == null) {
	return;
    }
    // First remove any existing rubber box.
    deleteRubberBand();
    // Now draw the new rubber box.
    //g.setColor(Color.blue);
    g.setXORMode(RubberBandColor);
    g.drawLine(mousePressedPt.x, mousePressedPt.y, x, y);
    g.drawLine(mousePressedPt.x+1, mousePressedPt.y+1, x+1, y+1);
    oldX = x;
    oldY = y;
}

private void deleteRubberBand() {
    Graphics g = container.getGraphics();
    if (g == null) {
	return;
    }
    g.setXORMode(RubberBandColor);
    //g.setColor(container.getBackground());
    g.drawLine(mousePressedPt.x, mousePressedPt.y, oldX, oldY);	
    g.drawLine(mousePressedPt.x+1, mousePressedPt.y+1, oldX+1, oldY+1);	
}

/**
* Static method to map from an AWT component to the associated Wrapper.
*/
public static Wrapper findWrapper(Component comp) {
    for (;;) {
	if (comp == null) {
	    //	component has no wrapper!
	    return null;
	}
	if (comp instanceof Wrapper) {
	    return (Wrapper) comp;
	}
	comp = comp.getParent();
    }
}

public static boolean getHideInvisibleBeans() {
    return !Wrapper.isShowingInvisibleBeans();
}

public static void setHideInvisibleBeans(boolean hide) {
    Wrapper.setShowingInvisibleBeans(!hide);
}

public static boolean showTimes() {
    return doShowTimes;
}

public static String getClipDir() {
    return clipDir;
}

public static String getClipFileName() {
    if (clipDir != null) {
	return clipDir+java.io.File.separator+clipFile;
    } else {
	return clipFile;
    }
}

public static String getClipResource() {
    return clipResource;
}

public static void setClipLabel(String label) {
    clipLabel = label;
}

public static String getClipLabel() {
    return clipLabel;
}

public static void setClipName(String name) {
    clipName = name;
}

public static String getClipName() {
    return clipName;
}

public static void setTmpDir(String itsDir) {
    tmpDir = itsDir;
}

public static String getTmpDir() {
    return tmpDir;
}

public boolean isModified(){
    return Modified;
}

public void setModified(boolean on){
    Modified = on;
}

public static boolean getDefineOnDemand() {
    return JarLoader.getDefineOnDemand();
}

public void setToolBox(IToolBox tb){
    toolbox = tb;
}

public IToolBox getToolBox() {
    return toolbox;
}

public void setPropertySheet(IPropertySheet ps){
    propertySheet = ps;
}

public void setStatusInfo(IStatusInfo sl){
    statusline = sl;
}

public void setMainMenu(IBuilderMenu bm){
    mainmenu = bm;
}

//====================================================
// implements saveXML() of WorkSpace
//====================================================
public void saveXML(File itsFile)
{
    super.saveXML(itsFile);    
    updateCaption();
}

//====================================================
// implements loadXML() of WorkSpace
//====================================================
public void loadXML(URL url) {
    setCurrentComponent(null);
    clearForm();
    super.loadXML(url);
    updateCaption();
}

public void saveComponentXML(File file)
{
    if (file == null)
	return;
    try {
	FileOutputStream f = new FileOutputStream(file);
	//archive.saveComponent(getCurrentWrapper(), f);
    }
    catch (Exception ex) {
	System.err.println(Constants.ERRID+" caught "+ex+" while trying to save the component in XML format");
	ex.printStackTrace();
	return;
    }
}

void enumerateLinks(IControlEnvCallBack callback){
    
}

public void updateCaption() {
    try {
	getFrame().setTitle(getFormName());
    } catch(Throwable e) {
	
    }
}

public void loadXML(File file)
{
    setCurrentComponent(null);
    clearForm();
    super.loadXML(file);
    updateCaption();
}

// helpers
//----------------------------------------------------------------------

// Log an error.

void error(String message, Exception ex) {
    String mess = message + ":\n" + ex;
    System.err.println(message);
    ex.printStackTrace();
    // Popup an ErrorDialog with the given error message.
    new ErrorDialog(getFrame(), mess);
}

void error(String message) {
    System.err.println(message);
    // Popup an ErrorDialog with the given error message.
    new ErrorDialog(getFrame(), message);
}

private static boolean debug = false;

static void debug(String msg) {
    if (debug) {
	System.err.println(Constants.ERRID+msg);
    }
}

}

/**
* Package private support class.  This provides a default AppletContext
* for beans which are applets.
*/

class BeanBoxAppletContext implements AppletContext {
    Applet target;
    Hashtable imageCache = new java.util.Hashtable();
    Map streamsMap = new HashMap();
    
    BeanBoxAppletContext(Applet target) {
        this.target = target;
    }

    public InputStream getStream(String key) {
        return (InputStream)streamsMap.get(key);
    }

    public void setStream(String key,
                      InputStream stream)
               throws IOException {
        streamsMap.put(key, stream);
    }

    public Iterator getStreamKeys() {
        return streamsMap.keySet().iterator();
    }

    public AudioClip getAudioClip(URL url) {
	// We don't currently support audio clips in the Beans.instantiate
	// applet context, unless by some luck there exists a URL content
	// class that can generate an AudioClip from the audio URL.
	try {
	    return (AudioClip) url.getContent();
	} catch (Exception ex) {
	    return null;
	}
    }
    
    public synchronized Image getImage(URL url) {
	Object o = imageCache.get(url);
	if (o != null) {
	    return (Image)o;
	}
	try {
	    o = url.getContent();
	    if (o == null) {
		return null;
	    }
	    if (o instanceof Image) {
		imageCache.put(url, o);
		return (Image) o;
	    }
	    // Otherwise it must be an ImageProducer.
	    Image img = target.createImage((java.awt.image.ImageProducer)o);
	    imageCache.put(url, img);
	    return img;
	} catch (Exception ex) {
	    return null;
	}
    }
    
    public Applet getApplet(String name) {
	return null;
    }
    
    public java.util.Enumeration getApplets() {
	java.util.Vector applets = new java.util.Vector();
	applets.addElement(target);
	return applets.elements();	
    }
    
    public void showDocument(URL url) {
	// We do nothing.
    }
    
    public void showDocument(URL url, String target) {
	// We do nothing.
    }
    
    public void showStatus(String status) {
	// We do nothing.
    }
}


/**
* Package private support class.  This provides an AppletStub
* for beans which are applets.
*/
class BeanBoxAppletStub implements AppletStub {
    transient boolean active;
    transient Applet target;
    transient AppletContext context;
    transient URL codeBase;
    transient URL docBase;
    
    BeanBoxAppletStub(Applet target,
	AppletContext context, URL codeBase, 
	URL docBase) {
        this.target = target;
	this.context = context;
	this.codeBase = codeBase;
	this.docBase = docBase;
    }
    
    public boolean isActive() {
	return active;
    }
    
    public URL getDocumentBase() {
	// use the root directory of the applet's class-loader
	return docBase;
    }
    
    public URL getCodeBase() {
	// use the directory where we found the class or serialized object.
	return codeBase;
    }
    
    public String getParameter(String name) {
	return null;
    }
    
    public AppletContext getAppletContext() {
	return context;
    }
    
    public void appletResize(int width, int height) {
	// we do nothing.
    }
}

/**
* Auxiliary class to deliver the InputStream
*/

class MyProducer implements InputStreamProducer {
    private String name;
    
    public MyProducer(String fileName) {
	this.name = fileName;
    }
    
    public synchronized InputStream getInputStream() {
	try {
	    return new FileInputStream(name);
	} catch (Exception ex) {
	    return null;
	}
    }
}
