/*
 * File:	Wrapper.java
 * Author:	Dmitry Rogatkin (based on Sun's code)
 * Date:	03/11/98
 *
 * Description:	Wrapper
 * This wrapper class keeps track of various BeanBox related
 * state for each bean in the composition window.
 *
 * Among other things, it draws the black-and-white hashed
 * border around the currenty active component.
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.core;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.beans.*;
import java.util.*;
import java.lang.reflect.Method;
import java.lang.reflect.Constructor;
import java.applet.Applet;

import rogatkin.xbox.shared.*;

// TODO: add right mouse menu to move bean to front, to back and delete

public class Wrapper extends 
Container
//javax.swing.JComponent
//Panel
implements Serializable, MouseListener, MouseMotionListener, KeyListener, IFCEvListener {
    
    //----------------------------------------------
    // Static constants
    //----------------------------------------------
    static final long serialVersionUID = 1144602051002987377L;
    static final String CLASSID = "xbox.Wrapper:";
    
    //----------------------------------------------
    // Private static constants
    //----------------------------------------------
    private final static int borderWidth = 5;
    private final static int resizeDelta = 8;
    
    // Shorthands for the cursors.
    private final static Cursor nwResizeCursor = Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR);
    private final static Cursor neResizeCursor = Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR);
    private final static Cursor swResizeCursor = Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR);
    private final static Cursor seResizeCursor = Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR);
    private final static Cursor eResizeCursor = Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR);
    private final static Cursor nResizeCursor = Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR);
    private final static Cursor sResizeCursor = Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR);
    private final static Cursor wResizeCursor = Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR);
    private final static Cursor moveCursor     = Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR);
    private final static Cursor defaultCursor  = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
    
    //----------------------------------------------
    // Private static variables
    //----------------------------------------------
    private static Hashtable eventModelCache = new Hashtable();
    private static Vector invisibleWrappers = new Vector();
    private static boolean showingInvisibleBeans = false;
    private static int hashBarWidth = 4;
    private static int hashBarLength = 104;
    private static Image xHashBar;
    private static Image yHashBar;
    
    private static String cachedbeanname = "";
    private static Wrapper cachedwrapper = null;
    
    //----------------------------------------------
    // Private variables
    //----------------------------------------------	
    private Class MS_AWT_UIHOST, MS_UICOMPONENT, MS_IUICOMPONENT;
    private Method md_getHeader;
    private static final Object[] VOID_PARAM = new Object[0];
    private WorkSpace beanbox;
    private BasicSpace basicspace;
    private Component child;
    private Object bean;
    private String beanName;
    // Forms related variables
    private String DesignName;
    // Design mode related variables
    private boolean selected;
    private boolean keepinplace;
    
    /*
    +----------+--------------+---------------+---------------+
    | PropName | db FieldName | Table/BeanName| DB BeanName   |
    +----------+--------------+---------------+---------------+
    |          |              |               |               |
    
    */
    
    private transient Hashtable dbBinds;
    private String [] dBTriple;
    
    // List of WrapperEventTargets that we fire events at.
    private Vector eventTargets;
    
    //----------------------------------------------
    // Private transient variables
    //----------------------------------------------
    private transient boolean active;
    private transient Cursor cursor = defaultCursor;
    private transient boolean sawMouseDown;
    
    // Table that maps events set names to EventDescriptors.
    private transient Hashtable esdMap;
    
    public static final boolean isDesignTime() {
	return Beans.isDesignTime();
    }
    
    public Wrapper(BasicSpace basicspace, Object bean, String beanName) {
        try {
            MS_AWT_UIHOST = Class.forName("com.ms.ui.AwtUIHost");
            MS_UICOMPONENT = Class.forName("com.ms.ui.UIComponent");
            MS_IUICOMPONENT = Class.forName("com.ms.ui.IUIComponent");
            md_getHeader = MS_AWT_UIHOST.getMethod("getHeader", new Class[0]);
        } catch(Throwable t) {
            //t.printStackTrace();
        }
	this.basicspace = basicspace;
	if (basicspace instanceof WorkSpace)
	    beanbox = (WorkSpace)basicspace;
	this.bean = bean;
	if (beanName == null) {
	    beanName = bean.getClass().getName();
	}
	this.beanName = beanName;
	
	setLayout(null);
	
	if (Beans.isInstanceOf(bean, javax.swing.JComponent.class)) {
	    child = (Component)Beans.getInstanceOf(bean, javax.swing.JComponent.class);
	} else if (Beans.isInstanceOf(bean, Component.class)) {
	    child = (Component)Beans.getInstanceOf(bean, Component.class);
	} else if (MS_AWT_UIHOST != null && MS_UICOMPONENT != null && Beans.isInstanceOf(bean, MS_UICOMPONENT)) {
            try {
                Constructor c = MS_AWT_UIHOST.getConstructor(new Class[] {MS_IUICOMPONENT});
                child = (Component)c.newInstance(new Object[] {Beans.getInstanceOf(bean, MS_AWT_UIHOST)});
            } catch(Throwable t) {
                t.printStackTrace();
            }
	} else {
	    invisibleWrappers.addElement(this);
	    child = new ALabel(beanName);
	    child.getPreferredSize();
	    if(!showingInvisibleBeans) {
		setVisible(false);
	    }
	}
	add(child);
	child.setLocation(borderWidth, borderWidth);
	initialize();
	attachListeners();
	attachOurActionListeners(); // new for our special controls
	dbBinds = new Hashtable(8);
        DesignName = null;
    }
    
    void initialize() {
	esdMap = new Hashtable();
	try {
	    BeanInfo bi = Introspector.getBeanInfo(bean.getClass());
	    EventSetDescriptor[] esds = bi.getEventSetDescriptors();
	    for (int i = 0; i < esds.length; i++) {
		esdMap.put(esds[i].getName(), esds[i]);
	    }
	} catch (IntrospectionException ex) {
	    System.err.println(CLASSID+" couldn't introspect on the bean: " + bean);
	}
	if (eventTargets == null) {
	    eventTargets = new Vector();
	}
    }
    
    private void readObject(ObjectInputStream s)
	throws ClassNotFoundException, IOException {
	s.defaultReadObject();
	initialize();
    }
    
    Hashtable getListeners(String eventName, String beanName){
	Hashtable eventNames = (Hashtable)beanbox.getEventMap().get(DesignName);
	if (eventNames == null)
	    return null;
	return (Hashtable)eventNames.get(eventName);
	
    }
    
    public boolean hasListenerFor(String eventName, String beanName){
	Hashtable listeners = getListeners(eventName, beanName);
	if (listeners == null)
	    return false;
	return (listeners.get(beanName) != null);
    }
    
    public void removeListenerFor(String eventName, String beanName){
	Hashtable listeners = getListeners(eventName, beanName);
	if (listeners == null)
	    return;
	listeners.remove(beanName);
    }
    
    public void addListenerFor(String eventName, String beanName){
	Hashtable em = beanbox.getEventMap();
	Hashtable eventNames = (Hashtable)em.get(DesignName);
	if (eventNames == null){
	    eventNames = new Hashtable();
	    em.put(DesignName, eventNames);
	}
	Hashtable listeners = (Hashtable)eventNames.get(eventName);
	if (listeners == null){
	    listeners = new Hashtable();
	    eventNames.put(eventName, listeners);
	}
	
	listeners.put(beanName, beanName);
    }
    
    public void removeItselfAsListener(){
	// 
	beanbox.enumerateListeners(new ItselfRemover(beanbox, getDesignName(), getBean()));
    }
    
    void attachOurActionListeners(){
	EventSetDescriptor esd = (EventSetDescriptor) esdMap.get(Constants.COMMAND);
	if (esd != null)
	    addEventTarget(Constants.COMMAND, null, this/*getOurListener()*/);
    }
    
    public synchronized void addEventTarget(String eventSetName,
	Wrapper targetWrapper, Object listener) {
	WrapperEventTarget et = new WrapperEventTarget();
	et.eventSetName = eventSetName;
	if (targetWrapper != null) {
	    et.targetBean = targetWrapper.getBean();
	}
	et.targetListener = listener;
	eventTargets.addElement(et);
	EventSetDescriptor esd = (EventSetDescriptor) esdMap.get(eventSetName);
	if (esd == null) {
	    System.err.println(CLASSID+"Internal error: Wrapper.addEventTarget missing event set");
	    System.err.println("        eventSetName = " + eventSetName);
	    System.err.println("        bean = " + bean);
	    return;
	}
	Method adder = esd.getAddListenerMethod();
	Method remover = esd.getRemoveListenerMethod();
	if (adder == null || remover == null) {
	    System.err.println(CLASSID+"Internal error: Wrapper.addEventTarget missing add/remote listener");
	    System.err.println("        eventSetName = " + eventSetName);
	    System.err.println("        bean = " + bean);
	    return;
	}
	try {
	    Object args[] = { listener };
	    adder.invoke(bean, args);
	} catch (Exception ex) {
	    System.err.println(CLASSID+" adding event listener for " + eventSetName + " failed:");
	    System.err.println("    " + ex);
	}
    }
    
    /**
    * Temporarily remove any event listeners.
    */
    public void removeListeners() {
	Enumeration enum = eventTargets.elements();
	while (enum.hasMoreElements()) {
	    WrapperEventTarget et = (WrapperEventTarget)enum.nextElement();
	    EventSetDescriptor esd = (EventSetDescriptor) esdMap.get(et.eventSetName);
	    Method remover = esd.getRemoveListenerMethod();
	    try {
		Object args[] = { et.targetListener };
		remover.invoke(bean, args);
	    } catch (Exception ex) {
		System.err.println(CLASSID+" removing event listener for "
		    + et.eventSetName + " failed:");
		System.err.println("    " + ex);
		//ex.printStackTrace();
	    }
	}
	// Remove mouse and kbd listeners.
	listenForMiceAndKey(false);
    }
    
    /**
    * (Re)-attach any event listeners.
    */
    public void attachListeners() {   
	Enumeration enum = eventTargets.elements();
	while (enum.hasMoreElements()) {
	    WrapperEventTarget et = (WrapperEventTarget)enum.nextElement();
	    EventSetDescriptor esd = (EventSetDescriptor) esdMap.get(et.eventSetName);
	    Method adder = esd.getAddListenerMethod();
	    try {
		Object args[] = { et.targetListener };
		adder.invoke(bean, args);
	    } catch (java.lang.reflect.InvocationTargetException ex) {
		System.err.println(CLASSID+" adding event listener for "
		    + et.eventSetName + " failed:");
		System.err.println("    bean = " + bean);
		System.err.println("    " + ex.getTargetException());
		ex.getTargetException().printStackTrace();
	    } catch (Exception ex) {
		System.err.println(CLASSID+" adding event listener for "
		    + et.eventSetName + " failed:");
		System.err.println("    bean = " + bean);
		System.err.println("    " + ex);
		ex.printStackTrace();
	    }
	}
	// Reattach mouse and kbd listeners.
	listenForMiceAndKey(true);
    }
    
    
    public void writeNakedBean(ObjectOutputStream oos) throws IOException {
	// First, detach all event listeners.
	removeListeners();
	
	// Now write the bean.
	oos.writeObject(bean);
	
	// Now rettach all event listeners.
	removeItselfAsListener();
    }
    
    /**
    * Cleanup is called when a Wrapper has been "cut" from the beanbox.
    */
    public void cleanup() {
	if (bean instanceof Applet) {
	    Applet apl = (Applet)bean;
	    apl.stop();
	    apl.destroy();
	}
	removeListeners();	
	// We should also remove ourself from any event sources...
	removeItselfAsListener();
    }
    
    public BasicSpace getWorkSpace() {
	return beanbox;
    }
    
    public Object getPropertyValue(String propertyName){
	PropertyDescriptor pd = null;
	try{
	    pd = new PropertyDescriptor(propertyName, getBean().getClass());
	} catch (IntrospectionException ie) {
	} catch (Throwable se){
              // it can be com.ms.security.SecurityExceptionEx
	}
	if (pd == null) try { // very bad case
	    // set/get property have different name than property
	    // it can cause decreasing perfomance, how to escape from that
	    // 1. always use standard naming convention
	    // 2. keep 3 names (prop, getter, setter) it can be done in binding
	    BeanInfo bi = Introspector.getBeanInfo(getBean().getClass());
	    pd = BasicSpace.findPropWName(bi.getPropertyDescriptors(), propertyName);
	} catch (IntrospectionException ex) {
	    return null;
	}
	if (pd == null) { // it can be table interface property, try it
	    if (getBean() instanceof IRowProperties) {
		IRowProperties table = (IRowProperties)getBean();
		table.setPropertyName(propertyName);
		return table.getPropertyValue();
	    }
	    return null;
	}
	Method getter = pd.getReadMethod();
	Object args[] = {};
	try {
	    Object result = getter.invoke(getBean(), args);
	    return result;
	} catch (Exception e) {
	}
	return null;
    }
    
    public void setPropertyValue(String propertyName, Object property){
	PropertyDescriptor pd = null;
	try{
	    pd = new PropertyDescriptor(propertyName, getBean().getClass());
	} catch (IntrospectionException ie) {
	} catch (Throwable se){
              // it can be com.ms.security.SecurityExceptionEx
	}
	if (pd == null) try { // very bad case
	    BeanInfo bi = Introspector.getBeanInfo(getBean().getClass());
	    pd = BasicSpace.findPropWName(bi.getPropertyDescriptors(), propertyName);
	} catch (IntrospectionException ex) {
	    return;
	}
	if (pd == null) { // it can be table interface property, try it
	    if (getBean() instanceof IRowProperties) {
		IRowProperties table = (IRowProperties)getBean();
		table.setPropertyName(propertyName);
		table.setPropertyValue(property);
	    }
	    return;
	}
	Method setter = pd.getWriteMethod();
	Object args[] = { property };
	try {
	    Object result = setter.invoke(getBean(), args);
	} catch (Exception e) {
	    System.err.println(CLASSID+e+" in writing property "+propertyName);
	}
    }
    
    public void setPropValWithConversion(String propertyName, Object property){
	PropertyDescriptor pd = null;
	try{
	    pd = new PropertyDescriptor(propertyName, getBean().getClass());
	} catch (IntrospectionException ie) {
	} catch (Throwable se){
              // it can be com.ms.security.SecurityExceptionEx
	}
	if (pd == null) try { // very bad case
	    BeanInfo bi = Introspector.getBeanInfo(getBean().getClass());
	    pd = BasicSpace.findPropWName(bi.getPropertyDescriptors(), propertyName);
	} catch (IntrospectionException ex) {
	    System.err.println(CLASSID+" can't introspect bean "+ex);
	    return;
	}
	if (pd == null) { // it can be table interface property, try it
	    if (getBean() instanceof IRowProperties) {
		IRowProperties table = (IRowProperties)getBean();
		table.setPropertyName(propertyName);
		table.setPropertyValue(property);
	    }
	    return;
	}
	Method setter = pd.getWriteMethod();
	Class[] parms = setter.getParameterTypes();
	if (parms[0] != property.getClass()){
	    if (parms[0] == String.class)
		property = property.toString();
	}
	Object args[] = { property };
	try {
	    Object result = setter.invoke(getBean(), args);
	} catch (Exception e) {
	    System.err.println(CLASSID+e+" in writing property "+propertyName);
	}
    }
    
    /**
    * get the wrapped bean.
    */
    public Object getBean() {
	return bean;
    }
    
    /**
    * get the wrapped beanName
    */
    public String getBeanName() {
	return beanName;
    }
    
    /**
    Methods for binding component properties to data base
    **/
    public void addBindItem(String prop, String field, String table, String dBBean){
	
	dBTriple = new String[3];
	dBTriple[0] = field;
	dBTriple[1] = table;
	dBTriple[2] = dBBean;
	dbBinds.put(prop, dBTriple);
    }
    
    public void removeBindItem(String prop) {
	dbBinds.remove(prop);
    }
    
    public String[] getBindItem(String prop) {
	return (String[])dbBinds.get(prop);
    }

    public Hashtable getdbBinds() {
	return dbBinds;
    }
	
    final public String getDesignName() {
	return DesignName;
    }
	
    final public void setDesignName(String name) {
	beanbox.renameWrapper(DesignName, name, this);
	DesignName = name;
	if (child instanceof ALabel)
	    ((ALabel)child).setLabel(DesignName);
    }
    // command listener interface for binding support
    
    public void Command (FCEvent e){
	if (e.getCommand() != null)
	    return; // we don't work with text command
	BListManager lm;
	if (e.getParam1() != null && (cachedbeanname == null || !e.getParam1().equals(cachedbeanname))) {
	    cachedwrapper =	beanbox.findWrapperWithName(e.getParam1());
	    cachedbeanname = e.getParam1();
	}
	Object [] values;
	switch (e.getIDCommand()){
	case Constants.CMDBuildBindingList:
	    lm = new BListManager(beanbox);
	    lm.prepareExchangeData((Hashtable)e.getParam2(), null, null);
	    lm = null; 
	    return;
	case Constants.CMDGetBindedElements:
	    lm = new BListManager(beanbox);
	    lm.prepareExchangeData(null, getDesignName(), (Hashtable)e.getParam2());
	    lm = null; 
	    return;
	case Constants.CMDLoadPage:
	    new FormLoadThread(e.getParam1(), beanbox);
	    return;
	}
	if (cachedwrapper == null)
	    return;
	switch (e.getIDCommand()){
	case Constants.CMDSetPropertyFor:
	    values = (Object [])e.getParam2();
	    cachedwrapper.setPropertyValue((String)values[0], values[1]);
	    return;
	case Constants.CMDGetPropertyFor:
	    values = (Object [])e.getParam2();
	    values[0] = cachedwrapper.getPropertyValue((String)values[0]);
	    return;
	case Constants.CMDSetConvPropertyFor:
	    values = (Object [])e.getParam2();
	    cachedwrapper.setPropValWithConversion((String)values[0], values[1]);
	    return;
	}
	if (!(cachedwrapper.getBean() instanceof IRowProperties))
	    return;
	IRowProperties table = (IRowProperties)cachedwrapper.getBean();
	switch (e.getIDCommand()){
	case Constants.CMDNextRow:
	    table.setNextRow(true);
	    return;
	case Constants.CMDSetRow:
	    try {
		table.setRow(Integer.parseInt((String)e.getParam2()));
	    } catch (NumberFormatException e1) {
		table.setRow(0);
	    } catch (NullPointerException e1) {
		table.setRow(0);
	    } catch (ClassCastException e1) {
		table.setRow(0);
	    }
	    return;
	case Constants.CMDTruncByCurrent:
	    table.setNextRow(false);
	    return;
	}
    }
    //-----------------------------------------------------------
    
    /**
    * get the AWT component used to represent the wrapped bean.
    */
    public Component getChild() {
	return child;
    }
    
    public void doLayout() {
	child.doLayout();
	//		sizeToFit();
	child.repaint();
    }
    
    private void sizeToFit() {
	if (childHasStupidPreferredSize()) {
	    return;
	}
	Dimension d = getPreferredSize();
	setSize(d.width, d.height);
    }
    
    
    public void setActive(boolean isActive) {
	active = isActive;
	repaint();
    }
    
    public boolean isSelected(){
	return selected;
    }
    
    public void setSelected(boolean on) {
	selected = on;
    }
    
    public boolean isKeepInPlace() {
	return keepinplace;
    }
    
    public void setKeepInPlace(boolean on) {
	keepinplace = on;
    }
    
    /**
    * getPreferredSize
    *
    * Returns the preferred size of this wrapper including
    * the border widths.
    */
    public Dimension getPreferredSize() {
	Dimension childSize = getPreferredChildSize();
	
	if (childHasStupidPreferredSize()) {
	    childSize = child.getSize();
	}
	int width = childSize.width;
	int height = childSize.height;
	if (width == 0 && height == 0) {
	    childSize = child.getSize();
	    width = childSize.width;
	    height = childSize.height;
	}
	// Make sure the child is at least its minimum size.
	Dimension minSize = getMinimumChildSize();
	if (minSize.height > height) {
	    height = minSize.height;
	}
	if (minSize.width > width) {
	    width = minSize.width;
	}
	width += (2 * borderWidth);
	height += (2 * borderWidth);
	return new Dimension(width, height);
    }
    
    /**
    * getMinimumSize
    *
    * Returns the minimum size of this wrapper including
    * the border widths.
    */
    public Dimension getMinimumSize() {
	Dimension minSize = getMinimumChildSize();
	minSize.width += 2*borderWidth;
	minSize.height += 2*borderWidth;
	return minSize;
    }
    
    /**
    * getPreferredChildSize
    *
    * Returns the prefered size of the child without
    * the border of the wrapper.
    */
    private Dimension getPreferredChildSize() {
	return child.getPreferredSize();
    }
    
    /**
    * getMinimumChildSize
    *
    * Returns the minimum size of the child without
    * the border of the wrapper.
    */
    private Dimension getMinimumChildSize() {
	Dimension size;
        if (MS_AWT_UIHOST != null && MS_AWT_UIHOST.isInstance(child)) {
            try {
                Method m = MS_UICOMPONENT.getMethod("getMinimumSize", new Class[0]);
                return (Dimension)m.invoke(md_getHeader.invoke(child, VOID_PARAM), VOID_PARAM);
            } catch(Throwable t) {
                t.printStackTrace();
            }
            return new Dimension(1,1);
        } else 
	    return child.getMinimumSize();
    }
    
    private boolean childHasStupidPreferredSize() {
	// We do a special check for demented behaviour from empty Panels
	// for the sake of applets.  If an applet does not explicitly
	// specify a dimension, then we allow it to be resized arbitrarily
	// rather than limiting it to the tiny size returned by the FlowLayout
	if (child instanceof Panel) {
	    Container cont = (Container)child;
	    LayoutManager lay = cont.getLayout();
	    if (cont.getComponentCount() == 0 && lay instanceof FlowLayout) {
		FlowLayout flow = (FlowLayout) lay;
		Dimension cd = getPreferredChildSize();
		Dimension fd = flow.preferredLayoutSize(cont);
		if (cd.width == fd.width && cd.height == fd.height) {
		    return true;
		}
	    }
	}
	return false;
    }
    
    public void setBounds(int x, int y, int width, int height) {
	super.setBounds(x, y, width, height);
	if(isDesignTime())
	    child.setBounds(borderWidth, borderWidth,
	    width-(2*borderWidth), height-(2*borderWidth));
	else
	    child.setBounds(0, 0, width, height);
	child.validate();
    }
    
    /**
    * getBorderWidth
    *
    * Returns the predefined border width for the design mode.
    * Returns 0 for the run mode.
    */
    public final int getBorderWidth() {
	if(isDesignTime())
	    return borderWidth;
	else
	    return 0;
    }
    
    /**
    * setBorders
    *
    * Borders are added in the design time mode.
    * No borders are added in the run time mode.
    */
    public void updateBorders() {
	Rectangle bbox = child.getBounds();
	Rectangle bbox0 = getBounds();
	boolean needsValidation = true;
	if(isDesignTime()) {	// Use borders for the design mode.
	    int dw = borderWidth << 1;
	    if(bbox0.width != bbox.width + dw || bbox0.height != bbox.height + dw) {
		super.setBounds(bbox0.x - borderWidth, bbox0.y - borderWidth,
		    bbox.width + dw, bbox.height + dw);
		child.setLocation(borderWidth, borderWidth);
	    }
	    else
		needsValidation = false;
	}
	else {	// No borders for the run time mode.
	    if(bbox0.width != bbox.width || bbox0.height != bbox.height) {
		super.setBounds(bbox0.x + borderWidth, bbox0.y + borderWidth,
		    bbox.width, bbox.height);
		child.setLocation(0, 0);
	    }
	    else
		needsValidation = false;
	}
	
	if(needsValidation)
	    validate();
    }
    
    public Rectangle getRealBounds(){
	Rectangle bbox = child.getBounds();
	Rectangle bbox0 = getBounds();
	if(isDesignTime()){
	    bbox.x = bbox0.x+borderWidth;
	    bbox.y = bbox0.y+borderWidth;
	}else{
	    bbox.x = bbox0.x;
	    bbox.y = bbox0.y;
	}
	
	return bbox;
    }
    
    public void setRealBounds(Rectangle r){
	setBounds(r);
	r.x = 0;
	r.y = 0;
	child.setBounds(r);
	updateBorders();
    }
    
    // Note that to avoid deadlocks, paint is *not* synchronized
    public void paint(Graphics g) {
	if (isDesignTime())
	    if (active) {
	    // no checking if size less than border
	    int width = getSize().width;
	    int height = getSize().height;
	    g.setColor(Color.black);
	    g.fillRect(0, 0, hashBarWidth, hashBarWidth);
	    g.fillRect(0, height-hashBarWidth, hashBarWidth, hashBarWidth);
	    g.fillRect(width-hashBarWidth, 0, hashBarWidth, hashBarWidth);
	    g.fillRect(width-hashBarWidth, height-hashBarWidth, hashBarWidth, hashBarWidth);
	    if (width > (hashBarWidth*3 + hashBarWidth*2)) {
		// creating horizontal middle point
		g.fillRect(width/2-hashBarWidth/2, 0, hashBarWidth, hashBarWidth);
		g.fillRect(width/2-hashBarWidth/2, height-hashBarWidth, hashBarWidth, hashBarWidth);
	    }
	    if (height > (hashBarWidth*3 + hashBarWidth*2)) {
		// creating vertical middle point
		g.fillRect(0, height/2-hashBarWidth/2, hashBarWidth, hashBarWidth);
		g.fillRect(width-hashBarWidth, height/2-hashBarWidth/2, hashBarWidth, hashBarWidth);
	    }
	} else if (selected){
	    int width = getSize().width;
	    int height = getSize().height;
	    
	    getHashBars(this);
	    
	    // Draw the bounding hasbox as a set of images.
	    
	    // First draw the top and bottom bars.
	    int nudge = 2 * hashBarWidth;
	    int bottomNudge = - ((height-hashBarWidth) % nudge);
	    for (int x = 0; x < width; x += hashBarLength) {
		g.drawImage(xHashBar, x, 0, null);
		g.drawImage(xHashBar, x + bottomNudge, height-hashBarWidth, null);
	    }
	    // Now draw the left and right bars.
	    int rightNudge = - ((width-hashBarWidth) % nudge);
	    for (int y = 0; y < height; y += hashBarLength) {
		g.drawImage(yHashBar, 0, y, null);
		g.drawImage(yHashBar, width-hashBarWidth, y+rightNudge, null);
	    }
	}
	// draw child
	super.paint(g);
    }
    
    public synchronized void activate() {
	active = true;
	repaint();
	child.requestFocus();
    }
    
    public synchronized void deactivate() {
	active = false;
	repaint();
    }
    
    public synchronized void doMouseStuff(MouseEvent evt) {
	// Code to handle drawing the resize/move cursor.
	
	if (!active || !Beans.isDesignTime()) {
	    return;
	}
	Component parent = getParent();
	
	int x = evt.getX();
	int y = evt.getY();
	int id = evt.getID();
	
	int lowX = borderWidth;
	int highX = getSize().width - borderWidth;
	int lowY = borderWidth;
	int highY = getSize().height - borderWidth;
	int midX =  highX /2;
	int midY =  highY /2;
	boolean onTheBorder = ((evt.getSource() == this) &&
	    (x < lowX || x > highX || y < lowY || y > highY));
	
	Cursor newCursor = defaultCursor;
	
	if (onTheBorder) {
	    if (id == MouseEvent.MOUSE_PRESSED) {
		// Check if we need to resize or move.
		if (cursor == nwResizeCursor || 
		    cursor == swResizeCursor ||
		    cursor == neResizeCursor ||
		    cursor == seResizeCursor) {
		    beanbox.startResize(this, x, y, cursor);
		} else if (cursor == nResizeCursor || 
		    cursor == wResizeCursor ||
		    cursor == eResizeCursor ||
		    cursor == sResizeCursor) {
		    beanbox.startResize(this, x, y, cursor);
		} else if (cursor == moveCursor) {
		    beanbox.startMove(this, x, y);
		} 
		
	    } else if (id == MouseEvent.MOUSE_ENTERED 
		|| id == MouseEvent.MOUSE_MOVED) {
		
		// If we're in a corner set the cursor to indicate a
		// resize, otherwise a move.
		lowX += resizeDelta;
		lowY += resizeDelta;
		highX -= resizeDelta;
		highY -= resizeDelta;
		
		if (x < lowX && y < lowY) {
		    newCursor = nwResizeCursor;
		} else if (x < lowX && y > highY) {
		    newCursor = swResizeCursor;
		} else if (x > highX && y < lowY) {
		    newCursor = neResizeCursor;
		} else if (x > highX && y > highY) {
		    newCursor = seResizeCursor;
		} else if (x > midX && x < (midX+borderWidth) && y > highY) {
		    newCursor = sResizeCursor;
		} else if (x > midX && x < (midX+borderWidth) && y < lowY) {
		    newCursor = nResizeCursor;
		} else if (x > highX && y > midY && y < (midY+borderWidth)) {
		    newCursor = eResizeCursor;
		} else if (x < lowX && y > midY && y < (midY+borderWidth)) {
		    newCursor = wResizeCursor;
		} else {
		    newCursor = moveCursor;
		}
	    }
	}
	
	if (newCursor != cursor) {
	    cursor = newCursor;
	    // To workaround a bug on JDK 1.1, we avoid changing the user
	    // visible cursor on a straight JDK 1.1 system.
	    if (!isJDK("1.1_Final")) {
		setCursor(cursor);
	    }
	}
    }
    
    
    
    /**
    * show or hide all the "invisible beans".
    */
    public static void setShowingInvisibleBeans(boolean show)  {
	showingInvisibleBeans = show;
	for (int i = 0; i < invisibleWrappers.size(); i++) {
	    ((Wrapper)invisibleWrappers.elementAt(i)).setVisible(show);
	}
    }
    
    public static boolean isShowingInvisibleBeans() {
	return showingInvisibleBeans;
    }
    
    //----------------------------------------------------------------------
    
    // Mouse listener methods for target bean.
    // We also report all mouse events to our containing beanbox.
    
    public void mouseClicked(MouseEvent evt) {
	doMouseStuff(evt);
	if(beanbox != null)
	    beanbox.mouseClicked(evt);
    }
    
    public void mousePressed(MouseEvent evt) {
	doMouseStuff(evt);
	if(beanbox != null)
	    beanbox.mousePressed(evt);
    }
    
    public void mouseReleased(MouseEvent evt) {
	doMouseStuff(evt);
	if(beanbox != null)
	    beanbox.mouseReleased(evt);
    }
    
    public void mouseEntered(MouseEvent evt) {
	doMouseStuff(evt);
	if(beanbox != null)
	    beanbox.mouseEntered(evt);
    }
    
    public void mouseExited(MouseEvent evt) {
	doMouseStuff(evt);
	if(beanbox != null)
	    beanbox.mouseExited(evt);
    }
    
    public void mouseDragged(MouseEvent evt) {
	doMouseStuff(evt);
	if(beanbox != null)
	    beanbox.mouseDragged(evt);
    }
    
    public void mouseMoved(MouseEvent evt) {
	doMouseStuff(evt);
	if(beanbox != null)
	    beanbox.mouseMoved(evt);
    }
    
    //----------------------------------------------------------------------
    public void keyTyped(KeyEvent e) {
    }
    
    public void keyPressed(KeyEvent e) {
    }
    
    public void keyReleased(KeyEvent e) {
	if (e.getKeyCode() == KeyEvent.VK_DELETE)
	    beanbox.keyDelete();
    }
    
    //----------------------------------------------------------------------    
    /**
    * @deprecated  Provided for backwards compatibility only.
    * We support old-style event handling as a backward compatibility
    * feature when we're wrapping a bean using the old event model.
    */
    
    public boolean handleEvent(Event evt) {
	//		if (evt.target instanceof UIComponent)
	//			return false;
	Component source = (Component) evt.target;
	MouseEvent me;
	int x = evt.x;
	int y = evt.y;
	if (source == child) {
	    // remap coordinate to child's space.
	    x = x - borderWidth;
	    y = y - borderWidth;
	}
	if (evt.id == Event.MOUSE_DOWN) {
	    me = new MouseEvent(source, MouseEvent.MOUSE_PRESSED,
		0, 0, x, y, 0, false);
	    mousePressed(me);
	} else if (evt.id == Event.MOUSE_UP) {
	    if (sawMouseDown) {
		me = new MouseEvent(source, MouseEvent.MOUSE_CLICKED,
		    0, 0, x, y, 0, false);
		mouseClicked(me);
	    }
	    sawMouseDown = false;
	    me = new MouseEvent(source, MouseEvent.MOUSE_RELEASED,
		0, 0, x, y, 0, false);
	    mouseReleased(me);
	} else if (evt.id == Event.MOUSE_MOVE) {
	    me = new MouseEvent(source, MouseEvent.MOUSE_MOVED,
		0, 0, x, y, 0, false);
	    mouseMoved(me);
	} else if (evt.id == Event.MOUSE_ENTER) {
	    sawMouseDown = false;
	    me = new MouseEvent(source, MouseEvent.MOUSE_ENTERED,
		0, 0, x, y, 0, false);
	    mouseEntered(me);
	} else if (evt.id == Event.MOUSE_EXIT) {
	    sawMouseDown = false;
	    me = new MouseEvent(source, MouseEvent.MOUSE_EXITED,
		0, 0, x, y, 0, false);
	    mouseExited(me);
	} else if (evt.id == Event.MOUSE_DRAG) {
	    me = new MouseEvent(source, MouseEvent.MOUSE_DRAGGED,
		0, 0, x, y, 0, false);
	    mouseDragged(me);  
	} else if (evt.id == Event.KEY_RELEASE) {
	    KeyEvent ke = new KeyEvent(source, KeyEvent.KEY_TYPED, evt.when, evt.modifiers, evt.key);
	    keyReleased(ke);
	}
	else
	    return false;
	return true;
    }
    
    
    // Check if our child needs to use the old AWT event mnodel.
    private synchronized boolean useNewEventModel() {
	// Check our cache first.
	Class clz; 
        if (MS_AWT_UIHOST != null && MS_AWT_UIHOST.isInstance(child)) {
            try {
	    clz = md_getHeader.invoke(child, VOID_PARAM).getClass();
            } catch(Throwable t) {
                t.printStackTrace();
            }
            clz = Object.class;
        } else 
	    clz = child.getClass();
	Boolean b = (Boolean) eventModelCache.get(clz);
	if (b != null) {
	    return b.booleanValue();
	}
	
	// We check whether the bean has any public methods that take
	// old AWT event objects.  If so, we assume it wants the
	// old event model, otherwise we assume it wants the new.
	boolean useNew = true;
	if (!(MS_AWT_UIHOST != null && MS_AWT_UIHOST.isInstance(child))) // do not check for AFC
	    try {
	    while (useNew && clz != null) {
		
		java.lang.reflect.Method methods[] = clz.getDeclaredMethods();
		
		for (int i = 0; i < methods.length; i++) {
		    java.lang.reflect.Method m = methods[i];
		    int mods = m.getModifiers();
		    if (!java.lang.reflect.Modifier.isPublic(mods)) {
			// Skip non=public methods.
			continue;
		    }
		    Class params[] = m.getParameterTypes();
		    if (params.length > 0 && params[0] == java.awt.Event.class) {
			// First arg is java.awt.Event.  We assume this is an
			// old-style AWT event handler method.
			useNew = false;
			break;
		    }
		}
		clz = clz.getSuperclass();
		if (clz.getName().indexOf("java.") == 0) {
		    // We've reached a java.* class, so we're done.
		    break;
		}
	    }
	} catch (Exception ex) {
	    System.err.println(CLASSID+" useOldEventModel caught: " + ex);
	}
	
	eventModelCache.put(clz, new Boolean(useNew));
	return useNew;
    }
    
    
    void listenForMiceAndKey(boolean enable) {
	// If the child uses the old event mode we rely on receiving events
	// through handleEvent.  Otherwise we use the new event model and
	// register explicit listeners for mouse events.
	if (!useNewEventModel()) {
	    return;
	}
	if (enable) {
	    this.addMouseListener(this);
	    this.addMouseMotionListener(this);
	    this.addKeyListener(this);
	    child.addMouseListener(this);
	    child.addMouseMotionListener(this);
	    child.addKeyListener(this);
	} else {
	    this.removeMouseListener(this);
	    this.removeMouseMotionListener(this);
	    this.removeKeyListener(this);
	    child.removeMouseListener(this);
	    child.removeMouseMotionListener(this);
	    child.removeKeyListener(this);
	}
    }
    
    private static synchronized void getHashBars(Component c) {
	if (xHashBar != null) {
	    return;
	}
	int len = hashBarLength + 20;
	
	xHashBar = c.createImage(len, hashBarWidth);
	yHashBar = c.createImage(hashBarWidth, len);
	
	Polygon poly = new Polygon();
	Graphics g = xHashBar.getGraphics();
	for (int i = 0; i < 4; i++) {
	    poly.addPoint(0,0);
	}
	poly.ypoints[2] = hashBarWidth;
	poly.ypoints[3] = hashBarWidth;
	
	for (int x = 0; x < (hashBarWidth+len); x += hashBarWidth) {
	    // draw alternate dark gray and light gray stripes.
	    if (((x/hashBarWidth)%2) == 0) {
		g.setColor(Color.gray);
		poly.xpoints[0] = x;
		poly.xpoints[1] = x + hashBarWidth/2;
		poly.xpoints[2] = x - hashBarWidth/2;
		poly.xpoints[3] = x - hashBarWidth;
		g.fillPolygon(poly);
		g.setColor(Color.white);
		poly.xpoints[0] = x + hashBarWidth/2;
		poly.xpoints[1] = x + hashBarWidth;
		poly.xpoints[2] = x;
		poly.xpoints[3] = x - hashBarWidth/2;
		g.fillPolygon(poly);
	    } else {
		g.setColor(Color.lightGray);
		poly.xpoints[0] = x;
		poly.xpoints[1] = x + hashBarWidth;
		poly.xpoints[2] = x;
		poly.xpoints[3] = x - hashBarWidth;
		g.fillPolygon(poly);
	    }
	}
	
	g = yHashBar.getGraphics();
	poly.xpoints[0] = 0;
	poly.xpoints[1] = hashBarWidth;
	poly.xpoints[2] = hashBarWidth;
	poly.xpoints[3] = 0;
	
	for (int y = 0; y < (hashBarWidth+len); y += hashBarWidth) {
	    // draw alternate dark gray and light gray stripes.
	    if (((y/hashBarWidth)%2) == 0) {
		g.setColor(Color.gray);
		poly.ypoints[0] = y;
		poly.ypoints[1] = y - hashBarWidth;
		poly.ypoints[2] = y - hashBarWidth/2;
		poly.ypoints[3] = y + hashBarWidth/2;
		g.fillPolygon(poly);
		g.setColor(Color.white);
		poly.ypoints[0] = y + hashBarWidth/2;
		poly.ypoints[1] = y - hashBarWidth/2;
		poly.ypoints[2] = y;
		poly.ypoints[3] = y + hashBarWidth;
		g.fillPolygon(poly);
	    } else {
		g.setColor(Color.lightGray);
		poly.ypoints[0] = y;
		poly.ypoints[1] = y - hashBarWidth;
		poly.ypoints[2] = y;
		poly.ypoints[3] = y + hashBarWidth;
		g.fillPolygon(poly);
	    }
	}
    }
    
    private static boolean isJDK(String version) {
	return System.getProperty("java.version").equals(version);
    }
}

// Class to hold state on event listener hookup for which this
// Wrapper's bean is a source.

class WrapperEventTarget implements Serializable {
    static final long serialVersionUID = 4831901854891942741L;
    
    String eventSetName;
    Object targetBean;
    Object targetListener;
}

class ItselfRemover implements IControlEnvCallBack {
    private BasicSpace bc;
    private String name;
    private Object itself;
    ItselfRemover(BasicSpace bc, String name, Object itself){
	this.bc = bc;
	this.name  = name;
	this.itself = itself;
    }
    public boolean changeListener(String src, String evt, String trg){
	if (name.equals(trg)){
	    Wrapper w = bc.findWrapperWithName(src);
	    w.removeListenerFor(evt, trg);
	    EventSetDescriptor []esd;
	    Object bean = w.getBean();
	    try {
		BeanInfo bi = Introspector.getBeanInfo(bean.getClass());
		esd = bi.getEventSetDescriptors();
		
	    } catch (IntrospectionException ex) {
		System.err.println(Wrapper.CLASSID+" couldn't introspect "+ex);
		return true;
	    }
	    Method m = null;
	    for (int i = 0; i < esd.length; i++) {
		if (evt.equals(esd[i].getDisplayName())){
		    m = esd[i].getRemoveListenerMethod();
		    break;
		}
	    }
	    Object arg[] = {itself};
	    // good approach if we cache somewhere pointers to beans
	    try {
		m.invoke(bean, arg);
	    }catch(Exception e){
		System.err.println(Wrapper.CLASSID+" change listener "+e);
		}/*catch(IllegalAccessException e){
		 System.err.println("CB: "+e);
		 }catch(IllegalArgumentException e){
		 System.err.println("CB: "+e);
		 }catch(InvocationTargetException e){
		 System.err.println("CB: "+e);
	}*/
	}
	return true;
    }
    
    public boolean changeBinding(String db, String fld, String ctrl){
	return false;
    }
}

class NameChanger implements IControlEnvCallBack {
    private String oldname;
    NameChanger(String oldname){
	this.oldname  = oldname;
    }
    public boolean changeListener(String src, String evt, String trg){
	// it can be dangerous, because we can destroy hashtable
	if (src.equals(oldname)){
	}
	if (trg.equals(oldname)){
	}
	return true;
    }
    public boolean changeBinding(String db, String fld, String ctrl){
	return false;
    }
}

class FormLoadThread extends Thread
{
    String filename;
    BasicSpace ws;
    public FormLoadThread(String filename, BasicSpace ws)
    {
        this.filename = filename;
        this.ws = ws;
        start();
    }
    
    public void run()
    {
        ws.loadXML(new File(filename));
    }
}
