/*
 * $File:		BuilderMenu.java$
 * $Author: dmitriy $ 
 * $Date: 2000/11/27 07:35:31 $
 * $Version:    1.0$
 * $Revision: 1.1 $
 * $History:$
 *              1. Added correct work with whitespace nodes.
 * $Description:	Detachable menu $
 *
 * Copyright (C) 1998 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://www.flash.net/~dima/java/
 */
package rogatkin.xbox.creator;

// system packages
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

// Swing
import javax.swing.*;
import javax.swing.event.*;

import rogatkin.xbox.shared.*;

class BuilderMenu extends JMenuBar implements IBuilderMenu, ActionListener {
    private JMenu editMenu;
    private JMenu viewMenu;
    private boolean tearOff = false;
    static final int BKGBEANPOS = 5;
    BuilderMenu() {
	JMenuItem item;
	JMenu fileMenu = new JMenu(CREATORCONSTANTS.File, tearOff);
	String fileOptions[] = {CREATORCONSTANTS.New, CREATORCONSTANTS.Open, CREATORCONSTANTS.Save, CREATORCONSTANTS.SaveAs};
        for (int i=0; i<fileOptions.length; i++){
            fileMenu.add(item = new JMenuItem(fileOptions[i]));
            try {
                Icon icon = new ImageIcon(CREATOR_RES_STRG.image(fileOptions[i]));
                item.setHorizontalTextPosition(JButton.RIGHT);
                item.setIcon(icon);
            } catch (Throwable e) {
            }
            item.addActionListener(this);
        }
        fileMenu.addSeparator();
	String fileSpecOptions[] = {CREATORCONSTANTS.ReloadJars, CREATORCONSTANTS.LoadJar, CREATORCONSTANTS.SaveComponent};
        for (int i=0; i<fileSpecOptions.length; i++){
	    fileMenu.add(item = new JMenuItem(fileSpecOptions[i]));
            item.addActionListener(this);
        }
	JMenu serSubMenu = new JMenu(CREATORCONSTANTS.Serialize);
	String serializeOptions[] = {CREATORCONSTANTS.Component, CREATORCONSTANTS.Load+"...", CREATORCONSTANTS.Save+"..."};
        for (int i=0; i<serializeOptions.length; i++){
            serSubMenu.add(item = new JMenuItem(serializeOptions[i]));
            item.addActionListener(this);
        }
	fileMenu.add(serSubMenu);
        fileMenu.addSeparator();
	fileMenu.add(item = new JMenuItem(CREATORCONSTANTS.Print));
        item.addActionListener(this);
        fileMenu.addSeparator();
	fileMenu.add(item = new JMenuItem(CREATORCONSTANTS.Properties));
        item.addActionListener(this);
        fileMenu.addSeparator();
	fileMenu.add(item = new JMenuItem(CREATORCONSTANTS.Exit));
        item.addActionListener(this);
	add(fileMenu);
	editMenu = new JMenu(CREATORCONSTANTS.Edit, tearOff);
	String editOptions[] = {CREATORCONSTANTS.Cut,
	    CREATORCONSTANTS.Copy,
	    CREATORCONSTANTS.Paste,
	    CREATORCONSTANTS.Delete};
        for (int i=0; i<editOptions.length; i++){
	    editMenu.add(item = new JMenuItem(editOptions[i]));
            item.addActionListener(this);
        }
	editMenu.addSeparator();
	editMenu.add(item = new JCheckBoxMenuItem(CREATORCONSTANTS.BackgroundBean));
	item.addActionListener(this);
	editMenu.add(item = new JMenuItem(CREATORCONSTANTS.BringForward));
	item.addActionListener(this);
	editMenu.add(item = new JMenuItem(CREATORCONSTANTS.BringBackward));
	item.addActionListener(this);
	editMenu.addSeparator();
	editMenu.add(item = new JMenuItem(CREATORCONSTANTS.Options));
	item.addActionListener(this);
	add(editMenu);
	
	viewMenu = new JMenu(CREATORCONSTANTS.View, tearOff);
	buildViewMenu();
	add(viewMenu);
	JMenu helpMenu = new JMenu(CREATORCONSTANTS.Help, tearOff);
	String helpOptions[] = {CREATORCONSTANTS.About+"...", "Visit xBox homepage", "Documentation..."};
        for (int i=0; i<helpOptions.length; i++){
	    helpMenu.add(item = new JMenuItem(helpOptions[i]));
	    item.addActionListener(this);
        }
	add(helpMenu);
    }
    
    public void buildViewMenu() {
        JMenuItem item;
	viewMenu.add(item = new JMenuItem());
        item.addActionListener(this);
	viewMenu.add(item = new JMenuItem());
        item.addActionListener(this);
	viewMenu.addSeparator();
	ButtonGroup group = new ButtonGroup();
	viewMenu.add(item = new  JRadioButtonMenuItem(CREATORCONSTANTS.AllInOne));
	item.setSelected(true);
        item.addActionListener(this);
	group.add(item);
	viewMenu.add(item = new  JRadioButtonMenuItem(CREATORCONSTANTS.TwoPane));
        item.addActionListener(this);
	group.add(item);
	viewMenu.add(item = new  JRadioButtonMenuItem(CREATORCONSTANTS.Detached));
        item.addActionListener(this);
	group.add(item);
	viewMenu.addSeparator();
	group = new ButtonGroup();
	viewMenu.add(item = new  JRadioButtonMenuItem(CREATORCONSTANTS.Metal));
        item.addActionListener(this);
	group.add(item);
	item.setSelected(true);
	viewMenu.add(item = new  JRadioButtonMenuItem(CREATORCONSTANTS.Motif));
        item.addActionListener(this);
	group.add(item);
	viewMenu.add(item = new  JRadioButtonMenuItem(CREATORCONSTANTS.Windows));
	item.addActionListener(this);
	group.add(item);
	viewMenu.addSeparator();
	// add bean specific commands
    }
    
    public void updateViewMenu(boolean cond1, boolean cond2) {
	viewMenu.getItem(0).setText(cond1?CREATORCONSTANTS.DISABLEDESIGNMODE:CREATORCONSTANTS.ENABLEDESIGNMODE);
	viewMenu.getItem(1).setText(cond2?CREATORCONSTANTS.SHOWINVISIBLEBEANS:CREATORCONSTANTS.HIDEINVISIBLEBEANS);
    }
    
    public void enablePaste(boolean on) {
	JMenuItem paste = (JMenuItem)editMenu.getItem(2);
	if (paste != null)
	    paste.setEnabled(on);
    }
    
    public void setBackgroundBean(boolean on) {
	JMenuItem bkgbean = (JMenuItem)editMenu.getItem(BKGBEANPOS);
	if (bkgbean != null)
	    bkgbean.setSelected(on);
    }
    
    public void enableBackgroundBean(boolean on) {
	JMenuItem bkgbean = (JMenuItem)editMenu.getItem(BKGBEANPOS);
	if (bkgbean != null)
	    bkgbean.setEnabled(on);
    }
    
    public void removeCustom() {
	// Remove any bean-specific menu items from the edit menu.
	for (int i = 10; i < editMenu.getItemCount(); i++) {
	    editMenu.remove(i);
	}
    }
    
    public void addEditDivider(){
	editMenu.addSeparator();
    }
    
    public void addEditCustom(String name){
        JMenuItem item;
	editMenu.add(item = new JMenuItem(name));
        item.addActionListener(this);
    }
    
    public void actionPerformed(ActionEvent a){
	fireCommandEvent(a.getActionCommand());
    }
    
    void fireCommandEvent(String com){
	Vector l;
	FCEvent s = new FCEvent (this, com);
	synchronized (this)
	{
	    l = (Vector) CommandListeners.clone();
	}
	for (int c=0;c<l.size();c++)
	{
	    IFCEvListener dal = (IFCEvListener)l.elementAt(c);
	    dal.Command(s);
	}
    }
    private Vector CommandListeners = new Vector();
    
    public synchronized void addCommandListener(IFCEvListener l)
    {
	CommandListeners.addElement(l);
    }
    
    public synchronized void removeCommandListener(IFCEvListener l)
    {
	CommandListeners.removeElement(l);
    }
    
}