/*
 * File:	EditToolPanel.java
 * Author:	Dmitry Rogatkin
 * Date:	03/12/98
 *
 * Description:	it implements tool palette for basic editing operations
 *				
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.creator;

// system packages
import java.awt.*;
import java.util.Vector;

// Swing
import javax.swing.*;

// our packages
import rogatkin.xbox.ui.*;
import rogatkin.xbox.io.ImageHelper;
import rogatkin.xbox.shared.*;
import rogatkin.xbox.core.*;
import rogatkin.xbox.ut.OrderedQueue;

public class EditToolPanel extends ToolBtnPalette implements ItemSelectedListener {
    private WorkSpace ws;

    public EditToolPanel(WorkSpace ws) {
        super(4);
        this.ws = ws;
        Vector stri = new Vector(EditButtonsImages.length);
        Vector imgi = new Vector(EditButtonsImages.length);
        for (int i=0; i<EditButtonsImages.length; i++){
            stri.addElement(EditButtonsText[i]);
            imgi.addElement(ImageHelper.loadImage(EditButtonsImages[i]+".gif"));
        }
        setMenuItems(stri);
        setMenuImages(imgi);
        init();
        addItemSelectedListener(this);
    }

    public void selected (ItemSelectedEvent e) {
        setSelection(e.getItemSelected());
        switch (e.getItemSelected()) {
            case 0:
                ws.getToolBox().setBeanSelected(false);
                break;
            case 1:
                alignBottom();
                break;
            case 2:
                alignLeft();
                break;
            case 3:
                alignRight();
                break;
            case 4:
                alignTop();
                break;
            case 5:
                centerHoriz();
                break;
            case 6:
                centerVert();
                break;
            case 8:
                equalVertGap();
                break;
            case 7:
                equalHorizGap();
                break;
            case 11:
                equalVertSize();
                break;
            case 10:
                equalHorizSize();
                break;
            case 9:
                equalSizes();
                break;
            case 12:
                showGrid();
                break;
            case 13:
                showLimitters();
                break;
            case 14:
                copyColor();
                break;
            default:
        }
        setSelection(0);
    }
    
    /* next release will be working with selection item array
    *
    */
    void alignBottom() {
        Wrapper w;
        int lowesty = 0;
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected() && w.getLocation().y + w.getSize().height > lowesty)
                lowesty = w.getLocation().y + w.getSize().height;
        }

        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected())
                w.setLocation(w.getLocation().x, lowesty - w.getSize().height);
        }
    }

    void alignTop() {
        Wrapper w;
        int highesty = -1;
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected() && w.getLocation().y < highesty || highesty == -1)
                highesty = w.getLocation().y;
        }

        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected())
                w.setLocation(w.getLocation().x, highesty);
        }
    }

    void alignLeft() {
        Wrapper w;
        int leftestx = -1;
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected() && w.getLocation().x < leftestx || leftestx == -1)
                leftestx = w.getLocation().x;
        }

        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected())
                w.setLocation(leftestx, w.getLocation().y);
        }
    }

    void alignRight() {
        Wrapper w;
        int rightestx = 0;
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected() && w.getLocation().x + w.getSize().width > rightestx)
                rightestx = w.getLocation().x + w.getSize().width;
        }

        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected())
                w.setLocation(rightestx - w.getSize().width, w.getLocation().y);
        }
    }

    void centerVert() {
        Wrapper w;
        Dimension sz = ws.getContainer().getSize();
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            w.setLocation(w.getLocation().x, (sz.height -
                                              w.getSize().height)/2);
        }
    }
    void equalVertGap() {
        Wrapper w;
        int highesty = -1;
        int lowesty = 0;
        int contcount = 0;
        int totalheight = 0;
        OrderedQueue oq = new OrderedQueue();
        oq.setOrder(OrderedQueue.ANCESSOR);
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected()) {
                if (w.getLocation().y < highesty || highesty == -1)
                    highesty = w.getLocation().y;
                if (w.getLocation().y + w.getSize().height > lowesty)
                    lowesty = w.getLocation().y + w.getSize().height;
                contcount++;
                totalheight += w.getSize().height;
                oq.push(w.getLocation().y, w);
            }
        }
        if (contcount < 3)
            return;
        int gap = ((lowesty - highesty) - totalheight) / (contcount -1);
        int lasty = 0;
        while((w=(Wrapper)oq.pop())!= null){
            if (lasty <= 0) {
                lasty = w.getLocation().y - gap;
            }
            w.setLocation(w.getLocation().x, lasty+gap);
            lasty = w.getLocation().y+w.getSize().height;
        }
    }
    void equalHorizGap() {
        Wrapper w;
        int highestx = -1;
        int lowestx = 0;
        int contcount = 0;
        int totalwidth = 0;
        OrderedQueue oq = new OrderedQueue();
        oq.setOrder(OrderedQueue.ANCESSOR);
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected()) {
                if (w.getLocation().x < highestx || highestx == -1)
                    highestx = w.getLocation().x;
                if (w.getLocation().x + w.getSize().width > lowestx)
                    lowestx = w.getLocation().x + w.getSize().width;
                contcount++;
                totalwidth += w.getSize().width;
                oq.push(w.getLocation().x, w);
            }
        }
        if (contcount < 3)
            return;
        int gap = ((lowestx - highestx) - totalwidth) / (contcount -1);
        int lastx = 0;
        while((w=(Wrapper)oq.pop())!= null){
            if (lastx <= 0) {
                lastx = w.getLocation().x - gap;
            }
            w.setLocation(lastx+gap, w.getLocation().y);
            lastx = w.getLocation().x+w.getSize().width;
        }
    }
    void equalVertSize() {
        Wrapper w = ws.getLastFocused();
        if (w == null)
            return;
        if (!w.isSelected())
            return;
        Dimension lead = w.getSize();
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected()) {
                w.setSize(w.getSize().width, lead.height);
            }
        }
    }
    void equalHorizSize() {
        Wrapper w = ws.getLastFocused();
        if (w == null)
            return;
        if (!w.isSelected())
            return;
        Dimension lead = w.getSize();
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected()) {
                w.setSize(lead.width, w.getSize().height);
            }
        }
    }
    void centerHoriz() {
        Wrapper w;
        Dimension sz = ws.getContainer().getSize();
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            w.setLocation((sz.width - w.getSize().width)/2, w.getLocation().y);
        }
    }
    void equalSizes() {
        Wrapper w = ws.getLastFocused();
        if (w == null)
            return;
        if (!w.isSelected())
            return;
        Dimension lead = w.getSize();
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected()) {
                w.setSize(lead.width, lead.height);
            }
        }
    }
    void showGrid() {
    }
    void showLimitters() {
    }
    void copyColor() {
        Wrapper w = ws.getLastFocused();
        if (w == null)
            return;
        if (!w.isSelected())
            return;
        Color color = w.getChild().getBackground();
        for (int i = 0; i < ws.getComponentCount(); i++) {
            w = (Wrapper) ws.getComponent(i);
            if (w.isSelected()) {
                w.getChild().setBackground(color);
                w.getChild().repaint();
            }
        }
    }

    private static String EditButtonsImages[] = { "SelArrow", "AlignBottom", "AlignLeft",
                                                  "AlignRight", "AlignTop", "CenterHorizontal", "CenterVertical",
                                                  "GapHorizontal", "GapVertical", "SizeBoth", "SizeHorizontal", "SizeVertical",
                                                  "Grid", "Region", "DupColor" };
    private static String EditButtonsText[] = { "Align controls to bottom edge",
                                                "Contol selection",
                                                "Align controls to left edge",
                                                "Align controls to right edge",
                                                "Align controls to top edge",
                                                "Center controls horizontally",
                                                "Center controls vertically",
                                                "Make gaps equally by horizontally",
                                                "Make gaps equally by vertically",
                                                "Make both sizes equal",
                                                "Make horizontal size equal",
                                                "Make vertical size equal",
                                                "Show grid",
                                                "Show limitters",
                                                "Copy color"
    };
}