/*
 * File:	FormPanelHolder.java
 * Author:	Dmitry Rogatkin
 * Date:	04/15/98
 *
 * Description:	FormPanelHolder implements a holder for FormPanel for the Designer
 *				
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.creator;

// system packages
import java.awt.*;
import java.awt.event.*;

// Swing
import javax.swing.*;
import javax.swing.border.*;

import rogatkin.xbox.core.*;
import rogatkin.xbox.shared.IStatusInfo;
import rogatkin.xbox.ui.FreePanel;

public class FormPanelHolder extends FreePanel implements IStatusInfo {
    private StatusPanel sp;
    public FormPanelHolder(BasicPanel bp) {
        setLayout(new BorderLayout());
        setBorder(BorderFactory.createLoweredBevelBorder());
        bp.setSize(300, 200);
        add(bp.getGui(), BorderLayout.CENTER);
        add(sp = new StatusPanel(), BorderLayout.SOUTH);
    }

    public void updateCursorCoord(int x, int y) {
        sp.updateCoord(x, y);
    }

    public void updateRectangleCoord(Rectangle box) {
        sp.updateRectangle(box);
    }
}

class StatusPanel extends JPanel {
    JLabel mousecoord;
    JLabel rectanglecoord;
    public StatusPanel(){
        setLayout(null);
        setBorder(BorderFactory.createEtchedBorder());
        setPreferredSize(new Dimension(160, 26));
        add(rectanglecoord = new JLabel("", SwingConstants.CENTER));
        add(mousecoord = new JLabel("", SwingConstants.CENTER));
        mousecoord.setBorder(BorderFactory.createLoweredBevelBorder());
        rectanglecoord.setBorder(BorderFactory.createLoweredBevelBorder());
        rectanglecoord.setBounds(10, 2, 164, 22);
        mousecoord.setBounds(180, 2, 88, 22);
        updateCoord(0, 0);
        updateRectangle(new Rectangle(0, 0, 0, 0));
    }
    
    void updateCoord(int x, int y) {
        mousecoord.setText(""+x+":"+y);
    }

    void updateRectangle(Rectangle r) {
        rectanglecoord.setText("("+r.x+":"+r.y+")-("+r.width+":"+r.height+")");
    }

}