/*
 * File:	PropertRendererEditor.java
 * Author:	Dmitry Rogatkin
 * Date:	03/30/98
 *
 * Description:	implements renderer/editor for property table cells
 *				
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.creator;

import java.awt.*;
import java.awt.event.*;
import java.util.EventObject;
import java.beans.*;

// Swing
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.border.BevelBorder;

import rogatkin.xbox.core.Wrapper;
import rogatkin.xbox.io.ImageHelper;
// TODO: rewrite to simplify

class PropertRendererEditor implements TableCellEditor, TableCellRenderer {
    Object editedvalue = null;
    JLabel label = new JLabel();
    SelectorEditor selector = new SelectorEditor();
    TextEditor text = new TextEditor();
    PropertyEditPanel canvas = new PropertyEditPanel();
    DesignNameEdit designname = new DesignNameEdit();

    Icon icodropdown = new ImageIcon(ImageHelper.getImageURL("DropDown.gif"));
    Icon iconwrench = new ImageIcon(ImageHelper.getImageURL("wrench.gif"));
    JLabel customize = new JLabel(iconwrench);
    CustomizerEditor customizeditor = new CustomizerEditor();

    public PropertRendererEditor(){
        customize.setIcon(iconwrench);
    }
    // renderer interface
    public Component getTableCellRendererComponent(JTable table,
                                                   Object value,
                                                   boolean isSelected,
                                                   boolean hasFocus,
                                                   int row,
                                                   int column){
        if (value instanceof PropertyElement){
            label.setIcon(null);
            if (column == 0){
                label.setText(((PropertyElement)value).getPropertyName());
                return label;
            } else {
                switch(((PropertyElement)value).getPropertyType()){
                    case PropertyElement.CANVAS:
                        return new CanvasRenderer(((PropertyElement)value).getPropertyEditor());
                    case PropertyElement.CHOICE:
                        label.setIcon(icodropdown);
                    case PropertyElement.TEXT:
                        label.setText(((PropertyElement)value).getPropertyEditor().getAsText());
                        return label;
                    case PropertyElement.NAME:
                        label.setText(((Wrapper)((PropertyElement)value).getPropertyValue()).getDesignName());
                        return label;
                    case PropertyElement.CUSTOMIZER:
                        return customize;
                    default:
                        return null;
                }
            }
        }
        return null;
    }

    // editor interface
    public Component getTableCellEditorComponent(JTable table,
                                                 Object value,
                                                 boolean isSelected,
                                                 int row,
                                                 int column){
        editedvalue = value;
        if (value instanceof PropertyElement){
            if (column == 0){
                return null;
            } else {
                switch(((PropertyElement)value).getPropertyType()){
                    case PropertyElement.CANVAS:
                        canvas.init(((PropertyElement)value).getPropertyEditor(), this);
                        return canvas;
                    case PropertyElement.CHOICE:
                        selector.init(((PropertyElement)value).getPropertyEditor(), this);
                        return selector;
                    case PropertyElement.TEXT:
                        text.init(((PropertyElement)value).getPropertyEditor(), this);
                        return text;
                    case PropertyElement.NAME:
                        designname.init((Wrapper)((PropertyElement)value).getPropertyValue(), this);
                        return designname;
                    case PropertyElement.CUSTOMIZER:
                        customizeditor.init((Customizer)((PropertyElement)value).getPropertyValue(), this);
                        return customizeditor;
                    default:
                        return null;
                }
            }
        }
        return null;
    }
    // cell editor interface
    public Object getCellEditorValue() {
        return editedvalue;
    }

  /*  void setCellEditorValue(Object _value) {
         editedvalue = _value;
    }*/

    public boolean isCellEditable(EventObject anEvent){
        return true;
    }
    
    public boolean shouldSelectCell(EventObject anEvent){
        //if (anEvent==null)
        //editcomponent.requestFocus();
        return anEvent instanceof AWTEvent;
    }

    public boolean stopCellEditing(){
        canvas.stopCellEditing();
        customizeditor.stopCellEditing();
        switch(((PropertyElement)editedvalue).getPropertyType()){
          case PropertyElement.TEXT:
            ((PropertyElement)editedvalue).getPropertyEditor().setAsText(text.getText());
            break;
          case PropertyElement.NAME:
            ((Wrapper)((PropertyElement)editedvalue).getPropertyValue()).setDesignName(designname.getText());
            break;
        }
        fireEditingStopped();
        return true;
    }
    
    public void cancelCellEditing(){
        fireEditingCanceled();
    }
    //
    //  Handle the event listener bookkeeping
    //
    public void addCellEditorListener(CellEditorListener l) {
        listenerlist.add(CellEditorListener.class, l);
    }
    
    public void removeCellEditorListener(CellEditorListener l) {
        listenerlist.remove(CellEditorListener.class, l);
    }
    
    /**
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see Eventlistenerlist
     */
    protected void fireEditingStopped() {
        // Guaranteed to return a non-null array
        Object[] listeners = listenerlist.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i]==CellEditorListener.class) {
                // Lazily create the event:
                if (changeEvent == null)
                    changeEvent = new ChangeEvent(this);
                ((CellEditorListener)listeners[i+1]).editingStopped(changeEvent);
            }	       
        }
    }
    
    /**
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see Eventlistenerlist
     */
    protected void fireEditingCanceled() {
        // Guaranteed to return a non-null array
        Object[] listeners = listenerlist.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for (int i = listeners.length-2; i>=0; i-=2) {
            if (listeners[i] == CellEditorListener.class) {
                // Lazily create the event:
                if (changeEvent == null)
                    changeEvent = new ChangeEvent(this);
                ((CellEditorListener)listeners[i+1]).editingCanceled(changeEvent);
            }	       
        }
    }

    private EventListenerList listenerlist = new EventListenerList();
    transient protected ChangeEvent changeEvent = null;
}

class CanvasRenderer extends JPanel{
    
    public CanvasRenderer(PropertyEditor pe) {
        editor = pe;
    }

    public void paint(Graphics g) {
        Rectangle box = new Rectangle(2, 2, getSize().width - 4, getSize().height - 4);
        editor.paintValue(g, box);
    }
    private PropertyEditor editor;
}

class SelectorEditor extends JComboBox /*implements ItemListener*/ {
    //public void itemStateChanged(ItemEvent evt) 
    // if (evt.getStateChange() == ItemEvent.SELECTED && !firstopen) 
    
    public SelectorEditor() {
        //addItemListener(this);
    }
    
    public void init(PropertyEditor pe, PropertRendererEditor shell){
        this.pe = pe;
        this.shell = shell;
        firstopen = true;
        if (getItemCount() > 0)
            removeAllItems();
        String tags[] = pe.getTags();
        for (int i = 0; i < tags.length; i++) {
            addItem(tags[i]==null?"":tags[i]);
        }
        if (pe.getAsText() != null)
            // This is a noop if the getAsText is not a tag.
            setSelectedItem(pe.getAsText());
        else {
            setSelectedIndex(0);
            pe.setAsText((String)getSelectedItem());
        }
        firstopen = false;
    }
    
    public void contentsChanged(ListDataEvent e){
        super.contentsChanged(e);
        if (!firstopen){
            String s = (String)getSelectedItem();
            if (s!=null)
                pe.setAsText(s);
            shell.stopCellEditing();
        }
    }

    private PropertyEditor pe;
    private PropertRendererEditor shell;
    private boolean firstopen;
}

class TextEditor extends JTextField {
    
    public TextEditor() {
        KeyListener keylistener = new KeyAdapter() {
            public void keyReleased(KeyEvent k) {
               if (k.getKeyCode() == KeyEvent.VK_ENTER){
                  editor.setAsText(getText());
                  shell.stopCellEditing();
               } else if (k.getKeyCode() == KeyEvent.VK_ESCAPE){
                  shell.cancelCellEditing();
               }
            }
         };
        addKeyListener(keylistener);
        //addMouseListener(MouseListener);
    }
    
    public void init(PropertyEditor pe, PropertRendererEditor shell) {
        editor = pe;
        this.shell = shell;
        setText(pe.getAsText());
    }

    private PropertyEditor editor;
    private PropertRendererEditor shell;
}

class DesignNameEdit extends JTextField {
    
    public DesignNameEdit() {
        KeyListener keylistener = new KeyAdapter() {
            public void keyReleased(KeyEvent k) {
            if (k.getKeyCode() == KeyEvent.VK_ENTER) {
            wrapper.setDesignName(getText());
            // more complex work waits as here
            // 1. find and replace event source in eventMap
            // 2. do replacing listeners for all event sources for us
            // we can do similar work for binding too
            // probably, we don't allow change this name if the control
            // envolved to event fairy, before user pull out it
            shell.stopCellEditing();
            } else if (k.getKeyCode() == KeyEvent.VK_ESCAPE){
            shell.cancelCellEditing();
            }
            }
            };
        addKeyListener(keylistener);
    }

    void init(Wrapper wrapper, PropertRendererEditor shell) {
        this.wrapper = wrapper;
        this.shell = shell;
        setText(wrapper.getDesignName());
        setToolTipText("Specify unique name of control");
    }

    private PropertRendererEditor shell;
    private Wrapper wrapper;
}

class CustomizerEditor extends JComponent {

    public void init(Customizer customizer, PropertRendererEditor shell){
        if (cw != null) {
            cw.dispose();
            cw = null;
        }
        this.customizer = customizer;
        this.shell = shell;
    }

    public void paint(Graphics g) {
        if (cw == null)
            cw = new CustomizerWindow(customizer, this, getLocationOnScreen());
        else
            cw.toFront();
    }
    
    public boolean stopCellEditing() {
        if (cw != null)
            cw.dispose();
        else
            return true;
        cw = null;
        return shell.stopCellEditing();
    }
    
    private CustomizerWindow cw;
    private Customizer customizer;
    private PropertRendererEditor shell;
}

class CustomizerWindow extends JWindow implements ActionListener {
    private JButton doneButton;
    private Component body;
    private CustomizerEditor ce;
    private final static int vPad = 5;
    private final static int hPad = 4;

    public CustomizerWindow(Customizer customizer, CustomizerEditor ce, Point l){
        getContentPane().setLayout(null);
        this.ce = ce;
        getContentPane().add(body = (Component)customizer);
        getRootPane().setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        doneButton = new JButton(CREATORCONSTANTS.Done);
        doneButton.addActionListener(this);
        getContentPane().add(doneButton);
        setLocation(l.x, l.y);
        pack();
        setVisible(true);
    }
    
    public void actionPerformed(ActionEvent evt) {
        // Button down.
        ce.stopCellEditing();
    }

    public void doLayout() {
        super.doLayout();
        Insets ins = getInsets();
        Dimension bodySize = body.getPreferredSize();
        Dimension buttonSize = doneButton.getPreferredSize();
        
        int width = ins.left + 2*hPad + ins.right + Math.max(bodySize.width, buttonSize.width);
        int height = ins.top + 3*vPad + ins.bottom + bodySize.height +
                     buttonSize.height;
        
        body.setBounds(ins.left+hPad, ins.top+vPad,
                       bodySize.width, bodySize.height);
        
        doneButton.setBounds((width-buttonSize.width)/2,
                             ins.top+(2*vPad) + bodySize.height,
                             buttonSize.width, buttonSize.height);
        
        getRootPane().setSize(width-ins.left-ins.right, height-ins.bottom-ins.top);
        setSize(width, height);
    }
}
