/*
 * File:	PropertyEditPanel.java
 * Author:	Dmitry Rogatkin
 * Date:	4/1/98
 *
 * Description:	PropertyEditPanel - table editor for properties
 * 
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.creator;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.border.BevelBorder;

public class PropertyEditPanel extends JPanel{
	
    public PropertyEditPanel() { 
    }
    
    public void init(PropertyEditor pe, PropertRendererEditor shell){
        if (pep != null) {
            pep.dispose();
            pep = null;
        }
        this.editor = pe;
        this.shell = shell;
    }

    public void paint(Graphics g) {
        if (pep == null)
            pep = new PropertyEditWindow(editor, this, getLocationOnScreen());
        else
            pep.toFront();
    }
    
    public boolean stopCellEditing(){
        if (pep != null)
            pep.dispose();
        else
            return true;
        pep = null;
        return shell.stopCellEditing();
    }

    private PropertyEditor editor;
    private PropertyEditWindow pep;
    private PropertRendererEditor shell;
}

class PropertyEditWindow extends JWindow implements ActionListener {
    private JButton doneButton;
    private Component body;
    private PropertyEditPanel panel;
    private final static int vPad = 5;
    private final static int hPad = 4;

    public PropertyEditWindow(PropertyEditor pe, PropertyEditPanel panel, Point l){
        this.panel = panel;
        getContentPane().setLayout(null);
        if (pe != null)
            getContentPane().add(body = pe.getCustomEditor());

        getRootPane().setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
        doneButton = new JButton(CREATORCONSTANTS.Done);
        doneButton.setToolTipText("Press when you finish editing the property");
        doneButton.addActionListener(this);
        getContentPane().add(doneButton);
        setLocation(l.x, l.y);
        pack();
        doLayout();
        setVisible(true);
    }
    
    public void actionPerformed(ActionEvent evt) {
        // Button down.
        panel.stopCellEditing();
    }

    public void doLayout() {
        super.doLayout();
        Insets ins = getInsets();
        Dimension bodySize = body.getPreferredSize();
        Dimension buttonSize = doneButton.getPreferredSize();
        
        int width = ins.left + 2*hPad + ins.right + Math.max(bodySize.width, buttonSize.width);
        int height = ins.top + 3*vPad + ins.bottom + bodySize.height +
                     buttonSize.height;
        
        body.setBounds(ins.left+hPad, ins.top+vPad,
                       bodySize.width, bodySize.height);
        
        doneButton.setBounds((width-buttonSize.width)/2,
                             ins.top+(2*vPad) + bodySize.height,
                             buttonSize.width, buttonSize.height);
        
        getRootPane().setSize(width-ins.left-ins.right, height-ins.bottom-ins.top);
        setSize(width, height);
    }
}