/*
 * File:	PropertySheet.java
 * Author:	Dmitry Rogatkin
 * Date:	03/2/98
 *
 * Description:	PropertySheet implements 
 *				
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.creator;

// Java
import java.beans.*;
import java.lang.reflect.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Hashtable;
import java.util.Vector;

// Sun's Swing
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.tree.*;

// Rogatkin's Soft
import rogatkin.xbox.ui.*;
import rogatkin.xbox.core.*;
import rogatkin.xbox.shared.IBindingList;
import rogatkin.xbox.shared.IRowProperties;
import rogatkin.xbox.io.ImageHelper;

public class PropertySheet extends FreePanel
        implements ItemListener, ListSelectionListener, PropertyChangeListener,
         TreeSelectionListener, MouseListener, CellEditorListener, IPropertySheet {

    private Class MS_UI_COMPONENT;

    private boolean started;
    private JTabbedPane tv;
    // properties tab
    final private JTable cvprops = new JTable();
    // events tab
    private	JList listenerslist;
    private JTree eventslists;
    // db binding tab
    private JPanel cvbinds;
    private JComboBox propForDb;
    private	JList dbcntrls;
    private JList dbfields;
    
    private JLabel status;
    
    public PropertySheet(WorkSpace beanContainer, Wrapper target) {
        try {
            MS_UI_COMPONENT = Class.forName("com.ms.ui.UIComponent");
        } catch (ClassNotFoundException cnfe) {
            // will be null
        }
        if(beanContainer != null)
	    setParentFrame(beanContainer.getFrame());
	beanContainer.setPropertySheet(this);
	setLayout(new BorderLayout());
	setMinimumSize(new Dimension(100, 32));
	setPreferredSize(new Dimension(220, 360));
	
	initModelsAndViews();
	
	try {
            cvprops.setShowGrid(true);
	    PropertRendererEditor pre = new PropertRendererEditor();
	    pre.addCellEditorListener(this);
	    cvprops.setDefaultRenderer(PropertyElement.class, pre);
            cvprops.setDefaultEditor(PropertyElement.class, pre);
	    cvprops.setModel(new PropDataModel());
            cvprops.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
	    
	    cvbinds = new JPanel();
	    cvbinds.setLayout(new BorderLayout());
	    cvbinds.add(propForDb, "North");
	    cvbinds.add(new JSplitPane(JSplitPane.VERTICAL_SPLIT, 
		new ScrollPaneWithPrefSize(dbcntrls, 76, 48), new ScrollPaneWithPrefSize(dbfields, 76, 48)), "Center");
	    
	    tv = new JTabbedPane(SwingConstants.BOTTOM);
	    tv.addTab("Properties", new ScrollPaneForResizableTable(cvprops));
	    tv.addTab("Events", new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                new ScrollPaneWithPrefSize(eventslists, 76, 148), new ScrollPaneWithPrefSize(listenerslist, 76, 68)));
	    tv.addTab("Binding", cvbinds);
	    
	    // Add to this panel
	    add(tv, "Center");
	    add(status, "South");
	    
	    // Add listeners
	    // probably it will be changed to individual listener objects
	    listenerslist.addMouseListener(this);
	    dbcntrls.addListSelectionListener(this);
	    propForDb.addItemListener(this);
	    dbfields.addListSelectionListener(this);
	    eventslists.addTreeSelectionListener(this);
	    
	} catch (Exception e){
	    error("PropertySheet: problem in creating elements", e);
	}
	setTarget(target);
	started = true;
    }
    
    void initModelsAndViews(){
	// creation event tree view
	eventslists = new JTree();
	eventslists.setMinimumSize(new Dimension(0, 0));
	//      eventslists.setSelectionMode(SINGLESELECT);
	
	// creation a list of data collector components
	dbcntrls = new JList();
        dbcntrls.setCellRenderer(new FormComponentListRenderer());
	dbcntrls.setMinimumSize(new Dimension(0, 0));
	dbcntrls.setModel(new FormComponentsListModel());
	
	// creation a list of listeners
	listenerslist = new JList();
        listenerslist.setMinimumSize(new Dimension(0, 0));
	listenerslist.setCellRenderer(new FormComponentListRenderer());
        listenerslist.setSelectionMode(ListSelectionModel.SINGLE_SELECTION/*MULTIPLE_INTERVAL_SELECTION*/);
	listenerslist.setModel(new FormComponentsListModel());
	
	// creation a list of database fields
	dbfields = new JList();
	dbfields.setCellRenderer(new FormComponentListRenderer());
	dbfields.setMinimumSize(new Dimension(0, 0));
	dbfields.setModel(new FormComponentsListModel());
	
	
	status = new JLabel("Initializing...", SwingConstants.LEFT);
	
	// creation a binded properties combobox
	propForDb = new JComboBox();
        propForDb.setToolTipText("Select binded property");
	//		propForDb.setBackground(SystemColor.window);
	
    }
    
    public synchronized void setTarget(Wrapper tw) {
	tv.setVisible(true); //@@@
	status.setText("None's selected");
        processEvents = false;
	wrapper = tw;
	if (tw == null) {
	    tv.setVisible(false); //@@@
	    return;
	}
        
        if (tw.getWorkSpace() == null)
            return; // work around of Java2 (jdk1.2.2) JIT bug (volatile doesn't work here)
	
	WorkSpace beanBox = (WorkSpace)wrapper.getWorkSpace();

	// clear contents of lists
	if (propForDb.getItemCount() > 0)
	    propForDb.removeAllItems();

        FormComponentsListModel dbclm = (FormComponentsListModel)dbcntrls.getModel();
	dbclm.removeAllElements();
        dbclm.add(NOT_CONNECTED, null, null);
	fillListCondition(beanBox, dbclm, new isListBean());

	target = tw.getBean();
        PropDataModel propDataModel = (PropDataModel)cvprops.getModel();
        cvprops.editingStopped(new ChangeEvent(cvprops));
	propDataModel.removeAllElements(); // maybe re-new is faster
	propDataModel.add(new PropertyElement(null, wrapper, "<name>", PropertyElement.NAME, null));
	
	try {
	    BeanInfo bi = Introspector.getBeanInfo(target.getClass());
	    properties = bi.getPropertyDescriptors();
	    esd = bi.getEventSetDescriptors();
	    methodMap.clear();
	    BeanDescriptor bd = bi.getBeanDescriptor();
	    Class customizerclass = bd.getCustomizerClass();
	    if (customizerclass != null) {
		try {
		    Customizer customizer = (Customizer) customizerclass.newInstance();
		    customizer.setObject(target);
		    propDataModel.add(new PropertyElement(null, customizer, "Customizer", PropertyElement.CUSTOMIZER, null));
		} catch (Exception ex) {
		    System.err.println("Couldn't instantiate customizer: " + ex);
		}
	    }
	} catch (IntrospectionException ex) {
	    error("PropertySheet: Couldn't introspect", ex);
	    return;
	}
	
	for (int i = 0; i < properties.length; i++) {
	    // Don't display hidden or expert properties.
	    if (properties[i].isHidden() || properties[i].isExpert())
		continue;
	    
	    String name = properties[i].getDisplayName();
	    Class type = properties[i].getPropertyType();
	    Method getter = properties[i].getReadMethod();
	    Method setter = properties[i].getWriteMethod();
	    
	    // Only display read/write properties.
	    if (getter == null || setter == null)
		continue;
	    
	    try {
		Object args[] = { };
		Object value = getter.invoke(target, args);
		
		PropertyEditor editor = null;
		Class pec = properties[i].getPropertyEditorClass();
		if (pec != null) {
		    try {
			editor = (PropertyEditor)pec.newInstance();
		    } catch (Exception ex) {
			// Drop through.
		    }
		}
		if (editor == null) {
		    editor = PropertyEditorManager.findEditor(type);
		}
		
		if (editor == null) {
		    // last chanse, check for String[]
		    if (type == String[].class) {
			editor = new StringArrayPropEditor();
			// find corresponding String property if any and set its value
			if (name.endsWith("s")){
			    String corrname = name.substring(0, name.length()-1);
			    for (int j = 0; j < properties.length; j++) {
				if (corrname.equals(properties[j].getDisplayName()) &&
				    properties[j].getPropertyType() == String.class){
				    Method getter_j = properties[j].getReadMethod();
				    Object args_j[] = { };
				    Object value_j = getter_j.invoke(target, args_j);
				    editor.setAsText((String)value_j);
				    break;
				}
			    }
			}
		    } else if (type == int[].class) {
			editor = new IntArrayPropEditor();
		    } else // If we can't edit this component, skip it.
			continue;
		}
		
		// Don't try to set null values:
		if (value == null) {
		    continue; // skip null value
		}
		
		editor.setValue(value);
		// don't use standard mechanism, so comment out it
		// editor.addPropertyChangeListener(this);
		// Now figure out how to display it...
		int editortype;
		if (editor.isPaintable() && editor.supportsCustomEditor()) {
		    editortype = PropertyElement.CANVAS;
		} else if (editor.getTags() != null) {
		    editortype = PropertyElement.CHOICE;
		    if (type == boolean.class)
			propForDb.addItem(name);
		} else if (editor.getAsText() != null) {
		    editortype = PropertyElement.TEXT;
		    propForDb.addItem(name);
		} else {
		    System.err.println("Skipping non-displayable property " + name);
		    continue;
		}
		propDataModel.add(new PropertyElement(editor, value, name, editortype, properties[i]));
	    } catch (Exception ex) {
		System.err.println("Skipping property " + name + " : " + ex);
	    }
	}
	if (Beans.isInstanceOf(target, IRowProperties.class)) {
	    String[] addproperties = ((IRowProperties)target).getPropertyNames();
	    for (int i = 0; i < addproperties.length; i++)
		if (addproperties[i] != null && addproperties[i].length() > 0)
		propForDb.addItem(addproperties[i]);
	}
	cvprops.revalidate();
	cvprops.repaint();
	
	propForDb.repaint();
	
        DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("Events");
	boolean bindable = false;
	for (int i = 0; i < esd.length; i++) {
	    DefaultMutableTreeNode tree = new DefaultMutableTreeNode(esd[i].getDisplayName());
	    rootNode.add(tree);
	    Method methods[] = esd[i].getListenerMethods();
	    for (int j = 0; j < methods.length; j++) {
		tree.add(new DefaultMutableTreeNode(methods[j].getName()));
	    }
	    if (esd[i].getName().equals("propertyChange")) {
		bindable = true;
	    }
	    methodMap.put(esd[i].getDisplayName(), esd[i]);
	}
        eventslists.setModel(new DefaultTreeModel(rootNode));
        eventslists.expandPath(new TreePath(rootNode));
	rebuldListenersList(null);
	
	status.setText(tw.getBeanName());
	
	if (optionShowPropTab)
            tv.setSelectedIndex(0);
	
        processEvents = true;
	//setVisible(true);
    }

    int fillListCondition(WorkSpace beanBox, FormComponentsListModel list, IAllowing cond){
	String classname;
	int cn = 0;
	if (beanBox != null)
	{
	    int count = beanBox.getComponentCount();
	    for (int i = 0; i < count; i++) {
		if (beanBox.getComponent(i) instanceof Wrapper )
		{
		    Wrapper w = (Wrapper) beanBox.getComponent(i);
		    if (cond.satisfied(w)){
			classname = w.getBeanName();
			int ix = classname.lastIndexOf('.');
			if (ix >= 0)
			    classname = classname.substring(ix+1);
			list.add(w.getDesignName(), cond.getIcon(w), classname);
		    }
		    cn++;
		}
	    }
	}
	return cn;
    }
    
    // for PropertyChangeListener
    public void propertyChange(PropertyChangeEvent evt){
	// I use other mechanism informing that a property has been changed
	// see below
    }
    
    synchronized public void editingStopped(ChangeEvent e){
	PropDataModel propDataModel = (PropDataModel)cvprops.getModel();
	PropertRendererEditor pre = (PropertRendererEditor)e.getSource();
	PropertyElement pel = (PropertyElement)pre.getCellEditorValue();
	if (pel.getPropertyType() == PropertyElement.NAME) {
	    return;
	}
	// update changes property to target
	PropertyEditor pe = pel.getPropertyEditor();
	Object value = null;
	if (pe instanceof StringArrayPropEditor){
	    // array couldn't be changed
	    pe.getAsText();
	    String names = pel.getPropertyName();
	    if (names.endsWith("s")){
		String name = names.substring(0, names.length()-1);
		for (int j = 0; j < propDataModel.getRowCount(); j++){
		    if (((PropertyElement)propDataModel.getValueAt(j, 1)).getPropertyName().equals(name) &&
			((PropertyElement)propDataModel.getValueAt(j, 1)).getPropertyDescriptor().getPropertyType() == String.class){
			value = pe.getAsText();
			pel = (PropertyElement)propDataModel.getValueAt(j, 1);
			// set flag repaint of this row is pending
			cvprops.tableChanged(new TableModelEvent(propDataModel, j));
			break;
		    }
		}
		if (value == null)
		    return;
	    }
	    
	} else
            value = (pe != null)?pe.getValue():pe;
	PropertyDescriptor property = pel.getPropertyDescriptor();
        if (value == null || property == null)
            return;
	Method setter = property.getWriteMethod();
	try {
	    Object args[] = { value };
	    args[0] = value;
	    setter.invoke(target, args);
	} catch (java.lang.reflect.InvocationTargetException ex) {
	    if (ex.getTargetException() instanceof PropertyVetoException) {
		warning("Vetoed, reason is: " + ex.getTargetException().getMessage());
	    }
	    else
		error("InvocationTargetException while updating " + 
		property.getName(), ex.getTargetException());
	} catch (Exception ex) {
	    error("Unexpected exception while updating " + 
		property.getName(), ex);
	}
	// check for coresponding changes of other properties
	for (int i = 0; i < propDataModel.getRowCount(); i++) {
	    Object o;
	    pel = (PropertyElement)propDataModel.getValueAt(i, 1);
	    if (pel.getPropertyType() == PropertyElement.NAME)
		continue;
	    try {
		Method getter = pel.getPropertyDescriptor().getReadMethod();
		Object args[] = { };
		o = getter.invoke(target, args);
	    } catch (Exception ex) {
		o = null;
	    }
	    if (o == pel.getPropertyValue() || (o != null && o.equals(pel.getPropertyValue()))) {
		// The property is equal to its old value.
		continue;
	    }
	    pel.setPropertyValue(o);
	    // Make sure we have an editor for this property...
	    if (pel.getPropertyEditor() == null) {
		continue;
	    }
	    // The property has changed!  Update the editor.
	    pel.getPropertyEditor().setValue(o);
	    cvprops.tableChanged(new TableModelEvent(propDataModel, i));
	}
	
	// Make sure the target bean gets repainted.
	((WorkSpace)wrapper.getWorkSpace()).setModified(true);
	if (Beans.isInstanceOf(target, javax.swing.JComponent.class)) {
	    ((javax.swing.JComponent)(Beans.getInstanceOf(target, javax.swing.JComponent.class))).repaint();
	    ((javax.swing.JComponent)(Beans.getInstanceOf(target, javax.swing.JComponent.class))).doLayout();
	} else if (Beans.isInstanceOf(target, Component.class)) {
	    ((Component)(Beans.getInstanceOf(target, Component.class))).repaint();
	} else if (MS_UI_COMPONENT != null && Beans.isInstanceOf(target, MS_UI_COMPONENT)) {
            try {
                Method m = MS_UI_COMPONENT.getMethod("repaint", new Class[] {});
                m.invoke(Beans.getInstanceOf(target, MS_UI_COMPONENT), new Object[0]);
            } catch(Throwable t) {
                t.printStackTrace();
            }
	}
    }
    
    public void editingCanceled(ChangeEvent e){
    }
    
    // display binding for selected property
    synchronized void bindedPropertyChanged() {
	// get selection
	String selection = (String)propForDb.getSelectedItem();
	if (selection == null)
	    return;
	// get binding for selection
	String []dbtriple = wrapper.getBindItem(selection);
	int oldselection = dbcntrls.getSelectedIndex(); //getSelectedIndices();
	if (dbtriple == null) {
	    // no binding for this property
	    dbcntrls.setSelectedIndex(0); //NOT_CONNECTED
	} else {
	    Object []fieldn = ((FormComponentsListModel)dbcntrls.getModel()).find(dbtriple[2]);
	    if (fieldn == null){
		fieldn = ((FormComponentsListModel)dbcntrls.getModel()).add(
		    dbtriple[2], icoExclamation, null);
	    }
	    dbcntrls.setSelectedValue(fieldn, true);
	}
	if (oldselection == dbcntrls.getSelectedIndex())
	    bindedTableChanged();
    }
    
    // display fields for selected table
    synchronized void bindedTableChanged() {
	FormComponentsListModel dbflm = (FormComponentsListModel)dbfields.getModel();
	dbflm.removeAllElements();
	Object []dbn = (Object[])dbcntrls.getSelectedValue();
	if (dbn == null)
	    return; // no selection
	if (!((String)dbn[0]).equals(NOT_CONNECTED)) {
	    // here better use hash table like DesignName - DBBean Object
	    IBindingList da = (IBindingList)wrapper.getWorkSpace().
		findBeanWithNameAndClass((String)dbn[0], IBindingList.class);
	    if (da != null) {
		da.getBuiltList(true);
		if (da.isBuiltList()) {
		    String tablename = da.getTableName();
		    if (tablename == null)
			tablename = "";
		    dbflm.setTableName(tablename);
		    status.setText(da.getTableName());
		    for (int i = 0; i<da.getElementsCount(); i++)
			dbflm.add(da.getElementNames(i), null, da.getElementInfos(i));
		}
	    }
	    String selection = (String)propForDb.getSelectedItem();
	    if (selection == null)
		return;
	    // get binding for selection
	    String []dbtriple = wrapper.getBindItem(selection);
	    if (dbtriple == null)
		return;
	    Object[] fieldn = dbflm.find(dbtriple[0]);
	    if (fieldn == null) {
		fieldn = ((FormComponentsListModel)dbfields.getModel()).add(
		    dbtriple[0], new ImageIcon(ImageHelper.getImageURL("Exclamation.gif")),  null);
	    }
	    dbfields.setSelectedValue(fieldn, true);
	}
	else
	    bindedFieldChanged();
    }
    
    synchronized void bindedFieldChanged() {
	String selection = (String)propForDb.getSelectedItem();
	if (selection == null)
	    return;
	Object[] field = (Object[])dbfields.getSelectedValue();
	Object[] dbase = (Object[])dbcntrls.getSelectedValue();
	if (dbase != null && ((String)dbase[0]).equals(NOT_CONNECTED))
	    wrapper.removeBindItem(selection);
	else if (field != null && dbase != null)
	    wrapper.addBindItem(selection,
	    (String)field[0],
	    ((FormComponentsListModel)dbfields.getModel()).getTableName(),
	    (String)dbase[0]);
    }
    
    // Event processing
    //--MouseListener--
    public void mouseClicked(MouseEvent me) {
	if (me.getSource() == listenerslist){
	    if (me.getClickCount() == 2) {
		listenerslist.setSelectedIndex(
		    listenerslist.locationToIndex(me.getPoint()));
		changeListeners();
		listenerslist.repaint();
	    }
	}
    }
    
    public void mousePressed(MouseEvent e){
    }
    
    public void mouseReleased(MouseEvent e){
    }
    
    public void mouseEntered(MouseEvent e){
    }
    
    public void mouseExited(MouseEvent e){
    }
    
    // better to create separate classes (listeners) for each listbox, but
    // we should provide full access for them to PropertySheet
    
    //--ListSelectionListener--
    public void valueChanged(ListSelectionEvent le){
	if (le.getSource() == dbcntrls){
	    bindedTableChanged();
	} else if (le.getSource() == dbfields){
	    bindedFieldChanged();
	}
    }
    
    //--TreeSelectionListener--
    public void valueChanged(TreeSelectionEvent te){
	if (te.getSource() == eventslists){
	    rebuldListenersList(te.getPath());
	}
    }
    
    public void itemStateChanged(ItemEvent ie){
	if (ie.getSource() == propForDb){
	    if (ie.getStateChange() == ItemEvent.SELECTED)
		bindedPropertyChanged();
	}
    }
    
    // event handlers
    void changeListeners() {
	if (wrapper == null) // should be removed in release
	    return;
	TreePath tp = eventslists.getSelectionPath();
	if (tp == null)
	    return;
	Object[] selected = (Object[])listenerslist.getSelectedValue();
	if (selected == null)
	    return;
	String targetbeanname = (String)selected[0];
	Object arg[] = {wrapper.getWorkSpace().findWrapperWithName(targetbeanname).getBean()};
	Method m = null;
	//try {
	//System.err.println("Looking for "+targetbeanname+" "+arg[0]);
	if (wrapper.hasListenerFor(eventName, targetbeanname)){
	    wrapper.removeListenerFor(eventName, targetbeanname);
	    m = ((EventSetDescriptor)methodMap.get(eventName)).getRemoveListenerMethod();
	    selected[1] = icoUnlinked;
	} else {
	    wrapper.addListenerFor(eventName, targetbeanname);
	    m = ((EventSetDescriptor)methodMap.get(eventName)).getAddListenerMethod();
	    selected[1] = icoLinked;
	    // potential bug is here, because method invocatiom can fault, but status changed
	}
	//} catch(Throwable e){
	//	System.err.println("Exception "+e);
	//}
	if (m == null)
	    return;
	try {
	    m.invoke(wrapper.getBean(), arg);
	} catch(IllegalAccessException e){
	    System.err.println("LISTENERS: "+e);
	} catch(IllegalArgumentException e){
	    System.err.println("LISTENERS: "+e);
	} catch(InvocationTargetException e){
	    System.err.println("LISTENERS: "+e);
	}
    }
    
    void rebuldListenersList(TreePath tp) {
	FormComponentsListModel llm = (FormComponentsListModel)listenerslist.getModel();
	llm.removeAllElements();
	if (wrapper != null && tp != null) // should be removed in release
	{
	    WorkSpace beanBox = (WorkSpace)wrapper.getWorkSpace();
	    eventName = tp.getLastPathComponent().toString();
	    if (eventName != null){
		EventSetDescriptor ed = (EventSetDescriptor)methodMap.get(eventName);
		if (ed != null)
		    fillListCondition(beanBox, llm, 
		    new chkListenerBean(ed.getListenerType(),
		    wrapper, eventName, icoLinked, icoUnlinked));
	    }
	}
	listenerslist.repaint();
    }
    
    private void warning(String s) {
	new ErrorDialog(getParentFrame(), "Warning: " + s);
    }
    
    //----------------------------------------------------------------------
    // Log an error.
    
    private void error(String message, Throwable th) {
	String mess = message + ":\n" + th;
	System.err.println(message);
	// Popup an ErrorDialog with the given error message.
	new ErrorDialog(getParentFrame(), mess);
    }
    
    //----------------------------------------------------------------------
    
    private Object target;
    private Wrapper wrapper;
    private PropertyDescriptor properties[];
    private EventSetDescriptor esd[];
    transient Hashtable methodMap = new Hashtable();
    
    transient Class customizerClass;
    transient String eventName; // selected event name
    
    private boolean processEvents;
    
    static private String NOT_CONNECTED = "<no connection>";
    // options
    public boolean optionShowPropTab = false;
    // stanard images
    private Icon icoExclamation = new ImageIcon(ImageHelper.getImageURL("Exclamation.gif"));
    private Icon icoLinked = new ImageIcon(ImageHelper.getImageURL("Locked.gif"));
    private Icon icoUnlinked = new ImageIcon(ImageHelper.getImageURL("Unlocked.gif"));
}

class FormComponentsListModel extends DefaultListModel {
    public Object[] add(String text, Icon image, String tooltip){
        Object []oa = new Object[3];
        oa[0] = text;
        oa[1] = image;
        oa[2] = tooltip;
        addElement(oa);
	return oa;
    }
    public Object[] find(String key){
	for(int i = 0; i<size(); i++){
	    if(key.equals((String)((Object[])getElementAt(i))[0]))
		return (Object[])getElementAt(i);
	}
	return null;
    }
    
    public void setTableName(String text) {
        tablename = text;
    }
    
    public String getTableName() {
        return tablename;
    }
    
    private String tablename = null;
}

class FormComponentListRenderer extends JLabel implements ListCellRenderer {
    public FormComponentListRenderer() {
	setOpaque(true);
    }
    
    public Component getListCellRendererComponent(JList list,
	Object value, int index, boolean isSelected, boolean cellHasFocus) {
	setText((String)((Object[])value)[0]);
	setIcon((Icon)((Object[])value)[1]);
	setToolTipText((String)((Object[])value)[2]);
	if (list != null){
	    setForeground(isSelected ? list.getSelectionForeground() : list.getForeground());
	    setBackground(isSelected ? list.getSelectionBackground() : list.getBackground());
	}else {
	    setBackground(isSelected ? SystemColor.textHighlight : SystemColor.text);
	    setForeground(isSelected ? SystemColor.textHighlightText : SystemColor.textText);
	}
	return this;
    }
}

interface IAllowing {
    public abstract boolean satisfied(Wrapper w);
    public Icon getIcon(Wrapper w);
}

class isListBean implements IAllowing {
    public boolean satisfied(Wrapper w){
	return Beans.isInstanceOf(w.getBean(), IBindingList.class);
    }
    public Icon getIcon(Wrapper w){
	return null;
    }
}

class chkListenerBean implements IAllowing {
    Class listener;
    Icon locker;
    Icon unlckr;
    Wrapper sourceBean;
    String eventName;
    
    chkListenerBean(Class listener, Wrapper src, String en, Icon loc, Icon unloc){
	sourceBean = src;
	eventName = en;
	locker = loc;
	unlckr = unloc;
	this.listener = listener;
    }
    
    public boolean satisfied(Wrapper w){
	return Beans.isInstanceOf(w.getBean(), listener);
    }
    
    public Icon getIcon(Wrapper w){
	if (sourceBean.hasListenerFor(eventName, w.getDesignName()))
	    return locker;
	else
	    return unlckr;
    }
}

class PropDataModel extends AbstractTableModel {
    final private String propertiesHeadings[] = { "Name", "Value" };
    Vector props = new Vector();
    
    public int getColumnCount() { return 2; }
    
    public int getRowCount() { return props.size();}
    
    public Object getValueAt(int row, int col) {
	return props.elementAt(row);
    }
    
    public void setValueAt(Object value, int row, int col) {
	props.setElementAt(value, row);
    }
    
    public String getColumnName(int col) {
	return propertiesHeadings[col];
    }
    public Class getColumnClass(int c) {
	return PropertyElement.class;
    }
    public boolean isCellEditable(int row, int col){
	if (col == 1)
	    return true;
	else
	    return false;
    }
    
    public void add(PropertyElement el){
	props.addElement(el);
    }
    
    public void removeAllElements(){
	props.removeAllElements();
    }
}

class ScrollPaneWithPrefSize extends JScrollPane {
    ScrollPaneWithPrefSize(Component c, int pw, int ph) {
	super(c);
	setMinimumSize(new Dimension(0, 0));
	// splitter gives first panel preferred size and second the rest
	setPreferredSize(new Dimension(pw, ph));
    }
}

class ScrollPaneForResizableTable extends JScrollPane {
    ScrollPaneForResizableTable(Component c) {
	super(c);
    }
    
    public void setBounds(int x, int y, int w, int h) {
	super.setBounds(x, y, w, h);
	((JTable)getViewport().getView()).sizeColumnsToFit(JTable.AUTO_RESIZE_ALL_COLUMNS);
    }
}
