/*
 * File:	ReadyFormCreator.java
 * Author:	Dmitry Rogatkin
 * Date:	03/29/98
 *
 * Description:	Main module
 *				
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.creator;

// system packages
import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

// our packages
import rogatkin.xbox.core.*;
import rogatkin.xbox.shared.*;
import rogatkin.xbox.ui.*;

public class ReadyFormCreator {

    public ReadyFormCreator() {
	
    }
    
    public static void main(String args[])  {
	// Parse the arguments.
	if (args.length > 4) {
			throw new Error("Incorrect arguments number "+args.length);
	}
        for (int i = 0; i < args.length; i++)
        {
	    if (args[i].equals("-version")) {
		System.out.println(Constants.PRODUCTID+ WorkSpace.getVersionID());
		System.exit(0);
	    } else if (args[i].equals("-jarsdir")){
		i++;
		if (i < args.length)
		    CompManager.setJarsdirParm(args[i]);
	    } else {
		WorkSpace.setTmpDir(args[i]);
	    }
        }
	if (args.length > 0) {
	    if (args[0].equals("-version")) {
		System.out.println(Constants.PRODUCTID+" "+WorkSpace.getVersionID());
		System.exit(0);
	    } 
	}
	try {
	    System.setErr(new PrintStream(new FileOutputStream(Constants.PRODUCTID+".log"), true));
	} catch (IOException e)
	{
	    System.err.println(Constants.PRODUCTID+" Can't redirect the error stream.");
	}
	
	ReadyFormCreator fb = new ReadyFormCreator();
	fb.init();
    }
    
    void init() {
	Starter starter = new StartFrame();
	starter.setDescription("Loading components...");
	
	try {
	    CompManager cm = new CompManager();
	    cm.buildControlList(starter, CompManager.getJarsdirParm(), null);
	    basicpanel = new JBasicPanel();
	    // Set to the design mode.
	    Beans.setDesignTime(true);
	    basicpanel.init(cm);
	    createWorkEnvironment();
        } finally {
	    starter.dispose();
	}
    }
    
    void createWorkEnvironment(){
	ToolFrame tf = new ToolFrame(basicpanel);
	//TODO read configuration and set last sizes
	tf.makeLayout();
	tf.setVisible(true);
    }
    
    private BasicPanel basicpanel;
}