/*
 * File:	ToolBox.java
 * Author:	Dmitry Rogatkin
 * Date:	03/2/98
 *
 * Description:	ToolBox implements 
 *				
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.creator;

/**
 * The ToolBox is a panel that shows icons and ID strings
 * for the available JavaBeans in the current BeanBoxFrame.
 */
// system packages
import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

// our packages
import rogatkin.xbox.ui.*;
import rogatkin.xbox.shared.*;
import rogatkin.xbox.core.*;

public class ToolBox extends FreePanel implements ItemSelectedListener, IToolBox {
	
    private final static int rowHeight = 20;
    private final static int numCols = 3;
    
    private static boolean debug = false;
    private static String jarsDirectory = null;

    private static Cursor crosshairCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
    private static Cursor defaultCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);

	private CompManager ctrlManager;
	private int topPad = 0;
        private int sidePad = 0;
        private Object pendingBean;
        private String pendingBeanLabel;
	private volatile int beanIndex;
	private boolean beanSelected = false;
	private WorkSpace beancontainer;

	//ToolPalette toolBoxMenu = new ToolPalette();
	ToolBtnPalette toolBoxMenu = new ToolBtnPalette(numCols);
    
	public ToolBox(WorkSpace bc) {
		beancontainer = bc;
		ctrlManager = beancontainer.getCompManager();

		setLayout(new BorderLayout());
		setBackground(SystemColor.control);
		setFont(new Font("Dialog", Font.PLAIN, 10));

		try {
			jbInit();
		}
		catch (Exception e) {
			e.printStackTrace();
		}

		// Insets doesn't provide correct infromation while it is hidden
	}

	void jbInit() throws Exception{
    
		toolBoxMenu.addItemSelectedListener(this);

		add(toolBoxMenu, "Center");

		if(ctrlManager != null) {
			toolBoxMenu.setMenuItems(ctrlManager.getControlNames()/*beanLabels*/);
			toolBoxMenu.setMenuImages(ctrlManager.getControlImages()/*beanIcons*/);
			toolBoxMenu.init();
		}
		else
			System.out.println("ToolBox.jbInit: ctrlManager is not initialized.");
    }
	
	public String getProfileString(String keyname){
		// Don't hard code jars directory.
		// jarsDirectory is set by an argument of jview.
		return new String(jarsDirectory);
	}

    //----------------------------------------------------------------------

	public boolean isBeanSelected()
	{
		return beanSelected;
	}
    
	public void setBeanSelected(boolean on)
	{
		beanSelected = on;
		if (isBeanSelected()) {
			toolBoxMenu.setSelection(getCurrentBeanIndex());
			setCursor(crosshairCursor);
			beancontainer.setCursor(crosshairCursor);
		}
		else {
			toolBoxMenu.setSelection(-1);
			setCursor(defaultCursor);
			beancontainer.setCursor(defaultCursor);
		}
		//ToolBoxMenu.repaint();
	}
	
	public int getCurrentBeanIndex()
	{
		return beanIndex;
	}

	public void setCurrentBeanIndex(int r)
	{
		beanIndex = r;
	}
	
	// Item Selected listener methods for this panel.

    public void selected (ItemSelectedEvent e) {
		setCurrentBeanIndex(e.getItemSelected());
		setBeanSelected(true);
		
		synchronized (this) {
			notifyAll();
		}
    }

	//----------------------------------------------------------------------

    public void error(String message, Exception ex) {
		String mess = message + ":\n" + ex;
		System.err.println(message);
		//ex.printStackTrace();
		// Popup an ErrorDialog with the given error message.
		new ErrorDialog(null, mess);
    }

    public void error(String message) {
		String mess = message;
		System.err.println(message);
		// Popup an ErrorDialog with the given error message.
		new ErrorDialog(null, mess);
	}

    static  void debug(String msg) {
		if (debug) {
			System.err.println("FB: "+msg);
		}
    }
}

