/*
 * File:	ToolFrame.java
 * Author:	Dmitry Rogatkin
 * Date:	03/17/98
 *
 * Description:	ToolFrame implements 
 *				
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.creator;

// system packages
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.beans.Beans;

// Swing
import javax.swing.*;
import javax.swing.filechooser.*;

// our packages
import rogatkin.xbox.ui.*;
import rogatkin.xbox.io.*;
import rogatkin.xbox.shared.*;
import rogatkin.xbox.core.*;
import rogatkin.xbox.ut.WindowCloser;
import rogatkin.xbox.ut.INFO;

public class ToolFrame extends JFrame 
    implements IFCEvListener, WindowListener, ActionListener {
    // appearance modes
    final static int ALLINONE = 0;
    final static int TWOFRAME = 1;
    final static int INDIVIDUAL = 2;
    
    public ToolFrame(BasicPanel bp) {
	super(DefCaption);
	setIconImage(ImageHelper.getAppImage());
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
	
	//TODO read mode from last configuration
	mode = ALLINONE;
	this.bp = bp;
	bc = bp.getWorkSpace();
	bc.setFrame(this);
	createMenu();
	createTools();
	ep = new EditToolPanel(bc);
        fh = new FormPanelHolder(bp);
        bc.setStatusInfo(fh);
	fh.setPreferredSize(new Dimension(400, 480));
	setJMenuBar(bm);
        MenuSize = bm.getPreferredSize();
	getContentPane().setLayout(new BorderLayout());
	getContentPane().add(makeToolBar(), BorderLayout.NORTH);
	addWindowListener(this);
    }
    
    void createMenu() {
	try {
	    bm = new BuilderMenu();
	    bm.addCommandListener(bc);
	    bm.addCommandListener(this);
	    bm.enablePaste(false);
	    bm.enableBackgroundBean(false);
	    bc.setMainMenu(bm);
	} catch(Exception e){
	    INFO.println("Exception happened in menu creation "+e);
	}
    }
	
    void createTools() {
	try {
	    tb = new ToolBox(bc);
	    ps = new PropertySheet(bc, null);
	    bc.setToolBox(tb);
	} catch(Exception e) {
	    INFO.println("Exception happened in tools creating "+e);
	}
    }
	
    public void makeLayout() {
	switch(mode) {
	case ALLINONE:
	    for (int i = 0; i < getContentPane().getComponentCount(); i++)
		if (getContentPane().getComponent(i) instanceof JSplitPane)
		getContentPane().remove(i);
	    reattach(tb);
	    reattach(ep);
	    reattach(ps);
	    reattach(fh);
	    
	    JSplitPane tp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, 
		new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, tb, ep), ps);
	    getContentPane().add(new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, tp, fh), BorderLayout.CENTER);
	    //			bc.validate();
	    setResizable(true);
	    setLocation(20, 20);
	    setSize(660, 540);
	    pack();
	    setVisible(true);
	    showTools(true);
	    bc.updateMenuBar();
	    break;
	case TWOFRAME:
	    for (int i = 0; i < getContentPane().getComponentCount(); i++)
		if (getContentPane().getComponent(i) instanceof JSplitPane)
		getContentPane().remove(i);
	    reattach(tb);
	    reattach(ep);
	    reattach(ps);
	    if (fh.getParentFrame() == this) {
		MainFrame frame = new MainFrame ();
		fh.setParentFrame(frame);
		frame.setIconImage(ImageHelper.getAppImage());
		frame.setLayout(new BorderLayout());
		frame.setBackground(SystemColor.control);
		frame.add(fh, BorderLayout.CENTER);			
		new WindowCloser(frame);
		frame.setBounds(170, 20, 400, 550);
		frame.pack();
		frame.setVisible(true);
	    }
	    JSplitPane up = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, tb, ep);
	    getContentPane().add(new JSplitPane(JSplitPane.VERTICAL_SPLIT, up, ps), BorderLayout.CENTER);
	    //			bc.validate();
	    pack();
	    setResizable(true);
	    setLocation(575, 20);
	    setSize(230, 500);
	    break;
	case INDIVIDUAL:
	    for (int i = 0; i < getContentPane().getComponentCount(); i++)
		if (getContentPane().getComponent(i) instanceof JSplitPane)
		getContentPane().remove(i);
	    
	    JFrame ff;
	    if (tb.getParentFrame() == this) {
		ff = new JFrame("Beans palette");
		ff.setSize(130, 146);
		ff.setIconImage(ImageHelper.getAppImage());
		ff.setLocation(575, 98);
		tb.setParentFrame(ff);
		ff.getContentPane().setLayout(new BorderLayout());
		ff.getContentPane().add(tb, BorderLayout.CENTER);
		ff.pack();
		ff.setVisible(true);
		new WindowCloser(ff);
	    }
	    if (ep.getParentFrame() == this) {
		ff = new JFrame("Tools palette");
		ff.setSize(130, 146);
		ff.setLocation(716, 98);
		ff.setIconImage(ImageHelper.getAppImage());
		ff.getContentPane().add(ep, BorderLayout.CENTER);
		ep.setParentFrame(ff);
		ff.pack();
		ff.setVisible(true);
		new WindowCloser(ff);
	    }
	    if (ps.getParentFrame() == this) {
		ff = new JFrame("Property sheet");
		ff.setSize(220, 360);
		ff.setLocation(575, 236);
		ff.setIconImage(ImageHelper.getAppImage());
		ff.getContentPane().add(ps, BorderLayout.CENTER);
		ps.setParentFrame(ff);
		ff.pack();
		ff.setVisible(true);
		new WindowCloser(ff);
	    }
	    if (fh.getParentFrame() == this) {
		MainFrame frame = new MainFrame ();
		fh.setParentFrame(frame);
		frame.setIconImage(ImageHelper.getAppImage());
		frame.setLayout(new BorderLayout());
		frame.setBackground(SystemColor.control);
		frame.add(fh, BorderLayout.CENTER);
		//bc.validate();
		new WindowCloser(frame);
		frame.setBounds(170, 20, 400, 550);
		frame.pack();
		frame.setVisible(true);
	    }
	    KeepSize = getSize();
	    Insets is = getInsets();
	    setLocation(575, 20);
	    setSize(MenuSize.width+is.left+is.right,
		is.top+is.bottom+MenuSize.height+toolbarh);
	    setResizable(false);
	    break;
	}   
    }

    void reattach(FreePanel fp) {
	Frame owner = fp.getParentFrame();
	if (owner != this) {
	    if (owner != null)
		owner.dispose();
	    fp.setParentFrame(this);
	}
    }

    public void showTools(boolean on) {
	if (isVis == on)
	    return;
	isVis = on;
    }
    
    ToolBox getToolBox() {
	return tb;
    }

    JToolBar makeToolBar() {
	JToolBar result = new JToolBar();
	try {
	    JButton btn;
	    result.add(btn = new JButton(new ImageIcon(CREATOR_RES_STRG.image(CREATORCONSTANTS.New))));
	    btn.setToolTipText(CREATORCONSTANTS.New);
	    btn.addActionListener(this);
	    result.addSeparator();
	    result.add(btn = new JButton(new ImageIcon(CREATOR_RES_STRG.image(CREATORCONSTANTS.Open))));
	    btn.setToolTipText(CREATORCONSTANTS.Open);
	    btn.addActionListener(this);
	    result.add(btn = new JButton(new ImageIcon(CREATOR_RES_STRG.image(CREATORCONSTANTS.Save))));
	    btn.setToolTipText(CREATORCONSTANTS.Save);
	    btn.addActionListener(this);
	    toolbarh = result.getPreferredSize().height;
	} catch (Throwable e) {
	    INFO.println("Excpetion "+e+" in toolbar creation");
	}
	return result;
    }

    public boolean canClose() {
	// we don't handle when property changed as modification PropertyChangeListener
	if (bc.isModified()){
	    int iRet = JOptionPane.showConfirmDialog(this,
		"Do you want to save "+bc.getFormName()+bc.defaultXMLExt+"?",
		Constants.PRODUCTID,
		JOptionPane.YES_NO_CANCEL_OPTION); 
	    
	    if (iRet == JOptionPane.YES_OPTION)
	    {
		bc.saveXML(bc.getFormName().equals(bc.untitledName)?askUserFile(true):
	    new File(bc.getFormDirectory(), bc.getFormName()+bc.defaultXMLExt));
	    }
	    else if (iRet == JOptionPane.CANCEL_OPTION || iRet == JOptionPane.CLOSED_OPTION)
	    {
		return false;
	    }
	    return true;
	}
	else
	    return true;
    }

    public File askUserFile(boolean isSave) {
	JFileChooser fc = new JFileChooser(bc.getFormDirectory()+File.separator+
	    bc.getFormName()+bc.defaultXMLExt/*, FileSystemView.getFileSystemView()*/);
	javax.swing.filechooser.FileFilter ff;
	fc.addChoosableFileFilter(ff=new XMLFilter());
	fc.setFileFilter(ff);
	int result = 0;
	if(isSave) {
	    fc.setDialogTitle("Save the form in XML file");
	    result = fc.showSaveDialog(this);
	} else {
	    fc.setDialogTitle("Load a form from XML file");
	    result = fc.showOpenDialog(this);
	}
	if (result != JFileChooser.APPROVE_OPTION)
	    return null;
	File resultFile = fc.getSelectedFile();
	if(isSave) {
	    String name = resultFile.getName();
	    if (name.indexOf('.') < 0) {
		return new File(resultFile.getParent(), name+BasicSpace.defaultXMLExt);
	    }
	}
	return resultFile;
    }

    public void actionPerformed(ActionEvent a) {
	if (a.getSource() instanceof JButton)
	    doMenuItem(((JButton)a.getSource()).getToolTipText());
    }
    
    public void Command (FCEvent e) {
	doMenuItem(e.getCommand());
    }

    void doMenuItem(String it) {
	if (inprocess)
	    return;
	try {
	    inprocess = true;
	    // I devided menu action between
	    if (it == null || it == "")
		return;
	    if (it.equals(CREATORCONSTANTS.Exit)) {
		if (canClose())
		    System.exit(0);
	    } else if (it.equals(CREATORCONSTANTS.New)) {
		if (canClose()) {
		    bc.clearForm();
		    bc.setFormName(bc.untitledName);
		    bc.updateCaption();
		}
	    } else if (it.equals(CREATORCONSTANTS.Detached)) {
		mode = INDIVIDUAL;
		makeLayout();
		//detachAll(true);
	    } else if (it.equals(CREATORCONSTANTS.AllInOne)) {
		mode = ALLINONE;
		makeLayout();
	    } else if (it.equals(CREATORCONSTANTS.TwoPane)) {
		mode = TWOFRAME;
		makeLayout();
	    } else if (it.equals(CREATORCONSTANTS.SaveAs)) {
		bc.saveXML(askUserFile(true));
		bc.setModified(false);
	    } else if (it.equals(CREATORCONSTANTS.Open)) {
		if (canClose())
		    bc.loadXML(askUserFile(false));
	    } else if (it.equals(CREATORCONSTANTS.Save)) {
		//System.err.println("["+bc.getFormDirectory()+ bc.getFormName()+bc.defaultXMLExt+"[");
		bc.saveXML(bc.getFormName().equals(bc.untitledName)?askUserFile(true):
		new File(bc.getFormDirectory(), bc.getFormName()+bc.defaultXMLExt));
		bc.setModified(false);
	    } else if (it.equals(CREATORCONSTANTS.BackgroundBean)) {
		bc.setBackgroundBean(bc.getCurrentWrapper());
		bc.updateMenuBar();
	    } else if (it.equals(CREATORCONSTANTS.Metal)) {
		try {
		    UIManager.setLookAndFeel("javax.swing.plaf.metal.MetalLookAndFeel");
		    SwingUtilities.updateComponentTreeUI(this);
		} catch (Exception e) {
		}
	    } else if (it.equals(CREATORCONSTANTS.Motif)) {
		try {
		    UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");
		    SwingUtilities.updateComponentTreeUI(this);
		} catch (Exception e) {
		}
	    } else if (it.equals(CREATORCONSTANTS.Windows)) {
		try {
		    UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
		    SwingUtilities.updateComponentTreeUI(this);
		} catch (Exception e) {
		}
	    } else if (it.equals(CREATORCONSTANTS.DISABLEDESIGNMODE)) {
		bc.setDesignTime(false);
		bc.repaintCurrentComponent();
		((BuilderMenu)bm).updateViewMenu(Beans.isDesignTime(), bc.getHideInvisibleBeans());
		showTools(false);
	    } else if (it.equals(CREATORCONSTANTS.ENABLEDESIGNMODE)) {
		bc.setDesignTime(true);
		bc.repaintCurrentComponent();
		showTools(true);
		((BuilderMenu)bm).updateViewMenu(Beans.isDesignTime(), bc.getHideInvisibleBeans());
	    } else if (it.equals(CREATORCONSTANTS.HIDEINVISIBLEBEANS)) {
		bc.setHideInvisibleBeans(true);
		((BuilderMenu)bm).updateViewMenu(Beans.isDesignTime(), bc.getHideInvisibleBeans());
	    } else if (it.equals(CREATORCONSTANTS.SHOWINVISIBLEBEANS)) {
		bc.setHideInvisibleBeans(false);
		((BuilderMenu)bm).updateViewMenu(Beans.isDesignTime(), bc.getHideInvisibleBeans());
	    } else if (it.equals(CREATORCONSTANTS.Options)) {
		new ReadyFormFrame("options.xml");
	    } else if (it.equals(CREATORCONSTANTS.SaveComponent)) {
		bc.saveComponentXML(askUserFile(true));
	    } else if (it.equals(CREATORCONSTANTS.Copy)) {
		bc.Command(new FCEvent (this, it, Constants.CMDCopy));
	    } else if (it.equals(CREATORCONSTANTS.Paste)) {
		bc.Command(new FCEvent (this, it, Constants.CMDPaste));
	    } else if (it.equals(CREATORCONSTANTS.Delete)) {
		bc.Command(new FCEvent (this, it, Constants.CMDDelete));
	    } else if (it.equals(CREATORCONSTANTS.BringForward)) {
		bc.Command(new FCEvent (this, it, Constants.CMDBringFwd));
	    } else if (it.equals(CREATORCONSTANTS.BringBackward)) {
		bc.Command(new FCEvent (this, it, Constants.CMDBringBck));
	    } else if (it.startsWith(CREATORCONSTANTS.About)) {
		new AboutDialog(this);
	    }
	} finally {
	    inprocess = false;
	}
    }

    //======== WindowListener ================
    public void windowOpened(WindowEvent e) {
    }
    
    public void windowClosing(WindowEvent e) {
	if (canClose())
	    e.getWindow().dispose();
    }
    
    public void windowClosed(WindowEvent e) {
	System.exit(0);
    }
    
    public void windowIconified(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }
    
    public void windowActivated(WindowEvent e) {
    }
    
    public void windowDeactivated(WindowEvent e) {
    }
    public void setTitle(String formname){
	super.setTitle(DefCaption+formname);
    }
    
    private final static String DefCaption = Constants.PRODUCT_ID+' '+Constants.COMP_NAME+" - ";
    
    private Dimension KeepSize = null;
    private int toolbarh = 20;
    private boolean deatTrig = false;
    private boolean isVis = false;
    private EditToolPanel ep;
    private FreePanel ps;
    private ToolBox tb;
    private BasicPanel bp;
    private BuilderMenu bm;
    private FormPanelHolder fh;
    private Dimension MenuSize;
    private WorkSpace bc;
    private int mode;
    
    private volatile boolean inprocess = false;
}

class XMLFilter extends javax.swing.filechooser.FileFilter {
    final static String xml = "xml";
    
    // Accept all directories and (gif || jpg || tiff) files.
    public boolean accept(File f) {
        if(f.isDirectory()) {
            return true;
        }
        String s = f.getName();
        int i = s.lastIndexOf('.');
        if(i > 0 &&  i < s.length() - 1) {
            String extension = s.substring(i+1).toLowerCase();
            if (xml.equals(extension)) {
                return true;
            } else {
                return false;
            }
        };
        return false;
    }
    
    // The desctiption of this filter
    public String getDescription() {
        return "xBox files (*.xml)";
    }
}
