/*
 * File:		ImageHelper
 *
 * Description:	Utilities for Java images.
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.io;

import java.awt.*;
import java.applet.*;
import java.net.*;

import rogatkin.xbox.shared.Constants;


public class ImageHelper {
	
	private final static String IMAGE_DIR = Constants.Internals.IMAGEPATH;
	private static Applet applet = null;
//	private static URL url = null;
	
/*	public final static void init(com.ms.ui.AwtUIApplet aplt){
		if ( aplt != null ) {
			applet = aplt;
//			url = applet.getCodeBase();
		}
	}*/

	public final static void init(Applet aplt){
		if ( aplt != null ) {
			applet = aplt;
//			url = applet.getCodeBase();
		}
	}
        
        public final static Image getAppImage() {
            return loadImage("rfico.gif");
        }

	/**
	* waitForImage
	*
	* Waits until the whole image is loaded.
	*/
	public static void waitForImage(Image img) {
		MediaTracker tracker = new MediaTracker(new Canvas());
		try {
			tracker.addImage(img, 0);
			tracker.waitForID(0);
		}
		catch (InterruptedException e) {
		}
	}

	/**
	* loadImage
	*
	* Loads an image completely in Application/Applet mode.
	* @param name the name of the image relative to CLASSPATH/IMAGE_DIR.
	*/
	public static Image loadImage(String imageName) {
		return loadImage(imageName, true);
	}

	/**
	* loadImage
	*
	* Loads an image in Application/Applet mode.	
	* @param name the name of the image relative to CLASSPATH/IMAGE_DIR.
	* @parem wait is true if image needed to be loaded completely.
	*/
 	public static Image loadImage(String imageName, boolean wait)
	{
		if (applet != null)
			return loadImage(applet, imageName, wait);
		else {
			URL url=ClassLoader.getSystemResource((imageName.indexOf('/')>=0?"":IMAGE_DIR) + imageName);
			return loadImage(url, wait);
		}
	}

	/**
	* loadImage
	*
	* Loads an image completely in Application mode.
	* @param URL the URL of the image.
	*/
	public static Image loadImage(URL url) {
		return loadImage(url, true);
	}

	/**
	* loadImage
	*
	* Loads an image completely in Application mode.
	* @param URL the URL of the image.
	* @parem wait is true if image needed to be loaded completely.
	*/
	public static Image loadImage(URL url, boolean wait) {
		Image img = null;

		if(url != null) {
			img = Toolkit.getDefaultToolkit().getImage(url);
		
			if(img != null && wait)
				waitForImage(img);
		}

		return img;
	}

	/**
	* loadImage
	*
	* Loads an image completely in Applet mode.
	* @param name the name of the image relative to the applet's code base.
	*/
	public static Image loadImage(Applet app, String name) {
		return loadImage(app, name, true);
   }

	/**
	* loadImage
	*
	* Loads an image in Applet mode.
	* @param name the name of the image relative to the applet's code base.
	* @parem wait is true if image needed to be loaded completely.
	*/
	public static Image loadImage(Applet app, String name, boolean wait) {
		Image img = null;
		
		if(app != null) {
			try {
				img = app.getImage(new URL(app.getCodeBase(),(name.indexOf('/')>=0?"":IMAGE_DIR)), name);
			}
			catch (NullPointerException e) {
			}
			catch (MalformedURLException e) {
			}
		}
		
		if(img != null && wait)
			waitForImage(img);
		return img;
	}
	
	public static URL getImageURL(String name){
		return ClassLoader.getSystemResource(name.indexOf('/')>=0?name:(IMAGE_DIR + name));
	}
}
