/*
 * File:		SimpleResourceConnection.java
 * Author:		Dmitry Rogatkin (originaly based on Sun's source)
 * Date:		03/11/98
 *
 * Description:	
 *				
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.io.simpleresource;

import java.net.*;
import java.io.*;
import rogatkin.xbox.io.*;

public class SimpleResourceConnection extends URLConnection {
    private static boolean debug = false;

    private Object resource;	// the resource we are fetching
    private String cookie;	// identification of the loader instance to use
    private String name;	// name of the resource
    private final String prefix = SimpleClassLoader.urlPrefix;
    private final int prefixLength = prefix.length();

    protected SimpleResourceConnection (URL url)
		throws MalformedURLException, IOException
    {
	super(url);
	debug("SimpleResourceConnection("+url+")");
	String file = url.getFile();
	if (file.startsWith("/")) {
	    file = file.substring(1);
	}
	if (! file.startsWith(prefix)) {
	    throw new MalformedURLException("SimpleResource file should start with /SIMPLE");
	}
	cookie = file.substring(prefixLength, file.indexOf("/+/"));
	name = file.substring(file.indexOf("/+/")+3);

	debug(" cookie: "+cookie);
	debug(" name: "+name);
    }

    public void connect() throws IOException {
	debug("Looking for "+cookie+", "+name+" in SimpleResourceLoader");
	Object o = SimpleClassLoader.getLocalResource(cookie, name);
	if (o == null) {
	    debug("Invalid resource name");
	    resource = null;
	    return;
	} else {
	    debug("Found resource "+o);
	    resource = o;
	}
    }

    public Object getContent() throws IOException {
	if (!connected) {
	    connect();
	}
	return resource;
    }

    public InputStream getInputStream() throws IOException {
	if (!connected) {
	    connect();
	}

	if (resource instanceof InputStream) {
	    return (InputStream) resource;
	}
	return SimpleClassLoader.getLocalResourceAsStream(cookie, name);
    }


    private void debug(String msg) {
	if (debug) {
	    System.err.println(msg);
	}
    }
}
