/*
 * File:		AlbumEditor.java
 * @Author:		Dmitry Rogatkin
 * @Date:		12/28/99
 *
 * Description:	main module of FormConstructor project
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.photoorganizer;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Rectangle;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Properties;
import java.beans.Beans;
import java.lang.reflect.Method;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import rogatkin.xbox.core.*;
import rogatkin.xbox.creator.*;
import rogatkin.xbox.ui.*;
import rogatkin.xbox.shared.*;
import rogatkin.*;
import photoorganizer.PhotoPlugin;

public class AlbumEditor extends JSplitPane implements PhotoPlugin, ActionListener {
    static final String CMD_SAVE_XML = "Save in XML";
    static final String CMD_LOAD_XML = "Load in XML";
    static final String CMD_PUBLISH_SVG = "Publish in SVG";
    static final String CMD_EDT_BRINGUP = "Bean forward";
    static final String CMD_EDT_BRINGDN = "Bean backward";
    
    // some string constants
    static final String MENU_EDIT = "Edit";

    private static BaseController controller;
    private JBasicPanel bp;
    private WorkSpace ws;
    private FreePanel ps;
    private ToolBox tb;
    private EditToolPanel ep;
    private CompManager cm;
    private String jarLocation;
    
    private String lastDirectory;
    
    public static BaseController getController() {
            return controller;
    }

    public AlbumEditor(BaseController controller) {
        this.controller = controller;
        bp = new JBasicPanel();
        bp.setMinimumSize(new Dimension(10, 60));
	Properties p = new Properties();
	try {
	    p.load(new FileInputStream(PROP_FILE));
	    jarLocation = p.getProperty(JAR_LOC);
	} catch(IOException ioe) {
	}
	cm = new CompManager();
	if (jarLocation != null)
	    cm.addBeansInJar(jarLocation);
        bp.init(cm);
	makeLayout();
    }
    
    void makeLayout() {
        ws = bp.getWorkSpace();
        ws.setFrame(controller.getMainFrame());
        ws.setDesignTime(true);
        tb = new ToolBox(ws);
        ps = new PropertySheet(ws, null);
        ws.setToolBox(tb);
	ep = new EditToolPanel(ws);
	statusBar = new StatusPanel();
        optionsTab = new OptionsTab();
        ws.setStatusInfo(new IStatusInfo() {
            public void updateCursorCoord(int x, int y) {
                statusBar.updateCoord(x, y);
            }

            public void updateRectangleCoord(Rectangle box) {
                statusBar.updateRectangle(box);
            }
        });
        setLeftComponent(new JSplitPane(JSplitPane.VERTICAL_SPLIT, 
		new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, tb, ep), ps));
        setRightComponent(bp.getGui());
    }

    public JPanel getOptionsTab() {
	return optionsTab; // no options tab for a while
    }

    public JComponent getStatusBar() {
	return statusBar;
    }

    public boolean addMenuElements(JMenu menu) {
        // lack in the current interface, there is no possibility
        // to add top level menu, and no edit menu also
        boolean result = false;
        String cmd = menu.getActionCommand()/*getText()*/;
        JMenuItem item;
        if (MENU_FILE.equals(cmd)) {
            menu.add(item = new JMenuItem(CMD_SAVE_XML));
            item.addActionListener(this);
            menu.add(item = new JMenuItem(CMD_LOAD_XML));
            item.addActionListener(this);
            result = true;
        } else if (MENU_TOOLS.equals(cmd)) {
            menu.add(item = new JMenuItem(CMD_PUBLISH_SVG));
            item.addActionListener(this);
            menu.add(menu = new JMenu(MENU_EDIT));
            menu.add(item = new JMenuItem(CMD_EDT_BRINGUP));
            item.addActionListener(this);
            menu.add(item = new JMenuItem(CMD_EDT_BRINGDN));
            item.addActionListener(this);
            result = true;
        }
        return result;
    }

    public String getName() {
	return "Editor";
    }
    
    public void actionPerformed(ActionEvent e) {
        String cmd = e.getActionCommand();
        if (CMD_SAVE_XML.equals(cmd)) {
            JFileChooser fc = new JFileChooser(lastDirectory);
            fc.setFileFilter(new ExtFilter(new String[] {"XML"}, "XML files (*.xml)"));
            if(fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
                File f = fc.getSelectedFile();
                String name = f.getName();
                if (name.indexOf('.') > 0)
                    ws.saveXML(f);
                else
                    ws.saveXML(new File(f.getParent(), f.getName()+".xml"));
                lastDirectory = f.getParent();
                ws.setModified(false);
            }
        } else if (CMD_PUBLISH_SVG.equals(cmd)) {
            JFileChooser fc = new JFileChooser(lastDirectory);
            fc.setFileFilter(new ExtFilter(new String[] {"XML", "SVG"}, "SVG files (*.svg;*.xml)"));
            if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
                File f = fc.getSelectedFile();
                String name = f.getName();
                FileWriter fw;
                try {
                    if (name.indexOf('.') > 0)
                        fw = new FileWriter(f);
                    else
                        fw = new FileWriter(new File(f.getParent(), f.getName()+".svg"));
                    lastDirectory = f.getParent();
                    writeSVGHeader(fw);
                    Dimension d = getSize();
                    fw.write("<svg xml:space=\"preserve\" width=\"");
                    fw.write(""+d.width);
                    fw.write("\" height=\"");
                    fw.write(""+d.height); fw.write("\">\n");
                    for (int i = bp.getComponentCount()-1; i >= 0; i--) {
                        try {
                            Object bean = ((Wrapper)bp.getComponent(i)).getBean();
                            Class cl = bean.getClass();
                            Method m = cl.getMethod("getSVGString", new Class[0]);
                            fw.write((String)m.invoke(bean, new Object[0]));
                            fw.write("\n");
                        } catch(Throwable t) {
                            // just skip if any exception happened
                            t.printStackTrace();
                        }
                    }
                    fw.write("</svg>");
                    fw.close();
                } catch(IOException ioe) {
                    ioe.printStackTrace();
                }
            }
        } else if (CMD_LOAD_XML.equals(cmd)) {
            JFileChooser fc = new JFileChooser(lastDirectory);
            fc.setFileFilter(new ExtFilter(new String[] {"XML"}, "XML files (*.xml)"));
            if(fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                ws.loadXML(fc.getSelectedFile());
                lastDirectory = fc.getSelectedFile().getParent();
                ws.setModified(false);
            }
        } else if (CMD_EDT_BRINGUP.equals(cmd)) {
            ws.Command(new FCEvent (this, cmd, Constants.CMDBringFwd));
        } else if (CMD_EDT_BRINGDN.equals(cmd)) {
            ws.Command(new FCEvent (this, cmd, Constants.CMDBringBck));
        }
    }
    
    void writeSVGHeader(FileWriter fw) throws IOException {
        fw.write("<?xml version=\"1.0\" standalone=\"no\"?>\n"+
                 //"<!DOCTYPE svg SYSTEM \"svg-19990812.dtd\">\n"+
                 "<!--\n"+
                 "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG August 1999//EN\"\n"+
                 "\"http://www.w3.org/Graphics/SVG/svg-19990812.dtd\">\n"+
                 "-->\n");
    }
    
    StatusPanel statusBar;
    JPanel  optionsTab;

    static final String JAR_LOC = "JarLocation";
    static final String PROP_FILE = "AlbumEditor.properties";
    class OptionsTab extends JPanel implements Persistable {
	
	OptionsTab() {
	    setLayout(new GranulatedLayout(8, 3, 22, 4,	10, 4, 2));
	    add(new JLabel("Jar file location"), "0,1,6");
	    add(tf_location = new JTextField(), "0,2,5");
	    JButton b;
	    add(b = new JButton("Browse"), "5,2,2");
	    b.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent a) {
		    JFileChooser chooser = new JFileChooser();
                    chooser.setFileFilter(new ExtFilter(new String[] {"JAR", "ZIP"}, "JAR files"));
		    int returnVal = chooser.showOpenDialog(OptionsTab.this);
		    if(returnVal == JFileChooser.APPROVE_OPTION) {
			    tf_location.setText(chooser.getSelectedFile().getAbsolutePath());
		    }
		}
	    });
	}
	
	public void save() {
	    jarLocation = tf_location.getText();
	    if (jarLocation.length() > 0) {
		Properties p = new Properties();
		p.put/*setProperty*/(JAR_LOC, jarLocation);
		try {
		    p.save/*store*/(new FileOutputStream(PROP_FILE), "Album beans location");
		} catch (IOException oie) {
		}
		cm.addBeansInJar(jarLocation);
	    }
	}
	
	public void load() {
	    if (jarLocation != null)
		tf_location.setText(jarLocation);
	}
	
	JTextField tf_location;
    }

}

class StatusPanel extends JPanel {
    JLabel mousecoord;
    JLabel rectanglecoord;
    public StatusPanel() {
        setLayout(new FlowLayout(FlowLayout.CENTER, 18, 0));
        add(rectanglecoord = new JLabel("", SwingConstants.CENTER));
        add(mousecoord = new JLabel("", SwingConstants.CENTER));
        mousecoord.setBorder(BorderFactory.createLoweredBevelBorder());
        rectanglecoord.setBorder(BorderFactory.createLoweredBevelBorder());
        updateCoord(0, 0);
        updateRectangle(new Rectangle(0, 0, 0, 0));
    }
    
    void updateCoord(int x, int y) {
        mousecoord.setText(""+x+":"+y);
    }

    void updateRectangle(Rectangle r) {
        rectanglecoord.setText("("+r.x+":"+r.y+")-("+r.width+":"+r.height+")");
    }
}

class ExtFilter extends FileFilter {
    String[] exts;
    String desc;
    ExtFilter(String []exts, String desc) {
        // only one filter is supported
        this.exts = exts;
        this.desc = desc;
    }
    public boolean accept(File f) {
        String name = f.getName();
        int pe = name.lastIndexOf('.');
        if (pe > 0) {
            for (int i = 0; i < exts.length; i++) {
                if (name.regionMatches(true, pe+1, exts[i], 0, 3))
                    return true;
            }
        }
        return f.isDirectory();
    }
    
    public String getDescription() {
        return desc;
    }
}