/*
 * File:	StartFrame.java
 * Author:	Dmitry Rogatkin
 * Date:	03/12/98
 *
 * Description:	StartFrame
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.ui;

import java.awt.*;
import java.io.*;

import javax.swing.*;
import javax.swing.border.BevelBorder;

import rogatkin.xbox.io.*;
import rogatkin.xbox.shared.Constants;

/**
 * Show a welcome message while xBox application is starting.
 */
public class StartFrame extends JWindow implements Starter {

    private static int width = 358;
	private static int height = 206;
	private JProgressBar pi = new  JProgressBar();
	private JLabel l = new JLabel("Starting...", SwingConstants.CENTER);
	private JLabel componentName = new JLabel(Constants.PRODUCTID+" "+rogatkin.xbox.core.BasicSpace.VERSIONID, SwingConstants.RIGHT);
	private JLabel copyright = new JLabel(Constants.COPYRIGHT, SwingConstants.RIGHT);
	private JLabel companyName = new JLabel(Constants.COMPANY_NAME, SwingConstants.RIGHT);
	private JLabel graf;
	private JLabel caption = new JLabel();
	private static int leftMarg = 18;

	public StartFrame(String caption, boolean splashScreen) {
	    this.caption.setText(caption);
	    setBackground(Color.white);
	    getContentPane().setLayout(null);
	    getRootPane().setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
	    
	    // Display the splash screen if requested
	    if (splashScreen) {
		l.setBounds(0, 135, width, 20);
		getContentPane().add(l);
		pi.setMinimum(0);
		pi.setBounds(8, 160, width-2*leftMarg, 16);
		
		getContentPane().add(pi);
		
		Font f = new Font("Arial", Font.BOLD, 16);
		componentName.setFont(f);
		componentName.setForeground(Color.red);
		getContentPane().add(componentName);
		
		Font f2 = new Font("Arial", Font.ITALIC, 12);
		copyright.setFont(f2);
		copyright.setForeground(Color.black);
		getContentPane().add(copyright);
		
		companyName.setFont(f2);
		companyName.setForeground(Color.black);
		getContentPane().add(companyName);
		try {
		    graf = new JLabel(new ImageIcon(ImageHelper.getImageURL("rflogo.gif")));
		} catch(NullPointerException e) {
		    graf = new JLabel("No image");
		}
		
		getContentPane().add(graf);
		getContentPane().add(this.caption);
		Dimension ss = Toolkit.getDefaultToolkit().getScreenSize();
		
		pack();
		setBounds((ss.width-width)/2, (ss.height-height)/2, width, height);
		show();
		
		Insets is = getInsets();
		graf.setBounds(is.left+leftMarg, is.top+4, 100, 100);
		componentName.setBounds(is.left+leftMarg+116, is.top+10, 200, 30);
		copyright.setBounds(is.left+leftMarg+116, is.top+50, 200, 30);
		companyName.setBounds(is.left+leftMarg+116, is.top+65, 200, 40);
		this.caption.setBounds(is.left+leftMarg, height-is.bottom-28, 160, 20);
	    }
	}
	
	/*
	protected Image loadImage(String imageName)
	{
	ClassLoader cl=this.getClass().getClassLoader();
	
	  InputStream is;
	  if(cl!=null)
	  is=cl.getResourceAsStream("images/"+imageName);
	  else
	  is=ClassLoader.getSystemResourceAsStream("images/"+imageName);
	  if(is!=null)
	  return Toolkit.getDefaultToolkit().createImage(new StreamedImageSource(is));
	  return null;
	  }
	*/
	final static String titlemessage = "xBox is starting...";

	public StartFrame()
	{
	    // The default is to display the splash screen
	    this(titlemessage, true);
	}
	
	public StartFrame(boolean splashScreen)
	{
	    this(titlemessage, splashScreen);
	}
	// interface Starter 
	public int getRange() {
	    return pi.getMaximum();
	}
	
	public void setRange(int r)
	{
	    pi.setMaximum(r);
	}
	
	public int getPos() {
	    return pi.getValue();
	}
	
	public void setPos(int p)
	{
	    pi.setValue(p);
	}
	
	public void setDescription(String text){
	    l.setText(text);
	}
	
	public void setComponentName(String cn){
	    componentName.setText(cn);
	}
}
