/*
 * File:	ToolBtnPalette.java
 * Author:	Dmitry Rogatkin
 * Date:	03/12/98
 *
 * Description:	ToolBtnPalette
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.ui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

// Swing
import javax.swing.*;

import rogatkin.xbox.io.*;
import rogatkin.xbox.shared.*;

public class ToolBtnPalette extends FreePanel implements ActionListener {
        static final String IMAGE_DIR = "toolbarButtonGraphics/development/";
	public ToolBtnPalette(int cols){
		Selection = -1;
		numCols = cols;
		setBorder(BorderFactory.createRaisedBevelBorder());
		setLayout(new GridLayout(0, numCols));
	}
	
	public void init(){
		if (MenuItems == null)
			return;
		for (int i = 0; i< MenuItems.size(); i++) {
			ToolButton tb = new ToolButton((String)MenuItems.elementAt(i),
			(MenuImages == null)?null:(Image)MenuImages.elementAt(i),
			(MenuTooltips == null)?null:(String)MenuTooltips.elementAt(i), i);
			tb.addActionListener(this);
			add(tb);
		}
	}
	
	public Dimension getMinimumSize() {
		return new Dimension(numCols*ButtonSize+numCols*2, ButtonSize*2+4);
	}
	
	public Dimension getPreferredSize() {
		if (MenuItems == null || MenuItems.size() == 0)
			return getMinimumSize();
		else
		return new Dimension(numCols*ButtonSize+numCols*2,
			(ButtonSize+2)*((MenuItems.size()+numCols)/numCols));
	}
	
	public Vector getMenuItems() {
		return MenuItems;
	}
	
	public void setMenuItems(Vector v) {
		MenuItems = v;
	}
	
	public Vector getMenuImages() {
		return MenuImages;
	}
	
	public void setMenuImages(Vector v) {
		MenuImages = v;
	}

	public Vector getMenuTooltips() {
		return MenuTooltips;
	}

	public void setMenuTooltips(Vector v) {
		MenuTooltips = v;
	}

	public void setSelection(int s) {
		// unselect if something selected
		if (Selection >= 0)
			((ToolButton)getComponent(Selection)).setSelected(false);
		Selection = s;
		if (Selection >= 0)
			((ToolButton)getComponent(Selection)).setSelected(true);
	}

	public synchronized void addItemSelectedListener(ItemSelectedListener l)
	{
		ItemSelectedListeners.addElement(l);
	}
	
	public synchronized void removeItemSelectedListener(ItemSelectedListener l)
	{
		ItemSelectedListeners.removeElement(l);
	}
	
	public void actionPerformed(ActionEvent e){
		FireItemSelected(((ToolButton)e.getSource()).getIndex());
	}

	protected void FireItemSelected(int itemNumber)
	{
		Vector l;
		ItemSelectedEvent s = new ItemSelectedEvent (this, itemNumber);
		synchronized (this)
		{
			l = (Vector) ItemSelectedListeners.clone();
		}
		for (int c=0;c<l.size();c++)
		{
			ItemSelectedListener isl = (ItemSelectedListener)l.elementAt(c);
			isl.selected(s);
		}
	}

	private Vector ItemSelectedListeners = new Vector();
	private Vector MenuItems = null;
	private Vector MenuImages = null;
	private Vector MenuTooltips = null;
	private int Selection;

	private int numCols;
	public final static int ButtonSize = 20;

}

class ToolButton extends JToggleButton {
	private int index;
	public ToolButton(String name, Image image, String tooltip, int i) {
		super(new ImageIcon((image == null)?ImageHelper.loadImage(ToolBtnPalette.IMAGE_DIR+"Bean16.gif", false):image));
		index = i;
		if (tooltip == null)
			setToolTipText(name);
		else
			setToolTipText(tooltip);
		setPreferredSize(new Dimension(ToolBtnPalette.ButtonSize, ToolBtnPalette.ButtonSize));
	}
	
	public int getIndex(){
		return index;
	}

/*
	protected Image loadImage(String imageName)
	{
		ClassLoader cl=this.getClass().getClassLoader();

		InputStream is;
		if(cl!=null)
			is=cl.getResourceAsStream("Images\\"+imageName);
		else
			is=ClassLoader.getSystemResourceAsStream("Images\\"+imageName);
		if(is!=null)
			return Toolkit.getDefaultToolkit().createImage(new StreamedImageSource(is));
		return null;
	}
*/
}