/*
 * File:		RangeredMap.java
 * Author:		Dmitry Rogatkin
 * Date:		11/18/98
 *
 * Description:	RangeredMap
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.ut;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.NoSuchElementException;

public class RangeredMap extends Hashtable {
    private Object[] orders = new Object[0];
    
    synchronized public Object put(Object key, Object value) {
	
	Object result = super.put(key, value);
	if (size() > orders.length)
	{
	    Object [] temp = new Object[size()];
	    System.arraycopy(orders, 0, temp, 0, orders.length);
	    temp[orders.length] = key;
	    orders = temp;
	}
	return result;
    }

    public Object put(Object key, int pos, Object value) {
	if (pos > orders.length)
	    return value;
	Object result = put(key, value);
	swap(pos, orders.length);
	return result;
    }

    synchronized public Object remove(Object key) {
	Object result = super.remove(key);
	if (result != null)
	{
	    Object []temp = new Object[size()];
	    int pos2 = 0;
	    for (int pos = 0; pos < orders.length; pos++)
	    {
		if (!orders[pos].equals(key))
		    temp[pos2++] = orders[pos];
	    }
	    orders = temp;
	}
	return result;
    }

    public void swap(int i1, int i2) {
	if (i1 >= orders.length || i1 < 0 || i2 >= orders.length || i2 < 0 || i1 == i2)
	    return;
        synchronized(orders) {
	   Object t = orders[i1];
	   orders[i1] = orders[i2];
	   orders[i2] = t;
        }
    }

    public Object get(int pos) {
	if (pos >= orders.length || pos < 0)
	    return null;
	return get(orders[pos]);
    }

    public Enumeration order() {
	return new Orders();
    }

    class Orders implements Enumeration {
	private int pos;
	public boolean hasMoreElements() {
	    return pos < orders.length-1;
	}

	public Object nextElement() {
	    if (pos >= orders.length)
		throw new NoSuchElementException ("Attemp to reach "+pos);
	    return get(orders[pos++]);
	}
    }
}