/*
 * File:	XMLToolFactory.java
 * Author:	Dmitry Rogatkin
 * Date:	09/28/98
 *
 * Description:	XMLToolFactory implementation 
 *
 * Copyright (C) 1998-2000 Dmitry Rogatkin.  All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Visit the Dmitry Rogatkin's Java page for up-to-date versions of this and other
 * fine Java utilities: http://drogatkin.openestate.net/
 */
package rogatkin.xbox.ut;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import org.w3c.dom.Document;
import org.xml.sax.Parser;

public class XMLToolFactory {
    public final static String DOM_DOCUMENT_PROP = "org.w3c.dom.document";

    static public Document createDocument(String parm_impl) {
	try {
             return (Document) Class.forName(parm_impl).newInstance();
	}
	catch (java.lang.Exception e)
	{
	    System.err.println(""+e);
	}
        return null;
    }

    static public void saveDocument(Document document, OutputStream out) throws IOException {
        boolean success = false;
        String className = System.getProperty(DOM_DOCUMENT_PROP);
        if (className == null)
            throw new IOException("Property "+DOM_DOCUMENT_PROP+" has not been defined.");
	try {
            Class docClass = Class.forName(className);
            if (document.getClass() == docClass) {
                docClass.getMethod("write", new Class[] {OutputStream.class}).invoke(document, new Object[] {out});
                success = true;
            }
	} catch(java.lang.Throwable e)
	{
            e.printStackTrace();
	}
	if (success)
	    return;
        try {
                document.getClass().getMethod("printWithFormat", new Class[] {OutputStreamWriter.class}).invoke(document, new Object[] {new OutputStreamWriter(out)});
                success = true;
	} catch(java.lang.Throwable e)
	{
            e.printStackTrace();
	}
    }

    static public Parser createParser(String parm_impl) {
	try {
	    return (Parser) Class.forName(parm_impl).newInstance();
	}
	catch (java.lang.Exception e)
	{
	}
        return null;
    }
}